/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup additional_data_group
    \brief Structure storing additional information on each tree pair.
*/

#include <string>
#include "TypeDefinitions.hpp"

#include <boost/serialization/vector.hpp>
#include <boost/serialization/base_object.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>


#ifndef _HPP_ADDITIONAL_INFOS_
#define _HPP_ADDITIONAL_INFOS_


// Really belongs to DataGenerationFromXML, but moved here to avoid circular include
struct OtherInfosContainer
    {

    
    public:
    
    const static size_t nbOfOtherPhraseElems=2+16;
    const static size_t nbOfOtherWordElems=4;
    const static size_t nbOfOtherTextElems=1;
    const static size_t nbOthers=nbOfOtherPhraseElems+nbOfOtherWordElems+nbOfOtherTextElems;
    const static char* const otherElemNameList[];
    const static size_t stringSizes[];
    
    const static size_t nbGroups=2;
    //const static size_t nbGroups=1;
    const static size_t sizeGroups[];
    const static size_t maxSizeOfAGroup=nbOthers;
    const static size_t groups[nbGroups][maxSizeOfAGroup];
    const static bool useGroups=true;
    
    static const char* const* otherWordNameList()
        {
        return &otherElemNameList[nbOfOtherPhraseElems];
        }

    static const char * const* otherPhraseNameList()
        {
        return otherElemNameList;
        }
   
    static const char * const* otherTextNameList()
        {
        return &otherElemNameList[nbOfOtherPhraseElems+nbOfOtherWordElems];
        }
   
    std::vector< std::vector<std::string> > otherInfosVect;
    OtherInfosContainer():otherInfosVect(nbOthers)
        {}
        
    void pushOtherWord(size_t i,const std::string& elem)
        {
        assert(i<nbOfOtherWordElems);// && i>=nbOfOtherPhraseElems);
        otherInfosVect[i+nbOfOtherPhraseElems].push_back(elem);
        }        
        
    void pushOtherPhrase(size_t i,const std::string& elem)
        {
        assert(i<nbOfOtherPhraseElems);// && i>=nbOfOtherPhraseElems);
        otherInfosVect[i].push_back(elem);
        } 

    void pushOtherText(size_t i,const std::string& elem)
        {
        assert(i<nbOfOtherTextElems);// && i>=nbOfOtherPhraseElems);
        otherInfosVect[i+nbOfOtherPhraseElems+nbOfOtherWordElems].push_back(elem);
        } 

    private:
    friend class boost::serialization::access;
    template<class Archive> void serialize(Archive & ar, const unsigned int version)
        {
            ar & otherInfosVect;
        }
        
    };

//Warning: when changing something here, pay attention to:
//  1- keep the two nbOfxxxElems consistent with the size of each array
//  2- no two identical string in any of the two arrays


#define OIC_LEM_INDEX 18
#define OIC_CAT_INDEX 0
#define OIC_TYPE_INDEX 1
#define OIC_POS_INDEX 19
#define OIC_CONTENTP_INDEX 20
#define OIC_CONTENT_INDEX 22

const char* const OtherInfosContainer::otherElemNameList[OtherInfosContainer::nbOthers]={"cat","type",
"level","voice","tense_none","tense_past","tense_present","tense_future","case","time","adverb","inserted","ka","complex","paren_b","request","question","negation",
"lem","pos","content_p","conj",
"content"};

const size_t OtherInfosContainer::stringSizes[OtherInfosContainer::nbOthers]={
sizeof("cat"),sizeof("type"),

sizeof("level"),sizeof("voice"),sizeof("tense_none"),sizeof("tense_past"),sizeof("tense_present"),sizeof("tense_future"),sizeof("case"),sizeof("time"),sizeof("adverb"),sizeof("inserted"),sizeof("ka"),sizeof("complex"),sizeof("paren_b"),sizeof("request"),sizeof("question"),sizeof("negation"),

sizeof("lem"),sizeof("pos"),sizeof("content_p"),sizeof("conj"),
sizeof("content")};

//const size_t OtherInfosContainer::sizeGroups[]={2,5};
//const size_t OtherInfosContainer::groups[OtherInfosContainer::nbGroups][OtherInfosContainer::maxSizeOfAGroup]={ {2,6},{0,1,3,4,5} };
const size_t OtherInfosContainer::sizeGroups[]={2,21};
const size_t OtherInfosContainer::groups[OtherInfosContainer::nbGroups][OtherInfosContainer::maxSizeOfAGroup]={ {18,22},{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,19,20,21} };

//const size_t OtherInfosContainer::sizeGroups[]={6};
//const size_t OtherInfosContainer::groups[OtherInfosContainer::nbGroups][OtherInfosContainer::maxSizeOfAGroup]={ {0,1,2,3,4,5} };


std::ostream &operator<< (std::ostream &stream,const OtherInfosContainer &oic)
  {
  stream<<"{ "<<OtherInfosContainer::nbOthers<<"| ";
  for( size_t i=0;i<OtherInfosContainer::nbOthers;++i)
    {
    stream<<i<<":"<<OtherInfosContainer::otherElemNameList[i]<<":"<<oic.otherInfosVect[i]<<" ";
    }
  stream<<"}";
  return stream;
  }
  
  


//typedef std::string POSTagType;
//typedef std::string LabelTypeAD;
struct AdditionalInfos
    {
    private:
    friend class boost::serialization::access;
    template<class Archive> void serialize(Archive & ar, const unsigned int version)
        {
            // serialize base class information
            //ar & sourceSentence;
            //ar & sourceTags;
            ar & sourceDepend;
            //ar & targetSentence;
            //ar & targetTags;
            ar & sourceOtherInfos;
            ar & targetOtherInfos;
        }
    
    public:
    std::vector<PosType> sourceDepend;
    OtherInfosContainer sourceOtherInfos;
    OtherInfosContainer targetOtherInfos;

    std::vector<std::string> source_phrase_f_list;
    std::vector<std::string> source_word_f_list;
    std::vector<std::string> target_phrase_f_list;
    std::vector<std::string> target_word_f_list;
/*    
    std::vector<LabelTypeAD> sourceSentence;
    std::vector<POSTagType> sourceTags;
    
    std::vector<LabelTypeAD> targetSentence;
    std::vector<POSTagType> targetTags;
    
    AdditionalInfos(const std::vector<LabelTypeAD>& sourceSentence,
                    const    std::vector<POSTagType>& sourceTags,
                    const    std::vector<PosType>& sourceDepend,
                    const    std::vector<LabelTypeAD>& targetSentence,
                    const    std::vector<POSTagType>& targetTags 
                    ):sourceSentence(sourceSentence),sourceTags(sourceTags),
                        sourceDepend(sourceDepend),targetSentence(targetSentence),
                            targetTags(targetTags)
        {
        
        }
*/  
    AdditionalInfos(
                const    std::vector<PosType>& sourceDepend,
                const    OtherInfosContainer& sourceOIC,
                const    OtherInfosContainer& targetOIC 
                ): sourceDepend(sourceDepend),sourceOtherInfos(sourceOIC),
                        targetOtherInfos(targetOIC)
    {
    
    }

    AdditionalInfos(){}
    };

std::ostream &operator<< (std::ostream &stream,const AdditionalInfos &a)
  {
  stream<<a.sourceDepend<<a.sourceOtherInfos<<a.targetOtherInfos;
  //stream<<"S1:"<<a.sourceSentence<<"  T1:"<<a.sourceTags<<"  D1:"<<a.sourceDepend<<"  S2:"<<a.targetSentence<<"  T2:"<<a.targetTags;
  return stream;
  }

#endif

