/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup data_creation_group
    \brief Create data from XML files.
*/

#include <vector>
#include <map>
#include <fstream>
#include <cassert>
#include <iostream>
#include <string>
#include <boost/foreach.hpp>
//#include <functional>


#include "pugixml/src/pugixml.hpp"

#include "TypeDefinitions.hpp"
#include "StringConversions.hpp"
#include "cTargetInfoList.cpp"
#include "binaryMerge.hpp"
#include "binaryMerge.cpp"

#include "DefaultDataPath.hpp"
#include "AdditionalInfos.hpp"
#include "IndexersAndEncoding.hpp" 

#ifndef _HPP_DATA_GENERATION_
#define _HPP_DATA_GENERATION_


//typedef std::pair< std::vector<PosType>, std::vector<PosType> > MatchLinkType;
typedef LinkWithGaps MatchLinkType;
typedef std::vector<MatchLinkType>  MatchListType;

/*
// Moved to:IndexersAndEncoding.hpp
struct AnalyzedIData
    {
    std::vector<std::string> labelList;
    std::vector<PosType> depend;
    std::vector<std::string> POSList;
    };
*/
class DataCreator
    {
    public:
    virtual void process(const std::string& id,const AnalyzedIData& aidI, const AnalyzedIData& aidJ,
                            const std::pair<MatchListType,MatchListType>& matchListPair)=0;
    };

/*********************************************************************************/




AnalyzedIData analyzeIData(pugi::xml_node idata)//,const char* attributeNameForWord=NULL)
    {
    AnalyzedIData result;
    int numPhrase=0;
    for (pugi::xml_node phrase = idata.first_child(); phrase; phrase = phrase.next_sibling())
        {
        assert(strcmp(phrase.name(),"phrase")==0);
        
        //assert(numPhrase==phrase.attribute("pnum").as_int());
        
        int dependVal=phrase.attribute("dpnd").as_int();
        if (dependVal==-1)
            {
            dependVal=POSTYPE_SPECIAL_VALUE;
            }
        else
            {
            assert(dependVal!= POSTYPE_SPECIAL_VALUE );
            }
        assert(isValidPosTypeValue(dependVal));
        
        //std::string phraseCat=word.attribute("cat").value();
        //std::string phraseType=word.attribute("type").value();
        
        for (size_t i=0;i<OtherInfosContainer::nbOfOtherPhraseElems;++i)
            {
            result.others.pushOtherPhrase(i,phrase.attribute(OtherInfosContainer::otherPhraseNameList()[i]).value());
            }
        
        pugi::xml_node word=phrase.child("word");
        assert(!word.next_sibling());
        
        for (size_t i=0;i<OtherInfosContainer::nbOfOtherWordElems;++i)
            {
            result.others.pushOtherWord(i,word.attribute(OtherInfosContainer::otherWordNameList()[i]).value());
            }
        
        // result.phrase_f_list.push_back( phrase.attribute("f").value() );
        // result.word_f_list.push_back( word.attribute("f").value() );
        // assert(result.phrase_f_list.size()==numPhrase+1);
        // assert(result.word_f_list.size()==numPhrase+1);
        
        std::string wordLem=word.attribute("lem").value();
        //std::string wordPOS=word.attribute("pos").value();
        //std::string wordContent_p=word.attribute("content_p").value();
        //std::string wordConj=word.attribute("conj").value();
        
        std::string wordContentTxt=word.child_value();
        result.others.pushOtherText(0,wordContentTxt);
        assert(OtherInfosContainer::nbOfOtherTextElems>=1);
        

        
        result.labelList.push_back(wordLem);
        result.depend.push_back(dependVal);
        //result.POSList.push_back(wordPOS);
        
        numPhrase++;
        }
    return result;
    
    }

std::vector<PosType> convertStringToPosList(const std::string& str)
    {
    std::vector<PosType> res;
    std::vector<int> intVect=convertToVectorInt(str);
    BOOST_FOREACH(int i,intVect)
        {
        assert(isValidNonSpecialPosTypeValue(i));
        res.push_back(i);
        }
    return res;
    }

std::pair< std::vector<PosType>, std::vector<PosType> > getLinkFromXmlNode(pugi::xml_node match)
    {
    std::string i_p_string= match.attribute("i_p").value();
    std::string j_p_string= match.attribute("j_p").value(); 
    return std::make_pair( convertStringToPosList(i_p_string),convertStringToPosList(j_p_string) );
    }

std::pair< std::vector<PosType>, std::vector<PosType> > getAddLinkFromXmlNode(pugi::xml_node match)
    {
    std::string add_i_p_string= match.attribute("i_p_add").value();
    std::string add_j_p_string= match.attribute("j_p_add").value(); 
    return std::make_pair( convertStringToPosList(add_i_p_string),convertStringToPosList(add_j_p_string) );
    }

MatchLinkType getLinkWithGapFromXML(pugi::xml_node match)
    {
    MatchLinkType link;
    std::pair< std::vector<PosType>, std::vector<PosType> > linkBase=getLinkFromXmlNode(match );
    std::pair< std::vector<PosType>, std::vector<PosType> > linkAdd=getAddLinkFromXmlNode(match );
    link.first=linkBase.first;
    link.second=linkBase.second;
    link.add_i_side=linkAdd.first;
    link.add_j_side=linkAdd.second;
    return link;
    }
    
std::pair<MatchListType,MatchListType> getMatchList(pugi::xml_node doc)
    {
    MatchListType match_list;
    MatchListType word_match_list;
    for (pugi::xml_node match = doc.child("match"); match; match = match.next_sibling("match"))
        {
        pugi::xml_node word=match.child("word");
        assert(!word.next_sibling());
        

        match_list.push_back(getLinkWithGapFromXML(match) );
        word_match_list.push_back(getLinkWithGapFromXML(word ) );
        }
    return std::make_pair(match_list,word_match_list);
    }  



void processPara(const std::string& para,DataCreator &dc)
    {
    //need to add header?
    std::string xmlheader("<?xml version='1.0' encoding='utf-8'?>\n");
    //std::string para=xmlheader+para0;
    
    pugi::xml_document doc;
    //const char* paraAsCharArray=para.c_str();
    //std::cout<<"para:"<<std::endl;
    //std::cout<<para<<std::endl;
    pugi::xml_parse_result result = doc.load_buffer(para.c_str(),para.size(),pugi::parse_default,pugi::encoding_utf8);
    //std::cout<<"loaded para"<<std::endl;
    //std::cout<<result.description()<<std::endl;
    assert(result);
    
    pugi::xml_node paraNode=doc.child("para_sentence");
    
    //doc.root().print(std::cout);
    //assert();
    pugi::xml_node isent = paraNode.child("i_sentence");
    pugi::xml_node jsent = paraNode.child("j_sentence");
    
    pugi::xml_node idata = paraNode.child("i_data");
    pugi::xml_node jdata = paraNode.child("j_data");
    assert(idata && jdata);
    //std::cout<<idata<<std::endl;
    //exit(1);
    //pugi::xml_node match_list = paraNode.child("match_list");
    std::string id=paraNode.attribute("id").value();
        
    AnalyzedIData aidI = analyzeIData(idata);
    AnalyzedIData aidJ = analyzeIData(jdata);
    
    std::pair<MatchListType,MatchListType> matchListPair=getMatchList(paraNode);
    
    dc.process(id,aidI,aidJ,matchListPair);
    
    }
  

bool processFileByPara(const char *filename,DataCreator &dc)
    {
    std::string paraStartTag("<para_sentence");
    std::string paraEndTag("</para_sentence>");
    
    std::ifstream ifs(filename);
    if (ifs.fail())
        {
        std::cerr<<"Error opening "<<filename<<std::endl;
        return false;
        }
    std::string line;
    std::string para;
    para.reserve(10000);
    bool inpara=false;
    std::string before,after;
    //std::cout<<"before while"<<std::endl;
    //std::cout<<ifs;
    while( std::getline(ifs,line) )
        {
        line+="\n";
        //std::cout<<"in while"<<std::endl;
        size_t pos_para_start=line.find(paraStartTag);
        size_t pos_para_end=line.find(paraEndTag);
        
        if (pos_para_start!=std::string::npos && pos_para_end!=std::string::npos)
            {
            throw std::exception();
            }
        if (pos_para_end!=std::string::npos)
            {
            assert(inpara);
            before=line.substr(0,pos_para_end);
            //after=line.substr(pos_para_end+paraEndTag.length());
            para+=before;
            para+=paraEndTag;
            //call processing of para
            //std::cout<<"Processing para:"<<std::endl;
            //std::cout<<para<<std::endl;
            processPara(para,dc);
            
            //line=after; //useless given that we enforce no para_start in line?
            para.clear();
            inpara=false;
            }
        else if (pos_para_start!=std::string::npos)
            {
            assert(!inpara && para.length()==0);
            //before=line.substr(0,pos_para_end);
            inpara=true;
            after=line.substr(pos_para_start);
            para+=after;
            }
        else if (inpara)
            {
            para+=line;
            }
        
        }
    assert(!inpara && para.length()==0);
    return true;
    }
    


/***************************************************************************************************
 *
 */
 

typedef std::vector<std::pair<PosType,PosType> > PosParentPairListType;
typedef std::pair<LabelType,PosParentPairListType> LabelPosParentPairType;
 
void addOccurrenceToInvertedIndex(  std::vector<OccListCompactPair>& invertedIndex,LabelType labelIdx,
                                        TreeIdType numTree,std::vector<std::pair<PosType,PosType> > posParentPairList )
    {
    if (labelIdx>=invertedIndex.size() )
        {
        invertedIndex.resize(labelIdx+1);
        }
    if (invertedIndex[labelIdx].size()>0 && numTree<=invertedIndex[labelIdx].back().numTree)
        {
        throw std::exception(); // insertion should be with increasing numTree
        }
    if (posParentPairList.size()>std::numeric_limits<PosType>::max())
        {
        throw std::exception(); // The ArrayOfPair implementation puts a limit on the number of occurrences we can add
        }
    if (posParentPairList.size()==0)
        {
        return;
        }
    invertedIndex[labelIdx].push_back(MappingsPerTreeCompactPair());
    invertedIndex[labelIdx].back().numTree=numTree;
    invertedIndex[labelIdx].back().mappingList.resize_with_erase( posParentPairList.size() );
    for (size_t i=0;i<posParentPairList.size();++i)
        {
        PosType pos=posParentPairList[i].first;
        PosType parent=posParentPairList[i].second;
        //assert( pos<=std::numeric_limits<PosType>::max() && parent<=std::numeric_limits<PosType>::max());
        invertedIndex[labelIdx].back().mappingList.setPair(i,pos,parent);
        }
    }

std::string numExt(std::string s,std::string& alternateLabel)
  {
  // handle <num:456>F -> <num>F  and 456F
  if (s.size()<=6 || s.compare(0,5,"<num:",0,5)!=0 ||!isdigit(s[6]) )
    {
    //std::cerr<<"no match 1"<<"#"<<s.size()<<"#"<<s.compare(0,5,"<num:")<<"#"<<isdigit(s[6])<<std::endl;
    return "";
    }
  std::string num;
  int cur=5;
  while (isdigit(s[cur]))
    {
    num.push_back(s[cur]);
    cur+=1;
    }
  if (s.compare(cur,1,">",0,1)!=0)
    {
    return "";
    }
  cur+=1;
  std::string suffix=s.substr(cur);
  alternateLabel=num+suffix;
  return "<num>"+suffix;
  }


std::string replaceNum(const std::string& s)
  {
  int cur=0;
  std::string res;
  //bool incompleteNum=false;
  while (cur<s.size())
    {
    //std::cout<<cur<<std::endl;
    if (s.compare(cur,5,"<num:",0,5)==0)
        {
        res+="<num>";
        cur+=5;
        while (s[cur]!='>')
            {
            cur++;
            if (cur>=s.size())
                {
                //incompleteNum=true;
                break;
                }
            }
            cur++;// warning: might push cur past s.size (OK here)
        }
    else
        {
        res.push_back(s[cur]);
        cur++;
        }
    }
  return res;

  }
  
std::string replaceGeneric(const std::string& s)
  {
  int cur=0;
  std::string res;
  //bool incompleteNum=false;
  while (cur<s.size())
    {
    //std::cout<<cur<<std::endl;
    if (s.compare(cur,2,"<@",0,2)==0)
        {
        cur+=2;
        std::string tag;
        while (cur<s.size() && s[cur]!=':')
            {
            tag.push_back(s[cur]);
            cur++;
            }
        if (cur>=s.size())
            {
            break;
            }
        cur++;
        while (cur<s.size() && s[cur]!='>')
            {
            cur++;
            }
        if (cur>=s.size())
            {
            break;
            }
        cur++;
        res+="<"+tag+">";
        }
    else
        {
        res.push_back(s[cur]);
        cur++;
        }
    }
  return res;
  }  
  
void updateInvertedIndexAndIndexer(const AnalyzedIData& aidSource,TreeIdType numTree,
                                                std::vector<OccListCompactPair>& invertedIndex,
                                                LabelIndexer& indexer,std::map<LabelType,size_t>& countOfEachLabel)
    {
    std::map<LabelType,PosParentPairListType> posParentPairListMap;
    for (size_t i=0;i<aidSource.labelList.size();++i)
        {
        assert(isValidNonSpecialPosTypeValue(i));

        PosType pos=i;
        PosType parent=aidSource.depend[i];
        
        std::string labelOrigin=aidSource.labelList[i];
        std::string label=replaceGeneric(labelOrigin);
        LabelType labelIdx=indexer.getIdAndUpdate(label);
        countOfEachLabel[labelIdx]++;
        posParentPairListMap[labelIdx].push_back(std::make_pair(pos,parent) );  
        
        // std::string altLabel;
        // std::string numExtracted=numExt(label,altLabel);
        // if (numExtracted.size()>0)
            // {
 
            // LabelType labelIdx=indexer.getIdAndUpdate(numExtracted);
            // countOfEachLabel[labelIdx]++;
            // posParentPairListMap[labelIdx].push_back(std::make_pair(pos,parent) );
            
            // labelIdx=indexer.getIdAndUpdate(altLabel);
            // countOfEachLabel[labelIdx]++;
            // posParentPairListMap[labelIdx].push_back(std::make_pair(pos,parent) );
            // std::cout<<label<<"-->"<<altLabel<<" | "<<numExtracted<<std::endl;
            // }
        // else
            // {
            // LabelType labelIdx=indexer.getIdAndUpdate(label);
            // countOfEachLabel[labelIdx]++;
            // posParentPairListMap[labelIdx].push_back(std::make_pair(pos,parent) );            
            // }
        

        }
    
    BOOST_FOREACH(const LabelPosParentPairType &mapPair, posParentPairListMap )
        {
        LabelType labelIdx=mapPair.first;
        addOccurrenceToInvertedIndex( invertedIndex,labelIdx,numTree,mapPair.second );
        }    
    
    
    }
  
void updateTargetInfo(TargetList& targetInfo,MatchListType matchList,std::vector<PosType> depend,size_t sourceSize,TreeIdType numTree)
    {
    size_t targetSize=depend.size();
    assert(targetInfo.size()==numTree);
    
    //std::cout<<depend<<std::endl;
    //std::cout<<matchList<<std::endl;
    
    targetInfo.pushElem();
    targetInfo.back().set_gap(matchList,depend,sourceSize);
    //std::cout<<targetInfo.back()<<std::endl;
    }
    



class DataCreatorSimple: public DataCreator
    {
    LabelIndexer indexer;
    std::map<LabelType,size_t> countOfEachLabel;
    std::vector<OccListCompactPair> invertedIndex;
    TargetList targetInfo;
    std::vector<LabelType> mostFrequentLabels;
    std::vector<std::string> treeToId;
    size_t countNbProcessed;
    bool otherSide;
    
    //bool computeAdditionalDatas;
    //std::vector<AdditionalDatas>;
    
    protected:
    virtual void _process(const std::string& id,const AnalyzedIData& aidSource, const AnalyzedIData& aidTarget,
                            const MatchListType& usedMatchList)
        {
        TreeIdType numTree =countNbProcessed;
        size_t sourceSize=aidSource.labelList.size();
        
        updateInvertedIndexAndIndexer(aidSource,numTree,invertedIndex,indexer,countOfEachLabel);
        updateTargetInfo(targetInfo,usedMatchList,aidTarget.depend,sourceSize,numTree);
        treeToId.push_back(id);
        countNbProcessed+=1;
        
        }
    
    public:
    DataCreatorSimple(bool otherSide):countNbProcessed(0),otherSide(otherSide){};
    
    size_t getNbProcessed()
        {
        return countNbProcessed;
        }
    void process(const std::string& id,const AnalyzedIData& aidSource, const AnalyzedIData& aidTarget,
                            const std::pair<MatchListType,MatchListType>& matchListPair)
        {
        //if(countNbProcessed%1000==0)
        //    {
        //    std::cout<<"Sentence pairs processed so far:"<<countNbProcessed <<std::endl;
        //    }
        
        //std::cout<<id<<std::endl;
        //std::cout<<aidSource<<std::endl;
        //std::cout<<aidTarget<<std::endl;
        
        // Using pointers fo easy swapping if we want the other side
        const AnalyzedIData* pSource=&aidSource;
        const AnalyzedIData* pTarget=&aidTarget;
        
        
        MatchListType usedMatchList=matchListPair.first;
        
        if (otherSide)
            {
            BOOST_FOREACH(MatchLinkType& link,usedMatchList)
                {
                link.invertSide();
                //link.first.swap(link.second);
                
                }
            std::swap(pSource,pTarget); 
            }
        
        BOOST_FOREACH(MatchLinkType& link,usedMatchList) {
            BOOST_FOREACH(PosType pos, link.add_i_side) {
                assert(std::set<PosType>(link.first.begin(), link.first.end()).count(pos) == 0);
                link.first.push_back(pos);
            }
            link.add_i_side.resize(0);
        }
        
        _process(id,*pSource,*pTarget,usedMatchList);
        }
    
    void computeMostFrequentLabels()
        {
        mostFrequentLabels.clear();
        std::transform(countOfEachLabel.begin(),countOfEachLabel.end(),
                                std::back_inserter(mostFrequentLabels),
                                boost::bind(&std::map<LabelType,size_t>::value_type::first,_1));
                                //std::select1st<std::map<LabelType,int>::value_type>());
        std::sort(mostFrequentLabels.begin(),mostFrequentLabels.end(),OrdererOfLabelByCount<LabelType>(countOfEachLabel));
        }
    
    virtual void saveAs(std::string filename,std::string side)
        {
        DataFilenameSufixes suffixes(side);
        
        indexer.saveAs((filename+suffixes.indexer).c_str() );   
        
        std::string invertedIndexFilename= filename+suffixes.invertedIndex;     
        saveVectorOccListCompactPair(invertedIndex, invertedIndexFilename.c_str());
        
        targetInfo.saveTo( (filename+suffixes.targetInfo).c_str() );

        saveGeneric(treeToId,(filename+suffixes.treeToId).c_str() );
        
        computeMostFrequentLabels();
        saveGeneric(mostFrequentLabels,(filename+suffixes.mostFrequentLabel).c_str() );

        } 
    };

void updateAdditionalInfos(std::vector<AdditionalInfos>& aDataVect,const AnalyzedIData& aidSource, const AnalyzedIData& aidTarget)
    {
    
    aDataVect.push_back(
        //AdditionalInfos(aidSource.labelList,aidSource.POSList,aidSource.depend,aidTarget.labelList,aidTarget.POSList)
        AdditionalInfos(aidSource.depend,aidTarget.others,aidTarget.others)
        );
    
    }

template< typename AdditionalInfosPackerType>
class DataCreatorWithAdditionalInfos: public DataCreatorSimple
    {
    //std::vector<AdditionalInfos> aDataVect;
    AdditionalInfosPackerType aip;
    protected:
    void _process(const std::string& id,const AnalyzedIData& aidSource, const AnalyzedIData& aidTarget,
                            const MatchListType& matchList)
        {
        DataCreatorSimple::_process(id,aidSource,aidTarget,matchList);
        //updateAdditionalInfos(aDataVect,aidSource,aidTarget);
        aip.packAdditionalInfos(aidSource,aidTarget);
        }    
    public:
    DataCreatorWithAdditionalInfos(bool otherSide):DataCreatorSimple(otherSide){}
    
    void saveAs(std::string datapath,std::string side)
        {
        DataCreatorSimple::saveAs(datapath,side);
        aip.optimizeAndSave(DataFilenameSufixes::getAdditionalDatasFilename(datapath,side).c_str());
        //aip.save(DataFilenameSufixes::getAdditionalDatasFilename(datapath,side).c_str());
        //saveGeneric(aDataVect,DataFilenameSufixes::getAdditionalDatasFilename(datapath,side).c_str());
        }
    };

   
template<typename DataCreatorType>
class DataCreatorBothSide:public DataCreator
    {
    DataCreatorType dc1;
    DataCreatorType dc2;
    public:
    DataCreatorBothSide():dc1(false),dc2(true){}
    
    size_t getNbProcessed()
        {
        size_t nbProcessed=dc1.getNbProcessed();
        assert(nbProcessed==dc2.getNbProcessed());
        return nbProcessed;
        }
    
    void process(const std::string& id,const AnalyzedIData& aidSource, const AnalyzedIData& aidTarget,
                            const std::pair<MatchListType,MatchListType>& matchListPair)
        {
        dc1.process(id,aidSource,aidTarget,matchListPair);
        dc2.process(id,aidSource,aidTarget,matchListPair);
        }
    void saveAs(std::string filename,std::string side1,std::string side2)
        {
        dc1.saveAs(filename,side1);
        dc2.saveAs(filename,side2);
        }
    };

void generateDatas(const std::vector<std::string>& xmlFilenameList,const char * dbPathPrefix)
    {
    //std::cout<<"in generateDatas"<<std::endl;
    //DataCreatorSimple dc(false);
    DataCreatorBothSide<DataCreatorWithAdditionalInfos<AdditionalInfosPacker> > dc;
    //std::cout<<"before proc"<<std::endl;
    size_t numFile=0;
    BOOST_FOREACH(const std::string& xmlFilename,xmlFilenameList)
        {
        numFile++;
        std::cout<<"Processing file "<<xmlFilename<<" ("<<numFile<<"/"<<xmlFilenameList.size()<<")  "<<dc.getNbProcessed()<<" sent. pairs processed"<<std::endl;
        processFileByPara(xmlFilename.c_str(),dc);
        }
    //std::cout<<"after proc"<<std::endl;
    dc.saveAs(dbPathPrefix,"1","2");
    
    }
  
#endif   

