/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup treelet_discovering_group
    \brief The DescendingSuperTreeletManager is mainly in charge of discovering the potential successors of a treelet that has already been computed.
*/
#include <cassert>
#include <vector>
#include <map>
#include <boost/foreach.hpp>

#include "TypeDefinitions.hpp"
#include "Tree.hpp"
#include "Treelets.hpp"
#include <iostream>
#include "TreeletComputationManager.hpp"

#ifndef _HPP_MY_DESCENDING_SUPERTREELETS_
#define _HPP_MY_DESCENDING_SUPERTREELETS_

typedef map< TreeletId,vector<TreeletId> > TreeletIdToTreeletIdListMap; //TODO: Change to unordered_map

/*
class DescendingSuperTreeletsNotFoundException: public std::exception
{
  virtual const char* what() const throw()
  {
    return "Descending SuperTreelet Not Found";
  }
};
*/
    
template<typename TreeletType>
class DescendingSuperTreeletsManagerGeneric
    {
    typedef TreeletComputationManagerGeneric<TreeletType> TreeletComputationManager;
    typedef typename TreeletType::TreeletSizeInfos TreeletSizeInfos;
    typedef typename TreeletType::TreeletStoreByChildrenIndices TreeletStoreByChildrenIndices;
    typedef typename TreeletType::TreeletSimpleIndexer TreeletSimpleIndexer;
    
    TreeletIdToTreeletIdListMap descendingSuperTreelets;
    map<PosType,vector<TreeletId> > nodeToBaseTreeletTable; //posNode->treeletId
    
    
    //map<TreeletId,size_t> sizeOfTreeletsMap;
    TreeletSizeInfos treeletSizeInfos;
    
    std::map<TreeletType,vector<TreeletId> > ascendingSubTreelets;
    
    int maxTopSiblings;
    map<TreeletId,vector<TreeletId> > dominatedBy;
    
    public:
    
    const map<TreeletId,vector<TreeletId> >& getDominatedBy() const
        {
        return dominatedBy;
        }
    
    const TreeletSizeInfos& getTreeletSizeInfos() const
        {
        return treeletSizeInfos;
        }
    
    DescendingSuperTreeletsManagerGeneric(int maxTopSiblings):maxTopSiblings(maxTopSiblings){};
    
    std::vector<TreeletType> computeListOfSuperTreelets(const TreeletType& treelet, bool computeAlsoAscendingSuperTreelet) const;

/*    
    size_t getSizeOfTreelet(TreeletId id) const
        {
            map<TreeletId,size_t> ::const_iterator it=sizeOfTreeletsMap.find(id);
            if (it==sizeOfTreeletsMap.end() )
                {
                throw DescendingSuperTreeletsNotFoundException();
                }
            else
                {
                return it->second;
                }        
        }   
*/        
    void initializeNodeToBaseForThisPosition(PosType positionOfNode)
        {
        nodeToBaseTreeletTable[positionOfNode]=vector<TreeletId>();
        }
        
    void addAscendingSubtreeletRelationship(TreeletId treeletId, TreeletType superTreelet)
        {
        //TODO: check that there cannot be double insertions...
        //std::cout<<"adding AscendingSubtreeletRelationship: "<<treeletId<<" --> "<<superTreelet<<std::endl;
        #ifndef NDEBUG
        BOOST_FOREACH(TreeletId tId,ascendingSubTreelets[superTreelet])
            {
            if (tId==treeletId)
                { 
                cerr<<treeletId<<" already in "<<ascendingSubTreelets[superTreelet]<<"for " <<superTreelet<<endl;
                assert(false);
                }
            }
        
        #endif //NDEBUG
        ascendingSubTreelets[superTreelet].push_back(treeletId);
        }
    
    bool hasInitialRightPart(TreeletType treelet, int curIndex)
        {
        if (curIndex==-1) return false;
        assert(curIndex>=0); 
        TreeletId rightId=treelet.getChildIndex(curIndex);
        PosType positionOfNode=treelet.getPosNodeAtChildNum(curIndex);
        BOOST_FOREACH(TreeletId tId,nodeToBaseTreeletTable[positionOfNode])
            {
            if (tId==rightId) return true;
            }
        return false;
        }
    
    void pushCompletedTreelet(const TreeletType& treelet)
        {
        TreeletId treeletId=treelet.getId();
        
        treeletSizeInfos.addTreelet(treeletId,treelet);
        //assert(sizeOfTreeletsMap.count( treeletId )==0);
        //sizeOfTreeletsMap[treeletId]=treelet.getSize();
        
        
        assert(descendingSuperTreelets.count(treeletId)==0);
        assert(treelet.isElementaryTreelet() || ascendingSubTreelets.count(treelet)==1);
        
        BOOST_FOREACH(TreeletId ascendingSubtreeletId, ascendingSubTreelets[treelet])
            {
            descendingSuperTreelets[ascendingSubtreeletId].push_back(treeletId);
            }
        // To ensure there is an entry for treeletId in descendingSuperTreelets (even if it lead to an empty list)
        descendingSuperTreelets[treeletId];    
        
        //descendingSuperTreelets.insert( make_pair(treeletId ,ascendingSubTreelets[treelet]) );
        
        if (treelet.isDescendingSupertreesRoot())
            {
            nodeToBaseTreeletTable[ treelet.getRootPos() ].push_back(treeletId);
            }
        //std::cout<<"pushCompletedTreelet "<<treelet<<std::endl;
        //std::cout<<"ascendingSubTreelets: "<<ascendingSubTreelets[treelet]<<std::endl;
        }
        
        
    std::map<TreeletId,std::set<TreeletId> > buildPrecLinks(PosType nodePos) const;
    void reduceDescendingSuperTreeletsList(PosType nodePos,const TreeletComputationManager &tcm,int curIndex=-1,map<TreeletId,int> *treeletIdToIndexGeneration=NULL);
    void pruneDescendingSuperTreeletsList(PosType nodePos, TreeletId treeletIndex, 
                                            std::vector<TreeletId> &listOfDominatingTreelets, 
                                            std::map<TreeletId,std::set<TreeletId> >  &precLinks);
    
    vector<TreeletId> getListOfDescendingIds(PosType nodePos) const;
    std::vector<TreeletType> computeListOfSuperTreeletsLatteralStrategy( const TreeletType& treelet, bool computeAlsoAscendingSuperTreelet,int curIndex,
        //const map< vector<TreeletId>,TreeletId > &mapTreeletsToId,
        const TreeletStoreByChildrenIndices &mapTreeletsToId,
        const TreeletSimpleIndexer& treeletIndexer,
        int maxNbGaps) const;
    
    const std::vector< TreeletId> getLeftTreeletDescendents(const TreeletType& treelet, int index,
    //const map< vector<TreeletId>,TreeletId > &mapTreeletsToId
    const TreeletStoreByChildrenIndices &mapTreeletsToId
    ) const;
    
    //vector<const Treelet *> convertListOfTreeletIdToListOfTreelets(vector<TreeletId> idList) const;
    const vector<TreeletId>& getDescendingsuperTreeletIndices(TreeletId childIndex, const TreeletType& treelet, int numChild) const;
    const vector<TreeletId>&  getBaseTreeletTable(PosType posCurLevel) const;
    
    void removeTopElementaryWildcard(PosType nodePos, TreeletId wildcardId);
    void removeWildcardRootedTreeletsFromNodeToBaseTable(PosType nodePos, 
        std::set<TreeletId> list_of_treelets_with_wildcard_roots);
    };

template<typename TreeletType>
const vector<TreeletId>&  DescendingSuperTreeletsManagerGeneric<TreeletType>::getBaseTreeletTable(PosType posNode) const
    {
    map<PosType,vector<TreeletId> >::const_iterator it=nodeToBaseTreeletTable.find(posNode);
    if (it==nodeToBaseTreeletTable.end() )
        {
        throw TreeletBase::DescendingSuperTreeletsNotFoundException();
        }
    else
        {
        return it->second;
        }
    }
    
template<typename TreeletType>
vector<TreeletId> DescendingSuperTreeletsManagerGeneric<TreeletType>::getListOfDescendingIds(PosType nodePos) const
    {
    std::vector<TreeletId> queue;
    std::set<TreeletId> alreadyQueued;
    vector<TreeletId> result;
    
    //result.push_back( TREELET_ID_SPECIAL_VALUE);
    //alreadyQueued.insert(TREELET_ID_SPECIAL_VALUE);
    
    map<PosType,vector<TreeletId> >::const_iterator it=nodeToBaseTreeletTable.find(nodePos);
    const std::vector<TreeletId>* pSuperTreelets;
    if (it==nodeToBaseTreeletTable.end())
        {
        pSuperTreelets=NULL;
        assert(false);
        throw runtime_error("bug");
        }
    else{
        pSuperTreelets=&( it->second );
        }
    BOOST_FOREACH(TreeletId superTreeletIndex, *pSuperTreelets)
        {
        assert(alreadyQueued.count(superTreeletIndex)==0);
        queue.push_back(superTreeletIndex);
        alreadyQueued.insert(superTreeletIndex);
        }
    //alreadyQueued=set([(node,None)])
    
    
    while (queue.size()>0)
        {
        TreeletId treeletIndex=queue.back();
        queue.pop_back();
        
        result.push_back(treeletIndex);
        
        std::map<TreeletId,std::vector<TreeletId> >::const_iterator it= descendingSuperTreelets.find(treeletIndex);
        assert(it!=descendingSuperTreelets.end());
        pSuperTreelets=&(it->second);
        
        BOOST_FOREACH(TreeletId superTreeletIndex, *pSuperTreelets)
            {
            if (alreadyQueued.count(superTreeletIndex)==0)
                {
                queue.push_back(superTreeletIndex);
                alreadyQueued.insert(superTreeletIndex);
                }
            }
        }
    return result;
        
    }
    
template<typename TreeletType>
std::vector<TreeletType> DescendingSuperTreeletsManagerGeneric<TreeletType>::computeListOfSuperTreelets( const TreeletType& treelet, bool computeAlsoAscendingSuperTreelet) const
    {
    vector<TreeletType> result;
    
    bool refuseToExtend=false;
    
    if (maxTopSiblings!=0 && !treelet.safeToExtendTopSiblings(maxTopSiblings))
    //if (maxTopSiblings!=0 && treelet.hasStrictlyMoreThanNTopSiblings(maxTopSiblings))
        {
        refuseToExtend=true;
        }
    size_t nbChildren=treelet.getNbOfChildren();
    for (unsigned int i=0; i<nbChildren; ++i)
        {
        TreeletId childIndex=treelet.getChildIndex(i);
        if (refuseToExtend && childIndex==TREELET_ID_SPECIAL_VALUE)
            {
            continue;
            }
        vector<TreeletId> descendingSuperTreeletsIndices;  //TODO: optimize with reference or pointer?
        if (childIndex==TREELET_ID_SPECIAL_VALUE)
            {
            PosType posNode=treelet.getPosNodeAtChildNum(i); // pRootNodeInQueryTree->get posOfChildren[i];
            map<PosType,vector<TreeletId> >::const_iterator it=nodeToBaseTreeletTable.find(posNode);
            if (it==nodeToBaseTreeletTable.end() )
                {
                throw TreeletBase::DescendingSuperTreeletsNotFoundException();
                }
            else
                {
                descendingSuperTreeletsIndices=it->second;
                }
            }
        else
            {
            TreeletIdToTreeletIdListMap::const_iterator it=descendingSuperTreelets.find(childIndex);
            if (it==descendingSuperTreelets.end() )
                {
                throw TreeletBase::DescendingSuperTreeletsNotFoundException();
                }
            else
                {
                descendingSuperTreeletsIndices=it->second;
                }
            }
        //std::cout<<descendingSuperTreeletsIndices.size()<<std::endl;
        BOOST_FOREACH(TreeletId  descendingSupertreeletIndex, descendingSuperTreeletsIndices)
            {
            //size_t sizeOfNewChild=getSizeOfTreelet(descendingSupertreeletIndex);
            TreeletType superTreelet=treelet.getExtendedDownwardTreelet(i,descendingSupertreeletIndex,treeletSizeInfos);
            superTreelet.setGeneratedAsDescendingSuperTreeletFlag(true);
            result.push_back(superTreelet);
            }
        }
        
    if (computeAlsoAscendingSuperTreelet && treelet.hasUpwardExtension() )  //pRootNodeInQueryTree->hasParent())
        {
        
        TreeletType superTreelet=treelet.getExtendedUpwardTreelet();
        superTreelet.setGeneratedAsDescendingSuperTreeletFlag(false);
        result.push_back(superTreelet);
        
        //size_t nbChildrenOfParent=treelet.pRootNodeInQueryTree->getParent()->getNbChildren();
        //size_t posInParent=treelet.pRootNodeInQueryTree->getPosInParent();
        
        //Treelet extendedUpwardTreelet()
        //vector<TreeletId> childrenIndices(nbChildrenOfParent,TREELET_ID_SPECIAL_VALUE);
        //childrenIndices[posInParent]=treelet.getIndex();
        //size_t sizeSuperTreelet=treelet.length+1
        //Treelet superTreelet(treelet.pRootNodeInQueryTree,childrenIndices,sizeSuperTreelet);
        //result.push_back(superTreelet);
        }
        
    return result;
    }

template<typename TreeletType>
const std::vector< TreeletId> DescendingSuperTreeletsManagerGeneric<TreeletType>::getLeftTreeletDescendents(const TreeletType& treelet, int index,
    //const map< vector<TreeletId>,TreeletId > &mapTreeletsToId
    const TreeletStoreByChildrenIndices& treeletStore
    ) const
    {
    // assert (checkTreeletHasAllTopChildrenEmptyAfterIndex(treelet,index));
    // vector<TreeletId> childVector=treelet.getChildrenIndices();
    // childVector[index]=TREELET_ID_SPECIAL_VALUE;
        
    // map< vector<TreeletId>,TreeletId >::const_iterator it1=mapTreeletsToId.find(childVector);
    // if (it1==mapTreeletsToId.end() )
        // {
        // throw logic_error("bug");
        // }

    // TreeletId leftTreeletId=it1->second;
        //assert (it++)==mapTreeletsToId.end();
    TreeletId leftTreeletId=treeletStore.getLeftTreeletId(treelet,index);

    TreeletIdToTreeletIdListMap::const_iterator it=descendingSuperTreelets.find(leftTreeletId);
    if (it==descendingSuperTreelets.end() )
        {
        throw TreeletBase::DescendingSuperTreeletsNotFoundException();
        }
    else
        {
        return it->second;
        }
    }
    
template<typename TreeletType>
vector<const TreeletType *> convertListOfTreeletIdToListOfTreelets(vector<TreeletId> idList,const typename TreeletType::TreeletSimpleIndexer& treeletIndexer)
    {
    vector<const TreeletType *> result;
    BOOST_FOREACH(TreeletId treeletId, idList)
        {
        const TreeletType& treelet=treeletIndexer.getTreelet(treeletId);
        result.push_back (&treelet);
        }
    return result;
    }
    
template<typename TreeletType>
const vector<TreeletId>& DescendingSuperTreeletsManagerGeneric<TreeletType>::getDescendingsuperTreeletIndices(TreeletId childIndex, const TreeletType& treelet, int numChild) const
    {
    if (childIndex==TREELET_ID_SPECIAL_VALUE)
        {
        PosType posNode=treelet.getPosNodeAtChildNum(numChild); 
        map<PosType,vector<TreeletId> >::const_iterator it=nodeToBaseTreeletTable.find(posNode);
        if (it==nodeToBaseTreeletTable.end() )
            {
            throw TreeletBase::DescendingSuperTreeletsNotFoundException();
            }
        else
            {
            return it->second;
            }
        }
    else
        {
        TreeletIdToTreeletIdListMap::const_iterator it=descendingSuperTreelets.find(childIndex);
        if (it==descendingSuperTreelets.end() )
            {
            throw TreeletBase::DescendingSuperTreeletsNotFoundException();
            }
        else
            {
            return it->second;
            }
        }
    }

template<typename TreeletType>
std::vector<TreeletType> DescendingSuperTreeletsManagerGeneric<TreeletType>::computeListOfSuperTreeletsLatteralStrategy( const TreeletType& treelet, bool computeAlsoAscendingSuperTreelet,int curIndex,
    //const map< vector<TreeletId>,TreeletId > &mapTreeletsToId,
    const TreeletStoreByChildrenIndices &mapTreeletsToId,
    const TreeletSimpleIndexer& treeletIndexer,int maxNbGaps) const
    {
    
    assert(treelet.checkTreeletHasAllTopChildrenEmptyAfterIndex(curIndex));
    vector<TreeletType> result;
    if (curIndex>=0)
        {
        const std::vector< TreeletId>& listOfDescLeftTreeletsId=getLeftTreeletDescendents(treelet, curIndex,mapTreeletsToId);
        std::vector< const TreeletType*> listOfDescLeftTreeletsPointers=convertListOfTreeletIdToListOfTreelets<TreeletType>(listOfDescLeftTreeletsId,treeletIndexer);
        
        TreeletId rightChildIndex=treelet.getChildIndex(curIndex);
        
        // if ( TreeletType::isUsingWildcard() && 
                    // noDiscontiguousSiblings && 
                    // treelet.isRootWildcard() && 
                    // rightChildIndex == TREELET_ID_SPECIAL_VALUE && 
                    // !treelet.hasContiguousChildrenBeforeIndex(curIndex)) {
            // // do not extend if we are going to create discontiguous siblings
        // } else { //extend right part
        
            const std::vector<TreeletId>& listOfDescRightTreeletsId=getDescendingsuperTreeletIndices(rightChildIndex, treelet, curIndex);
            //size_t sizeOfOrigRightChild=getSizeOfTreelet(rightChildIndex);
            
            //cerr<<"rightChildIndex:"<<rightChildIndex<<" listOfDescRightTreeletsId:" <<listOfDescRightTreeletsId<<endl;
            //cerr<<"listOfDescLeftTreeletsId:"<<listOfDescLeftTreeletsId<<endl;
            BOOST_FOREACH(TreeletId  rightIdDesc, listOfDescRightTreeletsId)
                {
                //size_t sizeOfNewChild=getSizeOfTreelet(rightIdDesc);
                TreeletType superTreelet=treelet.getExtendedDownwardTreelet(curIndex,rightIdDesc,treeletSizeInfos);
                superTreelet.setGeneratedAsDescendingSuperTreeletFlag(true);
                
                if (maxNbGaps<0||superTreelet.nbGaps()<=maxNbGaps)
                    {
                    result.push_back(superTreelet);
                    }
                }
        // }
        
        BOOST_FOREACH(const TreeletType*  leftDescTreeletPtr, listOfDescLeftTreeletsPointers)
            {
            assert (leftDescTreeletPtr->getChildIndex(curIndex)==TREELET_ID_SPECIAL_VALUE);
            TreeletType superTreelet=leftDescTreeletPtr->getExtendedDownwardTreelet(curIndex,rightChildIndex,treeletSizeInfos);
            
            //superTreelet.setRecursiveSplitIndices(rightChildIndex);
            
            //superTreelet.setChild(curIndex,rightChildIndex);  
            //size_t newSize=leftDescTreeletPtr->size + rightChildSize
            superTreelet.setGeneratedAsDescendingSuperTreeletFlag(true);
            if (maxNbGaps<0||superTreelet.nbGaps()<=maxNbGaps)
                {
                result.push_back(superTreelet);
                }
            }
        }
    
    else
        {
        if (treelet.hasSpecialSuperTreelets(curIndex))
            {
            //cerr<<treelet<<"  hasSpecialSuperTreelet"<<endl;
            treelet.addSpecialSupertreelets(result);
            }
        }
    /*
    if (computeAlsoAscendingSuperTreelet && treelet.hasUpwardExtension() )  //pRootNodeInQueryTree->hasParent())
        {
        
        Treelet superTreelet=treelet.getExtendedUpwardTreelet();
        superTreelet.setGeneratedAsDescendingSuperTreeletFlag(false);
        result.push_back(superTreelet);

        }
    */    
    return result;
    }
//int main(){ return 0; }


/*********************************************************************
 * Domination code
 */


/*
Construct a dictionary of the antexedents of treelets in surTreelets
Only construct for treelets reachable from (node,None)
*/ 
template<typename TreeletType>
std::map<TreeletId,std::set<TreeletId> > DescendingSuperTreeletsManagerGeneric<TreeletType>::buildPrecLinks(PosType nodePos) const
    {
    std::map<TreeletId,std::set<TreeletId> > precLinks;
    
    std::vector<TreeletId> queue;
    std::set<TreeletId> alreadyQueued;
    
    queue.push_back( TREELET_ID_SPECIAL_VALUE);
    alreadyQueued.insert(TREELET_ID_SPECIAL_VALUE);
    //alreadyQueued=set([(node,None)])
    
    
    while (queue.size()>0)
        {
        TreeletId treeletIndex=queue.back();
        queue.pop_back();
        
        const std::vector<TreeletId>* pSuperTreelets;
        if(treeletIndex==TREELET_ID_SPECIAL_VALUE)
            {
            map<PosType,vector<TreeletId> >::const_iterator it=nodeToBaseTreeletTable.find(nodePos);
            if (it==nodeToBaseTreeletTable.end())
                {
                pSuperTreelets=NULL;
                assert(false);
                throw runtime_error("bug");
                }
            else{
                pSuperTreelets=&( it->second );
                }
            }
        else
            {
            std::map<TreeletId,std::vector<TreeletId> >::const_iterator it= descendingSuperTreelets.find(treeletIndex);
            assert(it!=descendingSuperTreelets.end());
            pSuperTreelets=&(it->second);
            }
        
        BOOST_FOREACH(TreeletId superTreeletIndex, *pSuperTreelets)
            {
            precLinks[superTreeletIndex].insert(treeletIndex);
            if (alreadyQueued.count(superTreeletIndex)==0)
                {
                queue.push_back(superTreeletIndex);
                alreadyQueued.insert(superTreeletIndex);
                }
            }
        }
    return precLinks;
    }
    
template<typename TreeletType>
void DescendingSuperTreeletsManagerGeneric<TreeletType>::pruneDescendingSuperTreeletsList(PosType nodePos, TreeletId treeletIndex, 
                                            std::vector<TreeletId> &listOfDominatingTreelets, 
                                            std::map<TreeletId,std::set<TreeletId> >  &precLinks)
    {
    //std::cout<<"descendingSuperTreelets: "<<descendingSuperTreelets<<std::endl;
    //std::cout<<"listOfDominatingTreelets:"<<listOfDominatingTreelets<<std::endl;
    //std::cout<<"precLinks:"<<precLinks<<std::endl;
    //std::cout<<"t:"<<treeletIndex<<" n:"<<(int)nodePos<<std::endl;
    assert(precLinks.count(treeletIndex)==1);
    BOOST_FOREACH(TreeletId subTreeletIndex,precLinks[treeletIndex])//TODO: cas de treeletBase?
        {
        std::vector<TreeletId>* pSuperTreelets;
        if (subTreeletIndex==TREELET_ID_SPECIAL_VALUE)
            {
            pSuperTreelets=&(nodeToBaseTreeletTable[nodePos] );
            }
        else
            {
            std::map<TreeletId,std::vector<TreeletId> >::iterator it= descendingSuperTreelets.find(subTreeletIndex);
            assert(it!=descendingSuperTreelets.end());
            pSuperTreelets=&(it->second);            
            }    
/*            
        int indexToErase=-1;    
        for(size_t i=0;i<pSuperTreelets->size();++i)   //TODO: All of this is not very efficient...
            {
            if ( (*pSuperTreelets)[i] == treeletIndex)
                {
                assert (indexToErase==-1);
                indexToErase=i;
                }
            }
        assert (indexToErase!=-1);
        pSuperTreelets->erase(pSuperTreelets->begin()+ indexToErase); 
*/        
        //cerr<<*pSuperTreelets<<endl;
        std::set<TreeletId> superTreeletsSet(pSuperTreelets->begin(),pSuperTreelets->end() );
        assert(superTreeletsSet.size()==pSuperTreelets->size() );
        
        assert(superTreeletsSet.count(treeletIndex)==1);
        superTreeletsSet.erase(treeletIndex);
        
        std::copy(listOfDominatingTreelets.begin(), listOfDominatingTreelets.end(), std::inserter(superTreeletsSet,superTreeletsSet.end()) );
        
        assert(superTreeletsSet.count(treeletIndex)==0);
        
        pSuperTreelets->clear();
        pSuperTreelets->reserve(superTreeletsSet.size());
        std::copy(superTreeletsSet.begin(),superTreeletsSet.end(),std::back_inserter(*pSuperTreelets));
        
        }

    //updating precLinks
    
    assert(precLinks.count(treeletIndex)==1);
    std::set<TreeletId>& precLinksOfTreelet=precLinks[treeletIndex];
    
    BOOST_FOREACH(TreeletId superTreeletIndex,listOfDominatingTreelets)
        {
        assert(precLinks.count(superTreeletIndex)==1);
        std::set<TreeletId>& precSubTreeletsOfSuper=precLinks[superTreeletIndex];
        assert(precSubTreeletsOfSuper.count(treeletIndex)==1);
        precSubTreeletsOfSuper.erase(treeletIndex);
        std::copy(precLinksOfTreelet.begin(), precLinksOfTreelet.end(), std::inserter( precSubTreeletsOfSuper,precSubTreeletsOfSuper.begin() ));
        assert(precSubTreeletsOfSuper.count(treeletIndex)==0);
        }
    }

bool isSetIncludedInAnother(const std::set<TreeletId>& first, const std::set<TreeletId>& second) //TODO: optimisable by taking into account the fact that sets are sorted
    {
    BOOST_FOREACH(TreeletId id1,first)
        {
        if (second.count(id1)==0)
            {
            return false;
            }
        }
    return true;
    }
    
template<typename TreeletType>
void DescendingSuperTreeletsManagerGeneric<TreeletType>::removeTopElementaryWildcard(PosType nodePos, TreeletId wildcardId)
    {
    vector<TreeletId>& baseList=nodeToBaseTreeletTable[nodePos];
    assert(descendingSuperTreelets.count(wildcardId)==1);
    vector<TreeletId> newBaseList;
    
    // copy nodeToBaseTreeletTable without the wildcard
    bool hadWildcard=false;
    (void) hadWildcard; // NOP instruction to remove unused variable warning in non-debug mode
    
    BOOST_FOREACH(TreeletId tId,baseList)
        {
        if (tId==wildcardId)
            {
            assert(!hadWildcard);
            hadWildcard=true;
            }
        else
            {
            newBaseList.push_back(tId);
            }
        }
     /*
     // Actually possible that the wildcard was removed in a previous step
    if (!hadWildcard)
        {
        cerr<<"nodePos:"<<(unsigned int)nodePos<<endl;
        cerr<<"wildcardId:"<<wildcardId<<endl;
        cerr<<descendingSuperTreelets<<endl;
        cerr<<baseList<<endl;
        }
    assert(hadWildcard);
    */
    
    BOOST_FOREACH(TreeletId tId,descendingSuperTreelets[wildcardId])
        {
        newBaseList.push_back(tId);
        }
    //cerr<<nodeToBaseTreeletTable[nodePos]<<" - "<<wildcardId<<" + "<<descendingSuperTreelets[wildcardId]<<" = "<<newBaseList<<" :"<<(int)nodePos<<endl;
    nodeToBaseTreeletTable[nodePos]=newBaseList;    
    }
    

template<typename TreeletType>
void DescendingSuperTreeletsManagerGeneric<TreeletType>::removeWildcardRootedTreeletsFromNodeToBaseTable(PosType nodePos, 
        std::set<TreeletId> list_of_treelets_with_wildcard_roots)
    {
    vector<TreeletId>& baseList=nodeToBaseTreeletTable[nodePos];
    vector<TreeletId> newBaseList;
    
    // copy nodeToBaseTreeletTable without the wildcard
    BOOST_FOREACH(TreeletId tId,baseList)
        {
        if (list_of_treelets_with_wildcard_roots.count(tId)==0)
            {
            newBaseList.push_back(tId);
            }
        }
    nodeToBaseTreeletTable[nodePos]=newBaseList;    
    }
    
template<typename TreeletType>
void DescendingSuperTreeletsManagerGeneric<TreeletType>::reduceDescendingSuperTreeletsList(PosType nodePos,  const TreeletComputationManager &tcm,int curIndex,map<TreeletId,int> *treeletIdToIndexGeneration)
    {
    /*
    Modify surTreelets so that dominated treelets disappear
    */
    //cerr<<"reducing "<<(int)nodePos<<": "<<nodeToBaseTreeletTable[nodePos]<<" | "<<descendingSuperTreelets<<endl;
    map<TreeletId,set<TreeletId> > precLinks=buildPrecLinks(nodePos);
    //std::cout<<"orginal precLinks at nodePos "<< (int)nodePos <<" :\n"<<precLinks<<std::endl;
    // initialize queue with nodeToBaseTreeletsTable[nodePos]
    std::vector<TreeletId> queue(nodeToBaseTreeletTable[nodePos].begin(), nodeToBaseTreeletTable[nodePos].end() );
        
    //# processed: treelets already processed; (node,None) is considered already processed
    std::set<TreeletId> processed;
    processed.insert(TREELET_ID_SPECIAL_VALUE);
    
    while (queue.size()>0)
        {
        TreeletId treeletIndex=queue.back();
        queue.pop_back();
        
        if ( processed.count(treeletIndex)>0 ) {continue;}
        
        // We don't process a treelet until all of its subtreelets have been processed:
        assert(precLinks.count(treeletIndex)==1);
        if (!isSetIncludedInAnother(precLinks[treeletIndex],processed) ) {continue;}
           
        const std::vector<TreeletId> &superTreelets=descendingSuperTreelets[treeletIndex];
        
        // Update queue:
        std::copy(superTreelets.begin(), superTreelets.end(), std::back_inserter(queue)); //TODO: optimisable by not puting twice ids already in the queue ...

        //Find immediate dominating:
        
        
        if (treeletIdToIndexGeneration!=NULL)
            {
            assert(treeletIdToIndexGeneration->count(treeletIndex)==1);
            if ((*treeletIdToIndexGeneration)[treeletIndex]!=curIndex)
                {
                //cerr<<"skip "<<treeletIndex<<endl;
                processed.insert(treeletIndex);
                continue;
                }
            }
        
        vector<TreeletId> listOfDominatingTreelets=tcm.getImmediateDominatingTreeletsFromListUsingOwnIndexer(
                                                            treeletIndex,superTreelets);

        
        if (listOfDominatingTreelets.size()>0)
            {
            dominatedBy[treeletIndex]=listOfDominatingTreelets;
            //cerr<<"removing "<<treeletIndex<<endl;
            pruneDescendingSuperTreeletsList(nodePos,treeletIndex,listOfDominatingTreelets,  precLinks);
            }
        processed.insert(treeletIndex);
        }
    
    //cerr<<"reduced to "<<(int)nodePos<<": "<<nodeToBaseTreeletTable[nodePos]<<" | "<<descendingSuperTreelets<<endl;
    //std::cout<<"descendingSuperTreelets after reduction:\n"<<descendingSuperTreelets<<std::endl;
    //std::cout<<"nodeToBaseTreeletTable after reduction:\n"<<nodeToBaseTreeletTable[nodePos]<<std::endl;
    }

//typedef DescendingSuperTreeletsManagerGeneric<Treelet> DescendingSuperTreeletsManager;


#endif


