/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/


#include "para_sentence.hpp"
#include "dtree.hpp"

#include "TypeDefinitions.hpp"

#ifndef __EBMT_PP_CONVERSION_HPP__
#define __EBMT_PP_CONVERSION_HPP__


void set_word_info_from_other_infos(DtreeNode &w, const OtherInfosContainer& oic, int index) {
    w.word.lem     = oic.otherInfosVect.at(OIC_LEM_INDEX).at(index);
    w.word.cat     = oic.otherInfosVect.at(OIC_CAT_INDEX).at(index);
    w.word.pos     = oic.otherInfosVect.at(OIC_POS_INDEX).at(index);
    w.word.content = oic.otherInfosVect.at(OIC_CONTENT_INDEX).at(index);
    w.word.type = oic.otherInfosVect.at(OIC_TYPE_INDEX).at(index);
    //conversion string to short (not much checking here)
    w.word.content_p = atoi(oic.otherInfosVect.at(OIC_CONTENTP_INDEX).at(index).c_str());
}
    
void add_dtree_info_from_OtherInfosContainer(std::vector<DtreeNode>& words, const OtherInfosContainer& oic) {
    size_t nb_words = words.size();
    //std::cerr<<nb_words<<" == "<<oic.otherInfosVect[0].size()<<std::endl;
    assert (nb_words == oic.otherInfosVect[0].size());
    for (size_t i = 0; i < nb_words; i++) {
        set_word_info_from_other_infos(words[i], oic, i);
    }
}

void set_depend_information_of_dtree(std::vector<DtreeNode>& words, PosList dependency_list) {
    //assert(dtree.is_default_constructed());
	assert(words.size() == 0);
	int nb_words = dependency_list.size();
	//std::vector<Word*>& words = dtree.word_;
	
	// First, we set the word_ field to be the correct size
	words.resize(nb_words);
	
	// then we allocate the Word objects and set their num and dpnd fields
	for (int i = 0; i < nb_words; i++) {
		words[i].num = i;
		Dpnd dpnd = (dependency_list[i]==POSTYPE_SPECIAL_VALUE) ? -1 : dependency_list[i];
        words[i].dpnd=dpnd;
		if (dpnd == i) { // bug
            assert(false);
        }
	}
	
}

typedef std::vector<std::pair<std::vector<PosType>,
        std::vector<PosType> > > AlignmentWithUChar;
		
void set_alignment_of_para(std::vector<Alignment>& alignments, 
        std::vector<DtreeNode>& s_words, std::vector<DtreeNode>& t_words,
	    const AlignmentWithUChar& old_alignments) {
    //std::vector<Alignment> alignments;
	assert(alignments.size() == 0);
	alignments.resize(old_alignments.size());
    for (unsigned int i = 0; i < old_alignments.size(); i++) {
        Alignment& alignment = alignments[i];
        AlignIndex align_index = (AlignIndex) i;
        for (unsigned int j = 0; j < old_alignments[i].first.size(); j++) {
            WordIndex w = (WordIndex)old_alignments[i].first[j];
            assert(w< s_words.size());
            alignment.p1.push_back(w);
            s_words[w].align_index = align_index;
        }
        for (unsigned int j = 0; j < old_alignments[i].second.size(); j++) {
            WordIndex w = (WordIndex)old_alignments[i].second[j];
            assert(w< t_words.size());
            alignment.p2.push_back(w);
            t_words[w].align_index = align_index;            
        }
        alignment.accept = YES; 
    }
}




void set_alignment_of_tm(const std::set<PosType>& input_positions,
        const std::vector<PosType>& source_example_positions, 
		std::vector<Alignment>& alignments) {

	assert(input_positions.size() == source_example_positions.size() );
    assert(alignments.size() == 0);
    
    alignments.resize(input_positions.size());
    
    //std::vector<Alignment> alignments;
    // only need one alignment, but keep as a vector for the future
    int index=0;
    BOOST_FOREACH( PosType input_pos, input_positions) {
        assert(index < (int)source_example_positions.size() );
        Alignment& alignment = alignments[index];
        alignment.p1.push_back((WordIndex) input_pos);
        alignment.p2.push_back((WordIndex)source_example_positions[index]);
        alignment.accept = YES;    
        index++;
    }
}
	
    
//! setter for connected_image_
void set_connected_image_of_tm(const std::set<PosType>& image,
        std::vector<WordIndex>& connected_image_) {
    assert(connected_image_.size() == 0);
    connected_image_.reserve(image.size());
    BOOST_FOREACH(PosType pos, image) {
        connected_image_.push_back(pos);
    }
}
    
//! setter for additional_nodes_
void set_additional_nodes_of_tm(const std::vector<unsigned char>& add_nodes,
                std::vector<WordIndex>& additional_nodes_) {
                
    assert(additional_nodes_.size() == 0);
    additional_nodes_.reserve(add_nodes.size());
    BOOST_FOREACH(PosType pos, add_nodes) {
        additional_nodes_.push_back(pos);
    }

}

class TMBuilder {
public:
    static void build_para_sentence_in_place(const PosList& tgt_depend_list, 
        const std::vector<std::pair<std::vector<PosType>, std::vector<PosType> > >& match_vector,
            const AdditionalInfos& additional_info, const std::string& treeId,
            ParaSentence& result_para_sentence) {

        // int source_length = additional_info.sourceDepend.size();
        // int target_length = tgt_depend_list.size();
        
        assert(result_para_sentence.is_default_constructed());
        result_para_sentence.id_=treeId;
        
        //ParaSentence para_sentence(treeId, source_length, target_length);
        
        //result_para_sentence.set_alignment(match_vector);
        
        Dtree& source_dtree = result_para_sentence.s_dtree_;//get_modifiable_s_dtree();
        Dtree& target_dtree = result_para_sentence.t_dtree_;//para_sentence.get_modifiable_t_dtree();
        
        std::vector<DtreeNode>& source_words = source_dtree.nodes_;
        std::vector<DtreeNode>& target_words = target_dtree.nodes_;

        
        set_depend_information_of_dtree(source_words, additional_info.sourceDepend);
        set_depend_information_of_dtree(target_words, tgt_depend_list);
        
        add_dtree_info_from_OtherInfosContainer(source_words, additional_info.sourceOtherInfos);
        add_dtree_info_from_OtherInfosContainer(target_words, additional_info.targetOtherInfos);

        set_alignment_of_para(result_para_sentence.s_t_match_, source_words, target_words,
                                        match_vector);
        
        source_dtree.format_dtree();
        target_dtree.format_dtree();

    }

    static void build_tm_in_place_without_para_sentence(
        const std::set<PosType>& input_positions,
        const std::vector<PosType>& source_example_positions,
        const std::set<PosType>& image,
        const std::vector<PosType>& add_nodes,
        TM& tm) {

    assert(tm.i_s_match_.size() == 0);
    assert(tm.connected_image_.size() == 0);
    assert(tm.additional_nodes_.size() == 0);
    
    set_alignment_of_tm(input_positions, source_example_positions, tm.i_s_match_);
    set_connected_image_of_tm(image, tm.connected_image_);
    set_additional_nodes_of_tm(add_nodes, tm.additional_nodes_);
    
    //TODO:
    //tm.set_match_list(pProjInfo->matchList);
    }

    static void set_wildcards_of_tm(const std::set<PosType>& wildcards, TM& tm) {
        assert(tm.wildcards_.size() == 0);
        BOOST_FOREACH(PosType pos, wildcards) {
            assert(pos != POSTYPE_SPECIAL_VALUE);
            tm.wildcards_.insert(static_cast<WordIndex>(pos));
        }
    }
    
    static void build_tm_in_place(const PosList& tgt_depend_list, 
            const std::vector<std::pair<std::vector<PosType>, std::vector<PosType> > >& match_vector,
            const AdditionalInfos& additional_info, 
            const string& treeId, 
            const std::set<PosType>& input_positions,
            const std::vector<PosType>& source_example_positions,
            const std::set<PosType>& image,
            const std::vector<PosType>& add_nodes,
            TM& tm) {

        assert(tm.is_default_constructed());
        
        
        build_para_sentence_in_place(tgt_depend_list, match_vector, additional_info,
                    treeId, tm.para_sentence_);
            
        build_tm_in_place_without_para_sentence(input_positions, source_example_positions, image, add_nodes, tm);

    }
    
    

};
    
#endif //__EBMT_PP_CONVERSION_HPP__

