/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup result_output_group
    \brief Utility functions for converting to Perl objects.
*/

/*
extern "C" {
#include "perl.h"
}

#undef do_open
#undef do_close
*/

#include "TypeDefinitions.hpp"
#include "StringConversions.hpp"
#include "cTargetInfoList.cpp"
#include "AdditionalInfos.hpp"

#ifndef _HPP_PERL_CONVERSION_UTILS_
#define _HPP_PERL_CONVERSION_UTILS_

template<typename T>
void pushPosListToPerlArrayWithSpecialValueReplacement(AV* array, const T& posList)
    {
    
    BOOST_FOREACH(PosType pos,posList)
        {
        SV * svPos;
        if (pos==POSTYPE_SPECIAL_VALUE)
            {
            svPos =newSViv((int) -1);
            }
        else
            {
            svPos =newSViv((int) pos);
            }
        av_push(array, svPos);
        }
    }


void pushStringVectToPerlArray(AV* array, const std::vector<std::string>& stringVect)
    {
    BOOST_FOREACH(const std::string& elem,stringVect)
        {
        SV * svElem =newSVpvn(elem.c_str(),elem.size());
        SvUTF8_on(svElem);
        av_push(array, svElem);
        }
    }

SV* buildPerlVectFromStringVect(const std::vector<std::string>& stringVect)
    {
    AV* array =newAV();
    SV * svArray = newRV_noinc((SV *) array);
    pushStringVectToPerlArray(array,stringVect);
    return svArray;
    }
 
 
SV* buildPerlMatchArray(const std::vector< std::pair<std::vector<PosType>,std::vector<PosType> > >& matchList)
    {

    AV* aMatchMap=newAV();
    SV * svAMatchMap = newRV_noinc((SV *) aMatchMap);
    
    typedef std::pair<std::vector<PosType>,std::vector<PosType> > LinkTypeRH;
    BOOST_FOREACH(LinkTypeRH link,matchList)
        {
        HV * hLink=newHV();
        SV * svHLink = newRV_noinc((SV *) hLink);
        
        AV* aLeft=newAV();
        SV * svALeft = newRV_noinc((SV *) aLeft);
        pushPosListToPerlArrayWithSpecialValueReplacement(aLeft,link.first);    
        hv_store(hLink, "src", sizeof("src")-1 , svALeft, 0);


        AV* aRight=newAV();
        SV * svARight = newRV_noinc((SV *) aRight);
        pushPosListToPerlArrayWithSpecialValueReplacement(aRight,link.second);    
        hv_store(hLink, "tgt", sizeof("tgt")-1 , svARight, 0);
                     
        av_push(aMatchMap, svHLink);
        }
    return svAMatchMap;
    }                
 

    
SV * buildHashRefFromOtherInfosContainer(const OtherInfosContainer& oic  )
    {
    HV* hOthers =newHV();
    SV *result=newRV_noinc((SV *) hOthers);
    
    for(size_t i=0;i<OtherInfosContainer::nbOthers;++i)
        {
        const char * nameOfElem=OtherInfosContainer::otherElemNameList[i];
        hv_store(hOthers, nameOfElem, OtherInfosContainer::stringSizes[i]-1 , buildPerlVectFromStringVect(oic.otherInfosVect[i]), 0); 
        }
    
    return result;
    }
 
void addAddInfoToPerlHash(const AdditionalInfos& ai,
                            const PosList& targetDepend,
                            const std::vector< std::pair<std::vector<PosType>,std::vector<PosType> > >& matchList,
                             HV* hMatching)
    {
    AV* aSourceDepend =newAV();
    SV * svASourceDepend = newRV_noinc((SV *) aSourceDepend);
    pushPosListToPerlArrayWithSpecialValueReplacement(aSourceDepend,ai.sourceDepend);
    hv_store(hMatching, "src_dep", sizeof("src_dep")-1 , svASourceDepend, 0);
    
    //AV* aSourceSent =newAV();
   // SV * svaSourceSent = newRV_noinc((SV *) aSourceSent);
    //pushStringVectToPerlArray(aSourceSent,ai.sourceSentence);
    //hv_store(hMatching, "src_sent", sizeof("srd_sent")-1 , svaSourceSent, 0);        
    
    hv_store(hMatching, "i_data", sizeof("i_data")-1 , buildHashRefFromOtherInfosContainer(ai.sourceOtherInfos), 0); 
    hv_store(hMatching, "j_data", sizeof("j_data")-1 , buildHashRefFromOtherInfosContainer(ai.targetOtherInfos), 0); 
    
    AV* aTargetDepend =newAV();
    SV * svATargetDepend = newRV_noinc((SV *) aTargetDepend);
    pushPosListToPerlArrayWithSpecialValueReplacement(aTargetDepend,targetDepend);
    hv_store(hMatching, "tgt_dep", sizeof("tgt_dep")-1 , svATargetDepend, 0);
    
    
    SV * svAMatchMap = buildPerlMatchArray( matchList );                        
    hv_store(hMatching, "i_j_match_map", sizeof("i_j_match_map")-1 , svAMatchMap, 0);

    //hv_store(hMatching, "src_sent", sizeof("src_sent")-1 , buildPerlVectFromStringVect(ai.sourceSentence), 0); 
    //hv_store(hMatching, "tgt_sent", sizeof("tgt_sent")-1 , buildPerlVectFromStringVect(ai.targetSentence), 0); 
    //hv_store(hMatching, "src_POS", sizeof("src_POS")-1 , buildPerlVectFromStringVect(ai.sourceTags), 0); 
    //hv_store(hMatching, "src_POS", sizeof("tgt_POS")-1 , buildPerlVectFromStringVect(ai.targetTags), 0);
    }

void addBaseMatchingInfosToPerlHash(const PosList& mapp,
                                    const PosSet& image,
                                    const PosList& addNodes,
                                    const std::vector< std::pair<std::vector<PosType>,std::vector<PosType> > >& matchList,
                                    const std::string& treeId,
                                    HV* hMatching)
    {
        
    AV* aMapp =newAV();
    SV * svAMapp = newRV_noinc((SV *) aMapp);
    pushPosListToPerlArrayWithSpecialValueReplacement(aMapp,mapp);
    hv_store(hMatching, "mapp", sizeof("mapp")-1 , svAMapp, 0);

    AV* aImage =newAV();
    SV * svAImage = newRV_noinc((SV *) aImage);
    pushPosListToPerlArrayWithSpecialValueReplacement(aImage,image);    
    hv_store(hMatching, "image", sizeof("image")-1 , svAImage, 0);

    AV* aAdditionalNodes =newAV();
    SV * svAAdditionalNodes = newRV_noinc((SV *) aAdditionalNodes);
    pushPosListToPerlArrayWithSpecialValueReplacement(aAdditionalNodes,addNodes);    
    hv_store(hMatching, "add_nodes", sizeof("add_nodes")-1 , svAAdditionalNodes, 0);
  
    SV * svIdTree =newSVpvn(treeId.c_str(),treeId.size());
    SvUTF8_on(svIdTree);
    hv_store(hMatching, "id", sizeof("id")-1 , svIdTree, 0);

    SV * svAMatchMap = buildPerlMatchArray( matchList );
    
    
//        AV* aMatchMap=newAV();
//        SV * svAMatchMap = newRV_noinc((SV *) aMatchMap);
//        
//        typedef std::pair<std::vector<PosType>,std::vector<PosType> > LinkTypeRH;
//        BOOST_FOREACH(LinkTypeRH link,pProjInfo->matchList)
//            {
//            HV * hLink=newHV();
//            SV * svHLink = newRV_noinc((SV *) hLink);
//            
//            AV* aLeft=newAV();
//            SV * svALeft = newRV_noinc((SV *) aLeft);
//            pushPosListToPerlArray(aLeft,link.first);    
//            hv_store(hLink, "left", sizeof("left")-1 , svALeft, 0);


//            AV* aRight=newAV();
//            SV * svARight = newRV_noinc((SV *) aRight);
//            pushPosListToPerlArray(aRight,link.second);    
//            hv_store(hLink, "right", sizeof("right")-1 , svARight, 0);
//                         
//            av_push(aMatchMap, svHLink);
//            }
                    
    hv_store(hMatching, "match_map", sizeof("match_map")-1 , svAMatchMap, 0);

    }



void myPrintPosSetToStreamAsString(std::ostream& output, const std::set<PosType>& posSet,const std::string& delim )
    {
    bool first=true;
    BOOST_FOREACH(PosType pos, posSet)
        {
        if (first)
            {
            first=false;
            }
        else
            {
            output<<delim;
            }
        output<< (int) pos;
        }
    }

class AllMatchingsToPerlHashConverter
    {
    private:
    HV *hResult;
    AV *currentAVTreelet;
    HV *currentHMatching;
    public:
    AllMatchingsToPerlHashConverter(HV *hResult):hResult(hResult){}
    
    AllMatchingsToPerlHashConverter():hResult(NULL){}
    
    void pushTreelet(const PosSet& seqOfPos)
        {
        std::ostringstream oss;
        myPrintPosSetToStreamAsString(oss, seqOfPos, " ");
        std::string vectAsString=oss.str();
        pushTreelet(vectAsString);
        }
    
    void pushTreeletWithWildcard(const PosSet& seqOfPosTotal,const PosSet& seqOfPosNW)
        {
        std::ostringstream oss;
        myPrintPosSetToStreamAsString(oss, seqOfPosTotal, " ");
        oss<<"/";
        myPrintPosSetToStreamAsString(oss, seqOfPosNW, " ");
        std::string vectAsString=oss.str();
        pushTreelet(vectAsString);
        }
    
    void pushTreelet(const std::string &seqOfPosAsString)
        {
        assert(hResult!=NULL);
        currentAVTreelet = newAV();
        SV * svAvTreelet = newRV_noinc((SV *)currentAVTreelet);
        hv_store(hResult, seqOfPosAsString.c_str(), seqOfPosAsString.size(), svAvTreelet, 0);
        }
    
    
    void pushMatching(const std::string& treeId,
                            const PosList& mapp,
                            const PosSet& image,
                            const PosList& addNodes,
                            const std::vector< std::pair<std::vector<PosType>,std::vector<PosType> > >& matchList)
        {

        currentHMatching = newHV();
        SV * svHMatching = newRV_noinc((SV *) currentHMatching);

        addBaseMatchingInfosToPerlHash(mapp,
                                            image,
                                            addNodes,
                                            matchList,
                                            treeId,
                                            currentHMatching);
        
        av_push(currentAVTreelet,svHMatching);
   
        }

    void addAdditionalInfosToCurrentMatching(const AdditionalInfos& ai,
                                               const PosList& targetDepend,
                                            const std::vector< std::pair<std::vector<PosType>,std::vector<PosType> > >& matchList)
        {
        
        addAddInfoToPerlHash(ai,targetDepend,matchList,currentHMatching);
        
        }
    void finalize(){};
    
    };

#endif

