/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup result_output_group
    \brief Result Handler for outputing Perl objects.
*/

#include "GlobalTreeletRetrieving.hpp"
//#include "ResultsHandler.hpp" 
#include "PerlConversionsUtils.hpp"

#ifndef _HPP_PERLXS_RESULT_HANDLER_
#define _HPP_PERLXS_RESULT_HANDLER_


class PerlXSProcessor: public IndividualResultsProcessor
    {
    protected:
    AllMatchingsToPerlHashConverter perlConverter;
    //HV *hResult;
    //AV *currentAVTreelet;
    bool convertNumToTreeId;
    const std::vector<std::string>& numToTreeIdVector;
    
    public:
    PerlXSProcessor(HV *hResult,
                        const std::vector<std::string>& numToTreeIdVector,
                        bool convertNumToTreeId):
                        perlConverter(hResult),numToTreeIdVector(numToTreeIdVector),convertNumToTreeId(convertNumToTreeId)
                        {}
    
    void initializeForThisTreelet(const Treelet &treelet)
        {
        
        std::set<PosType> seqOfPos=treelet.getSeqOfPosRepresentation().getIterableSeqOfPos();
        perlConverter.pushTreelet(seqOfPos);
        }
    
    virtual void addAdditionalInfos(TreeIdType numTree)
        {}
    
    void processOneTreelet(TreeIdType numTree,
                            const PosList& mapp,
                            const ProjectedMappingInfos* pProjInfo,
                            size_t count_emitted,
                            MatchingsContainerSmartPtr occ,
                            size_t j,size_t numMapping)
        {
        if (pProjInfo==NULL)
            {
            throw std::exception();
            }
            
        std::string treeId=convertTreeId(numTree);


        perlConverter.pushMatching(treeId,
                                    mapp,
                                    *(pProjInfo->pConnectedImage),
                                    pProjInfo->additionalNodes,
                                    pProjInfo->matchList);
        addAdditionalInfos(numTree);
   
        }
        
    private:
    std::string convertTreeId(TreeIdType treeId)
        {
        if (convertNumToTreeId)
            {
            return numToTreeIdVector.at(treeId);
            }
        else
            {
            std::stringstream out;
            out << treeId;
            return out.str();
            }
        }
    
    };

class PerlXSProcessorWithAddInfos: public PerlXSProcessor
    {
    protected:
    const AddInfosVectorInterface& aDataVect;
    const TargetList& targetInfoList;
    public:
    PerlXSProcessorWithAddInfos(HV *hResult,
                        const std::vector<std::string>& numToTreeIdVector,
                        bool convertNumToTreeId,
                        const AddInfosVectorInterface& aDataVect,
                        const TargetList& targetInfoList):
                        PerlXSProcessor(hResult,numToTreeIdVector,convertNumToTreeId),aDataVect(aDataVect),targetInfoList(targetInfoList){}
    
    void addAdditionalInfos(TreeIdType numTree)
        {
        AdditionalInfos ai=aDataVect.at(numTree);
        const TargetInfo& targetInfo=targetInfoList[numTree];
        perlConverter.addAdditionalInfosToCurrentMatching(ai,targetInfo.depend, targetInfo.getMatchingInfosAsVector());
        }
    
    };

class TreeletRetrieverPerlXS: public TreeletRetrieverBase
    {
    public:
    TreeletRetrieverPerlXS(const char *datapath, const char *side, 
                        size_t maxNumberOfSamplePerTreelet, bool preserveOrder
                                ):TreeletRetrieverBase(datapath,side,maxNumberOfSamplePerTreelet,preserveOrder){}

    void handleRequestAsStringAndUsePerlHash(const std::string& labelListAsString, 
                                                const std::string& dependListAsString,
                                                HV *hResult)
        {

        PerlXSProcessor processor(hResult, 
                                     postProcessingInfos.getNumToTreeIdVector(), 
                                     true //convertNumToTreeId
                                     );
        
        handleRequestAsStringWithGivenProcessor(labelListAsString, dependListAsString, processor);

        }
    
    void handleRequestAsStringAndUsePerlHashWithAddInfos(const std::string& labelListAsString, 
                                                         const std::string& dependListAsString,
                                                         HV *hResult)
        {
        PerlXSProcessorWithAddInfos processor(hResult, 
                                             postProcessingInfos.getNumToTreeIdVector(), 
                                             true, //convertNumToTreeId
                                             postProcessingInfos.getAddInfoVect(),
                                             postProcessingInfos.getTargetInfoList());
                                
        handleRequestAsStringWithGivenProcessor(labelListAsString, dependListAsString, processor);

        }
    
                                 
    };

#endif

