/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup tree_data_struct_groups
    \brief Representation of treelets as a sequence of position in the query tree.
*/

#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include <boost/foreach.hpp>
#include <cassert>

#include <exception>

#include "TypeDefinitions.hpp"

#ifndef _HPP_SEQOFPOS_
#define _HPP_SEQOFPOS_

//using boost::BOOST_FOREACH;

//template <typename T>
//class SetAsSortedVector
//    {
//    std::vector<>
//    }

namespace TreeletBase
{


class SlotNotFoundException: public std::exception
{
  virtual const char* what() const throw()
  {
    return "Slot Not Found";
  }
};

/*
class SeqOfPosRepresentationWithWildcard: public SeqOfPosRepresentation
    {
    std::set<PosType> orderedSetOfWildcardPos;
    
    public:
    bool isIncludedIn(SeqOfPosRepresentation other) const
        {
        bool allIncluded=SeqOfPosRepresentation::isIncludedIn(other);
        if (!allIncluded) return false; //optimisation
        bool wildcardIncluded=includes(orderedSetOfWildcardPos.begin(),orderedSetOfWildcardPos.end(),other.orderedSetOfWildcardPos.begin(),other.orderedSetOfWildcardPos.end());
        return allIncluded && wildcardIncluded;
        }
    
    bool isEquivalentTo(const SeqOfPosRepresentation & other) const
        {
        bool allEquivalent=SeqOfPosRepresentation::isEquivalentTo(other);
        if (!allEquivalent) return false; //optimisation
        
        if ( other.orderedSetOfWildcardPos.size()!=orderedSetOfWildcardPos.size() )
            {
            return false;
            }
        else
            {
            return std::equal(orderedSetOfWildcardPos.begin(),orderedSetOfWildcardPos.end(),other.orderedSetOfWildcardPos.begin());
            }
        }
        
    void mergeWith(const SeqOfPosRepresentation & other)
        {
        SeqOfPosRepresentation::mergeWith(other);
        TO DO:
        std::set<PosType> temp;
        std::set_union(orderedSetOfPos.begin(),orderedSetOfPos.end(),other.orderedSetOfPos.begin(),other.orderedSetOfPos.end(),
                        std::inserter(temp,temp.end()));
        orderedSetOfPos.swap(temp);
        listOfStarredPos=listOfStarredPos1-setOfPos2+listOfStarredPos2-setOfPos1; 
       
        isInAConsistentState=false;    
        }
    
    SeqOfPosRepresentationWithWildcard merge(const SeqOfPosRepresentationWithWildcard & other) const
    {
    SeqOfPosRepresentation halfResult=SeqOfPosRepresentation::merge(other);
    SeqOfPosRepresentationWithWildcard result(halfResult);
    TO DO:
    std::set_union(orderedSetOfPos.begin(),orderedSetOfPos.end(),
                        other.orderedSetOfPos.begin(),other.orderedSetOfPos.end(),
                                std::inserter(result.orderedSetOfPos,result.orderedSetOfPos.end()));
    listOfStarredPos=listOfStarredPos1-setOfPos2+listOfStarredPos2-setOfPos1; //X
    result.isInAConsistentState=false;
    return result;
    }
    
    };
*/
class SeqOfPosRepresentation
    {
    //vector<PosType> seqOfPos;
    protected:
    std::set<PosType> orderedSetOfPos;
    
    std::map<PosType,SlotType> posToSlot;
    
    PosType parent;
    bool isInAConsistentState;
    friend std::ostream &operator<< (std::ostream &stream,const SeqOfPosRepresentation &sop);
    public:
        SeqOfPosRepresentation(): parent(POSTYPE_SPECIAL_VALUE), isInAConsistentState(true) {}   
        
        SeqOfPosRepresentation(PosType rootPos,PosType parent=POSTYPE_SPECIAL_VALUE):parent(parent), isInAConsistentState(false) 
            {
            orderedSetOfPos.insert(rootPos);
            }
        
        SeqOfPosRepresentation(const SeqOfPosRepresentation &objectToCopy) //copy constructor
            {
            parent=objectToCopy.parent;
            orderedSetOfPos=objectToCopy.orderedSetOfPos;
            isInAConsistentState=false;
            }
            
        SeqOfPosRepresentation& operator=(const SeqOfPosRepresentation &objectToCopy)
            {
            // no self-assignment problem here
            parent=objectToCopy.parent;
            orderedSetOfPos=objectToCopy.orderedSetOfPos;
            isInAConsistentState=false;
            return *this;           
            }
        

        void printReprAsStringToStream(std::ostream &stream) const
            {
            BOOST_FOREACH(PosType pos,orderedSetOfPos)
                {
                stream<<(int)pos;
                stream<<" ";
                }
            }
        
        inline SlotType getSlotOfPos(PosType pos) const
            {
                std::map<PosType,SlotType>::const_iterator it = getPosToSlotDict().find(pos);
                if (it==posToSlot.end())
                    {
                    throw SlotNotFoundException();
                    }
                return it->second;
            }
           
        PosType getParent() const
            {
            return parent;
            }
            
        void addParent(PosType parent)
            {
            this->parent=parent;
            }
            
        size_t getSizeOfSeq() const
            {
            return orderedSetOfPos.size();
            }
           
        const std::set<PosType>& getIterableSeqOfPos() const
            {
            return orderedSetOfPos;
            }
        
        const std::set<PosType>&  getIterableSeqOfPosNonWildcard() const
            {
            throw std::logic_error("Not implemented: should not be called");
            //return std::set<PosType>();
            }
       
        bool hasJunctionWith(SeqOfPosRepresentation other) const
            {
            std::vector<PosType> temp;
            //std::set<PosType>::iterator itSet=temp.begin();
            //std::vector<PosType> temp2;
            std::set_intersection(orderedSetOfPos.begin(),orderedSetOfPos.end(),
                            other.orderedSetOfPos.begin(),other.orderedSetOfPos.end(),
                                std::back_inserter(temp) );//temp.begin());
            if (!temp.empty())
                {
                return true;
                }
            else if (other.contains(parent) || contains(other.parent) )
                {
                return true;
                }
            else
                {
                return false;
                }
            }
            
        SlotType getSlotOfParent() const
            {
            return orderedSetOfPos.size();
            }
            
        bool isIncludedIn(SeqOfPosRepresentation other) const
            {
            return includes(other.orderedSetOfPos.begin(),other.orderedSetOfPos.end(),orderedSetOfPos.begin(),orderedSetOfPos.end());
            }
        
        bool contains(PosType pos) const
            {
            return orderedSetOfPos.find(pos)!=orderedSetOfPos.end();
            }
            
        void mergeWith(const SeqOfPosRepresentation &other)
            {
            //std::cout<<"merge sop: "<<*this<<" with "<<other<<std::endl;
            if ( other.contains(parent) )
                {
                assert (!contains(other.parent));
                parent=other.parent;
                }
            else 
                {
                assert(  (contains(other.parent) && !other.contains(parent))   // the other is rooted higher than self
                            || parent==other.parent );    // or the other and self are rooted at the same place
                }
                
            std::set<PosType> temp;
            std::set_union(orderedSetOfPos.begin(),orderedSetOfPos.end(),other.orderedSetOfPos.begin(),other.orderedSetOfPos.end(),
                            std::inserter(temp,temp.end()));
            orderedSetOfPos.swap(temp);
            
            isInAConsistentState=false;
            }
            
        SeqOfPosRepresentation merge(const SeqOfPosRepresentation &other) const
            {
            SeqOfPosRepresentation result;
            
            if ( other.contains(parent) )
                {
                assert (!contains(other.parent));
                result.parent=other.parent;
                }
            else 
                {
                result.parent=parent;
                assert(  (contains(other.parent) && !other.contains(parent))   // the other is rooted higher than self
                            || parent==other.parent );    // or the other and self are rooted at the same place
                }
                
            std::set_union(orderedSetOfPos.begin(),orderedSetOfPos.end(),
                                other.orderedSetOfPos.begin(),other.orderedSetOfPos.end(),
                                        std::inserter(result.orderedSetOfPos,result.orderedSetOfPos.end()));
            result.isInAConsistentState=false;
            return result;
            }            
            
            
        bool isEquivalentTo(const SeqOfPosRepresentation &other) const
            {
            if ( other.orderedSetOfPos.size()!=orderedSetOfPos.size() )
                {
                return false;
                }
            else
                {
                return std::equal(orderedSetOfPos.begin(),orderedSetOfPos.end(),other.orderedSetOfPos.begin());
                }
            }
            
        const std::map<PosType,SlotType> &getPosToSlotDict() const
            {
            // synamic cast so that getPosToSlotDict can be considered a const operation: d
            const_cast<SeqOfPosRepresentation *>(this)->makeConsistent();
            return posToSlot;
            }
            
        inline void makeConsistent()
            {
            if (isInAConsistentState)
                {
                return;
                }
            size_t i=0;
            posToSlot.clear();
            BOOST_FOREACH(PosType pos,orderedSetOfPos)
                {
                posToSlot.insert( std::pair<PosType,SlotType>(pos,i) );
                i++;
                }
            isInAConsistentState=true;
            }
            
        std::vector<PosType> getListOfCommonPos(SeqOfPosRepresentation other) const
            {
            std::vector<PosType> result;
            std::set_intersection(orderedSetOfPos.begin(),orderedSetOfPos.end(),other.orderedSetOfPos.begin(),other.orderedSetOfPos.end(),
                                    std::back_inserter(result) );
            return result;
            }

        std::vector<PosType> getPosListDifference(SeqOfPosRepresentation other) const
            {
            std::vector<PosType> result;
            std::set_difference(orderedSetOfPos.begin(),orderedSetOfPos.end(),other.orderedSetOfPos.begin(),other.orderedSetOfPos.end(),
                                    std::back_inserter(result) );
            return result;
            }
            
    };
  
std::ostream &operator<<(std::ostream &stream,const std::set<unsigned char> &s)
  {
  stream<<"Set(";
  BOOST_FOREACH(unsigned char elem,s)
    {
    stream<<(int)elem<<",";
    }
  stream<<")";
  return stream;
  }
  

  
std::ostream &operator<< (std::ostream &stream,const TreeletBase::SeqOfPosRepresentation &sop)
  {
  //using namespace baseDefinitions;
  
  stream<<"SOP(";
  stream<<(int)sop.parent;
  stream<<"|";
  set<PosType> copy(sop.orderedSetOfPos);
  stream<<copy;
  stream<<")";
  return stream;
  }
  
}; //namespace TreeletBase 


  


    
#endif






