/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup server_group
    \brief Utility functions for converting unicode strings into STL vectors of words.
*/

#include<vector>
#include<map>
#include "TypeDefinitions.hpp"

#include "utf8cpp/utf8.h" 
#include <stdint.h>
#include <sstream>

#include <fstream>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/map.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>

#ifndef _HPP_STRING_CONVERSIONS_
#define _HPP_STRING_CONVERSIONS_ 

class DependListConversionException: public std::exception
{
  virtual const char* what() const throw()
  {
    return "DependListConversionException";
  }
};


std::vector< std::vector<uint32_t> > splitUtf32Vector(const std::vector<uint32_t>& utf32Vector)
    {
    std::vector< std::vector<uint32_t> > result;
    
    std::vector<uint32_t> temp;
    BOOST_FOREACH(uint32_t utf32CodePoint, utf32Vector)
        {       
        if (utf32CodePoint==0x0020)
            {
            if (temp.size()>0)
                {
                result.push_back(temp);
                temp.clear();
                }
            }
        else
            {
            temp.push_back(utf32CodePoint);
            }
        }    

        if (temp.size()>0)
            {
            result.push_back(temp);
            temp.clear();
            }
    return result;
    }

std::vector< std::string> splitUtf8String(const std::string& s)
    {
    std::vector< std::string> result;
    
    std::vector<uint32_t> utf32Vector;
    utf8::utf8to32(s.begin(), s.end(), std::back_inserter(utf32Vector));
    
    std::vector< std::vector<uint32_t> > splittedVector = splitUtf32Vector(utf32Vector);
    
    BOOST_FOREACH(const std::vector<uint32_t>& utf32word, splittedVector)
        {
        std::string temp;
        utf8::utf32to8(utf32word.begin(),utf32word.end(),std::back_inserter(temp) );
        result.push_back(temp);
        }
    return result;
    }

std::vector<int> convertToVectorInt(const std::string& s)
    {
    std::vector<int> res;
    std::istringstream iss(s);
    std::copy(std::istream_iterator<int>(iss),
        std::istream_iterator<int>(),
            std::back_inserter<std::vector<int> >(res));
    return res;
    }

template<typename T>
std::vector<T> convertStringToVectorGeneric(const std::string& s)
    {
    std::vector<T> res;
    std::istringstream iss(s);
    std::copy(std::istream_iterator<T>(iss),
        std::istream_iterator<T>(),
            std::back_inserter<std::vector<T> >(res));
    return res;
    }


std::vector<PosType> parseDependList(std::string s)
    {
    std::vector<PosType> result;
    std::vector<int> vectorInt=convertToVectorInt(s);
    BOOST_FOREACH(int i, vectorInt)
        {
        if (i==-1)
            {
            result.push_back(POSTYPE_SPECIAL_VALUE);
            }
        else
            {
            if (i<0 || i> POSTYPE_MAXIMAL_VALUE || i==POSTYPE_SPECIAL_VALUE)
                {
                throw DependListConversionException();
                }    
            result.push_back( (PosType) i );
            }
        }
    return result;
    }    


/****************************************************
 *  Utilities for python conversion ...
 */


template<typename T>
void saveGeneric(const T& object, const char *filename)
    {
    std::ofstream ofs(filename,std::ofstream::binary);
    boost::archive::binary_oarchive oa(ofs);
    oa <<object;
    }

template<typename T>
void loadGeneric(T& object, const char *filename)
    {
    std::ifstream ifs(filename,std::ifstream::binary);
    boost::archive::binary_iarchive ia(ifs);
    ia>>object;
    }

typedef std::vector<std::string> StringVector;    
    
void saveStringVector(const StringVector& sv, const char *filename)
    {
    saveGeneric(sv,filename);
    }
    
#endif
    
