/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "tr_interface.hpp"
#include <iostream>
#include <boost/foreach.hpp>

using std::cout;
using std::endl;

template<typename T>
std::ostream &operator<< (std::ostream &stream,const std::vector<T> &vect)
  {
  stream<<"Vect("<<vect.size()<<"/"<<vect.capacity()<<")<";
  for (unsigned int i=0;i<vect.size();++i)
    {
    stream<<vect[i]<<" ";
    }
  stream<<">";
  return stream;
  }

  
template<typename T>
std::ostream &operator<< (std::ostream &stream,const std::set<T> &vect)
  {
  stream<<"Set(";
  BOOST_FOREACH(const T& elem,vect)
    {
    stream<<elem<<" ";
    }
  stream<<")";
  return stream;
  }
  
int main()
    {
    
    // First, define the search parameters (default should be fine in general)
    //SearchParameters sParams=get_default_search_params();
    //PostProcessingParameters ppParams=get_default_post_processing_parameters();
    
    // Then create the TR object
    // TreeletRetrieverForCPP tr("/home/fabien/progs/datatests/testLat","2",
                        // sParams,
                        // ppParams
                        // );

    TRInterfacePtr tr=get_default_TR("/home/fabien/progs/datatests/testLat","2");
          
    //TRInterfacePtr tr=get_client_TR("/home/fabien/progs/datatests/testLat","2","localhost","9998");
          
    // Perform a search
    TRResultPtr result=tr->retrieve("the man is here","1 2 -1 2");

    
    
    
    // Extract results from the search
    cout<<"nb matched pattern:"<<result->get_nb_of_pattern()<<endl;
    cout<<"matched pattern:"<<result->get_vector_of_pattern()<<endl;
    BOOST_FOREACH(const PatternType& pattern,result->get_vector_of_pattern())
        {
        int nb_tm=result->get_nb_of_tm_for_pattern(pattern);
        cout<<pattern<<" : "<<nb_tm<<endl;
        for (int i=0;i<nb_tm;i++)
            {
            cout<<i<<": ";
            const TM& tm=result->get_tm(pattern,i);
            cout<<tm.get_para_sentence_ref().id()<<endl;
            result->add_tm(pattern,tm); //doubling each tm for test
            PatternType pattern_bis=pattern;
            //pattern_bis.push_back(99);
            //result->add_tm(pattern_bis,tm); //doubling each tm to different pattern for test
            }
        }

    BOOST_FOREACH(const PatternType& pattern,result->get_vector_of_pattern())
        {
        int nb_tm=result->get_nb_of_tm_for_pattern(pattern);
        cout<<pattern<<" : "<<nb_tm<<endl;
        for (int i=0;i<nb_tm;i++)
            {
            cout<<i<<": ";
            const TM& tm=result->get_tm(pattern,i);
            cout<<tm.get_para_sentence_ref().id()<<endl;
            }
        }        
    
    return 0;
    }

