/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup server_group
    \brief Generic code for client.
*/

#include "TRClientServerCommonValues.hpp"
#include "AdditionalInfos.hpp"
#include "IndexersAndEncoding.hpp"
#include "TypeDefinitions.hpp"
#include "DefaultDataPath.hpp"


#include<set>
#include <vector>
#include <fstream>
#include <iostream>

#include <boost/serialization/vector.hpp>
#include <boost/serialization/base_object.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>


#include <boost/asio.hpp>

using boost::asio::ip::tcp;

#ifndef HPP_TR_CLIENT_GENERIC_
#define HPP_TR_CLIENT_GENERIC_

///////////////////////////
/*
class ConverterClass
    {
    public:
    
    void pushTreelet(const std::set<PosType>& seqOfPos)
        {
        std::cout<<"pushTreelet :"<<seqOfPos<<std::endl;
        }
    
    void pushTreelet(const std::string &seqOfPosAsString)
        {
        std::cout<<"pushTreelet :"<<seqOfPosAsString<<std::endl;
        }
    
    
    void pushMatching(const std::string& treeId,
                            const std::vector<PosType> & mapp,
                            const std::set<PosType>& image,
                            const std::vector<PosType>& addNodes,
                            const std::vector< std::pair<std::vector<PosType>,std::vector<PosType> > >& matchList)
        {
        std::cout<<"pushMatching: "<< treeId <<std::endl;
        std::cout<<"pushMatching: "<< mapp <<std::endl;
        }

    void addAdditionalInfosToCurrentMatching(const AdditionalInfos& ai,
                                               const std::vector<PosType>& targetDepend,
                                            const std::vector< std::pair<std::vector<PosType>,std::vector<PosType> > >& matchList)
        {
        std::cout<<"pushAI: "<< ai <<std::endl;
        std::cout<<"pushAI: "<< matchList <<std::endl;
        std::cout<<"addAdditionalInfosToCurrentMatching" <<std::endl;
        
        }
    
    };
*/

template<typename ConverterClass, typename ResultClass>
class StreamDeserializerGeneric
    {
    public:
    typedef std::vector<PosType> PosList;
    typedef std::set<PosType> PosSet;
    
    std::istream& stream;
    AdditionalInfosUnpackerBase aiUnpacker;
    std::vector<std::string> numToTreeIdVector;
    ConverterClass converter;
    
    //typedef ResultClass ResultClassType;
    
    StreamDeserializerGeneric(std::istream& stream,const char * addInfosFilename, const char * numToTreeIdVectorFilename):
                            stream(stream),aiUnpacker(addInfosFilename)
                            
        {
        loadNumToTree(numToTreeIdVectorFilename);
        }


    void loadNumToTree(const char *filename)
        {
        std::ifstream ifs(filename,std::ifstream::binary);
        try
            {
            boost::archive::binary_iarchive ia(ifs);
            ia>>numToTreeIdVector;
            }
        catch (boost::archive::archive_exception e)
            {
                std::cerr<<"Error loading numToTreeIdVector from file "<<filename<<std::endl;
                std::cerr<<"what:"<<e.what()<<std::endl;
                throw e;
            }               
        
        }
    
    void readOneReply(ResultClass hResult)
        {
        converter=ConverterClass(hResult);
        _readOneReply();
        }    

    void _readOneReply()
        {
        size_t nbRead=0;
        while( 1 )
            {
            bool successReading=readAndDeserializeTreelet();
            if(successReading)
                {
                nbRead++;
                }
            else
                {
                break;
                }
            }
#ifndef NDEBUG //just to avoid "nbTransmittedElems not used" warning when compiled with -DNDEBUG
        size_t nbTransmittedElems=
#endif
        readMessageLength();  //a bit hackish: if not NDEBUG, nbTransmittedElems get set here
        assert(nbRead==nbTransmittedElems);
        converter.finalize();
        }   


    bool readTreeletSeqOfPos()
        {
        //std::cout<<">>readTreeletSeqOfPos"<<std::endl;
        size_t messageLength=readMessageLength();
       
        if (messageLength==0)
            {
            //std::cout<<"<<readTreeletSeqOfPos: no more"<<std::endl;
            return false;
            }
        //std::cerr<<"ML:"<<messageLength<<std::endl;
        //assert(messageLength<10000);
        if (messageLength>maxAllowedMessageLengthForSeqOfPos)
            {
            // A bit brutal, but if there is a bug here, we don't want to allocate a huge amount of memory
            std::cerr<<"Fatal transmission error: request MessageLength for seqOfPos is: "<<messageLength<<std::endl;
            exit(1);
            }
        char *bufferMessage=new char[messageLength];
        stream.read(bufferMessage,messageLength);
        
        
        std::string code(bufferMessage,messageLength);
        delete[] bufferMessage;
        
        PrefixDecodeVectors decoder(code);
        
        std::set<PosType> seqOfPos;
        decoder.readPosListUCToSet(seqOfPos);
        //cerr<<"isFinished:"<<decoder.isFinished()<<endl;
        //assert(decoder.isFinished());
        if (decoder.isFinished())
            {
            converter.pushTreelet(seqOfPos);
            }
        else
            {
            std::set<PosType> seqOfPosNW;
            decoder.readPosListUCToSet(seqOfPosNW);
            converter.pushTreeletWithWildcard(seqOfPos,seqOfPosNW);
            }
        //std::cout<<"<<readTreeletSeqOfPos: more:"<<seqOfPos<<std::endl;
        return true;
        }
    bool readAndDeserializeTreelet()
        {
        //std::cout<<">>readAndDeserializeTreelet"<<std::endl;
        bool hasTreeletRemaining=readTreeletSeqOfPos();
        if (!hasTreeletRemaining)
            {
            //std::cout<<"<<readAndDeserializeTreelet: no more"<<std::endl;
            return false;
            }

        
        size_t nbRead=0;
        while(1)
            {
            bool successReading=readAndDeserializeMatching();
            if(successReading)
                {
                nbRead++;
                }
            else
                {
                break;
                }            
            }
        
#ifndef NDEBUG //just to avoid "nbTransmittedElems not used" warning when compiled with -DNDEBUG
        size_t nbTransmittedElems=
#endif
        readMessageLength();  //a bit hackish: if not NDEBUG, nbTransmittedElems get set here
        
        
        //std::cout<<"nbRead/nbTransmitted "<<nbRead<<" "<<nbTransmittedElems<<std::endl;
        assert(nbRead==nbTransmittedElems);
        //std::cout<<"<< readAndDeserializeTreelet: had more"<<std::endl;
        return true;    
        }


    size_t readMessageLength()
        {
	    unsigned char bufferMessageLength[nbBytesForEncodingMessageLength];
        stream.read((char *)bufferMessageLength,nbBytesForEncodingMessageLength);
        return decodeMessageLength(bufferMessageLength);    
        }

    bool readAndDeserializeMatching()
	    {
        size_t messageLength=readMessageLength();

        if (messageLength==0)
            {
            return false;
            }
            
        if (messageLength>maxAllowedMessageLengthForMatching)
            {
            // A bit brutal, but if there is a bug here, we don't want to allocate a huge amount of memory
            std::cerr<<"Fatal transmission error: request MessageLength for matching is: "<<messageLength<<std::endl;
            exit(1);
            }
            
        char *bufferMessage=new char[messageLength];
        stream.read(bufferMessage,messageLength);
        
        deserializeMatchingInfos(bufferMessage,messageLength);
        
        delete[] bufferMessage;
        return true;
        }
    
    void deserializeMatchingInfos(char *array, size_t length)
        {
        std::string code(array,length);  // kind of inefficient here...
        PrefixDecodeVectors decoder(code);
        
        size_t numTree=decoder.readOnePackedUInt();
        std::string treeId=numToTreeIdVector[numTree];
        
        PosList mapp;
        decoder.readPosListUCToVector(mapp);
        
        PosSet image;
        decoder.readPosListUCToSet(image);
        
        PosList addNodes;
        decoder.readPosListUCToVector(addNodes);
        
        std::vector<std::pair<std::vector<unsigned char>,std::vector<unsigned char> >  > matchListPartial;
        decoder.readMatchListUC(matchListPartial);
        
        converter.pushMatching(treeId, mapp, image, addNodes, matchListPartial);
        
        deserializeAddInfos(decoder);
        
        }  
    
    void deserializeAddInfos(PrefixDecodeVectors& decoder)
        {
        PosList targetDepend;
        decoder.readPosListUCToVector(targetDepend);
        
        std::vector<std::pair<std::vector<unsigned char>,std::vector<unsigned char> >  > matchListGlobal;
        decoder.readMatchListUC(matchListGlobal);
        
        AdditionalInfos ai;
        aiUnpacker.unpackWithGivenDecoder(decoder,ai);
        
        converter.addAdditionalInfosToCurrentMatching(ai,targetDepend,matchListGlobal);
        }

    
    };

    
template<typename StreamDeserializerType, typename ResultClass>
class TRClientGeneric
	{
	public:
	tcp::iostream socketstream;
	StreamDeserializerType streamDeserializer;
	TRClientGeneric(std::string datapath,std::string side,std::string host,std::string port):
	                                    socketstream(host,port),
	                                    streamDeserializer(
	                                        socketstream,
	                                        DataFilenameSufixes::getAdditionalDatasFilename(datapath,side).c_str(),
	                                        DataFilenameSufixes::getNumTreeToIdFilename(datapath).c_str()
	                                        )
                                            
	    {
        // std::cerr<<"Connecting to "<<datapath
                // <<" side:"<<side<<" (host:"<<host
                // <<" port:"<<port<<std::endl;
	    } 
	    
	bool isConnected() const
	    {
	    return (bool) socketstream;
	    }

	void sendRequest(std::string labelList,std::string depend)
	    {
	    socketstream<<labelList<<std::endl;
        socketstream<<depend<<std::endl;
        socketstream<<"EOS"<<std::endl;
	    
	    }

	//void readResultAsPerlHash(ResultClass hResult)
	//    {
	//    streamDeserializer.readOnereply(hResult);
	//    }
	    
	void readResult(ResultClass hResult)
	    {
	    streamDeserializer.readOneReply(hResult);
	    }
	};

    
#endif

