/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup tree_data_struct_groups
    \brief Description of the Treelet Class used in the wildcard case.
*/

#include <vector>
#include <algorithm>

#include "TypeDefinitions.hpp"
#include "Tree.hpp"

#include "SeqOfPosRepresentationWildcard.hpp"
#include "TreeletsBase.hpp"

#ifndef _HPP_MY_TREELETS_WILDCARD_
#define _HPP_MY_TREELETS_WILDCARD_

namespace TreeletWildcard
{


using TreeletBase::DescendingSuperTreeletsNotFoundException;
//using namespace streamOperatorsUtils;
using streamOperatorsUtils::operator<<;

// template<typename T1,typename T2>
// std::ostream &operator<< (std::ostream &stream,const std::map<T1,T2> &s)
  // {
  // stream<<"Map(";
  // typedef std::pair<T1,T2> KeyValPair;
  // BOOST_FOREACH(const KeyValPair& keyval,s)
    // {
    // stream<<"("<<keyval.first <<":" << keyval.second<<")\n";
    // }
  // stream<<")";
  // return stream;
  // }


class TreeletAsRecursiveRepresentation: public TreeletBase::TreeletAsRecursiveRepresentation
    {
    bool isRootWildcard;
    
    friend std::ostream &operator<< (std::ostream &stream,const TreeletAsRecursiveRepresentation &treeletRecRepr);
    public:
    
    //void computeTypeRepresentation(const TreeletSimpleIndexer& tsi)
    
    bool checkIfRootWildcard() const
        {
        return isRootWildcard;
        }
    
    void setRootWildcard(bool b)
        {
        isRootWildcard=b;
        }
    
    TreeletAsRecursiveRepresentation(const Node *pRootNodeInQueryTree,std::vector<TreeletId> childrenIndices,bool rootIsWildcard):
                TreeletBase::TreeletAsRecursiveRepresentation(pRootNodeInQueryTree,childrenIndices),isRootWildcard(rootIsWildcard) {};
    
    // Constructor for elementary treelets:            
    TreeletAsRecursiveRepresentation(const Node *pRootNodeInQueryTree,size_t nbChildren,bool rootIsWildcard):
        TreeletBase::TreeletAsRecursiveRepresentation(pRootNodeInQueryTree,nbChildren),
        isRootWildcard(rootIsWildcard){};    
        
    
    inline bool operator<(const TreeletAsRecursiveRepresentation &other) const
        {
        if (isRootWildcard!=other.isRootWildcard)
            {
            return isRootWildcard; //wildcard comes before regular nodes
            }
        else
            {
            return TreeletBase::TreeletAsRecursiveRepresentation::operator<(other);
            }
        };

        
    //TO DO: check the following
    /*
    const std::vector<TreeletId>& getChildrenIndices() const
        {
        return childrenIndices;
        }
        
    void replaceChild(size_t childNumber, TreeletId newChildId)
        {
        childrenIndices[childNumber]=newChildId;
        
        };
        
    bool hasUpwardExtension() const
        {
        return pRootNodeInQueryTree->hasParent();
        };
    */
    };
 

std::ostream &operator<< (std::ostream &stream,const TreeletAsRecursiveRepresentation &treeletRecRepr)
  {

  stream<<"t"<<"[("<<(int)treeletRecRepr.pRootNodeInQueryTree->getPosition()<< (treeletRecRepr.isRootWildcard?"*":"")<<") ";
  
  bool first=true;
  BOOST_FOREACH(TreeletId id, treeletRecRepr.childrenIndices)
    {
    if (first)
        {
        first=false;
        }
    else
        {
        stream<<" ";
        }
    if (id==TREELET_ID_SPECIAL_VALUE)
        {
        stream<<"N";
        }
    else
        {
        stream<<id;
        }
    }
  stream<<"]";
  return stream;
  }

  

// size_t getSizeOfTreeletFunction(const Treelet&); //to solve forwarding problems
// size_t getNonWildcardSizeOfTreeletFunction(const Treelet&); 


  
class Treelet: public TreeletBase::TreeletBaseClass
    {
    public:
        typedef SeqOfPosRepresentation SeqOfPosRepresentation_;
    //vector<PosType> seqOfPos;
    //PosType parent;
    //TreeletSignature signature; // such that t1.signature==t2.signature => occ(t1)==occ(t2)
    
    SeqOfPosRepresentation seqOfPos;
    TreeletAsRecursiveRepresentation recursiveRepr;
    //TreeletId index;
    //size_t size;
    size_t sizeNonWildcard;
    
    //size_t nbSentOcc;
    //bool nbSentOccIsSet;
    //bool treeletIdIsSet;
    //bool seqOfPosIsConsistent;
    std::vector<size_t> sizeOfChildrenVector;
    std::vector<size_t> sizeNonWildcardOfChildrenVector;
    
    // flag used for descending supertreelets management:
    //bool wasGeneratedAsDescendingSuperTreeletFlag;
    
    //bool isElementaryTreeletFlag;
    
    //std::string typeRepresentation;
    //bool isTypeRepresentationComputed;
    
    // int splitIndex;
    // TreeletId leftSplitTreeletIndex;
    // TreeletId rightSplitTreeletIndex;
    
    friend std::ostream &operator<< (std::ostream &stream,const Treelet &treelet);
    public:
        static bool needToComputeWildcardOcc()
        {
            return true;
        }

        static bool isUsingWildcard()
        {
            return true;
        }
    
        class TreeletSimpleIndexer  //TODO: check if more complex version is necessary
            {
            std::vector<Treelet> treeletList;
            //map<Treelet,TreeletId> treeletToId;
            friend std::ostream &operator<< (std::ostream &stream,const TreeletSimpleIndexer &treeletIdx);
            public:
                TreeletId addTreeletAndGetId(const Treelet &treelet)
                    {
                    treeletList.push_back(treelet);
                    return treeletList.size()-1;
                    };
                    
                Treelet &getTreelet(TreeletId treeletIndex) const
                    {
                    assert (treeletIndex!=TREELET_ID_SPECIAL_VALUE);
                    const Treelet& result=treeletList.at(treeletIndex);
                    // const_cast so that returned treelet can be modified, while still enforcing the fact that no treelet is added to treeletList
                    // TODO: check if necessary...
                    return const_cast<Treelet &>(result);
                    };
            };
    
        class TreeletSizeInfos
            {
            map<TreeletId,pair<size_t,size_t> > sizeOfTreeletsMap;
            public:
            pair<size_t,size_t> getSizeOfTreelet(TreeletId id) const
                {
                map<TreeletId,pair<size_t,size_t> >::const_iterator it=sizeOfTreeletsMap.find(id);
                if (it==sizeOfTreeletsMap.end() )
                    {
                    throw TreeletBase::DescendingSuperTreeletsNotFoundException();
                    }
                else
                    {
                    return it->second;
                    }
                }
             
            void addTreelet(TreeletId treeletId, const Treelet& treelet)
                {
                assert(sizeOfTreeletsMap.count( treeletId )==0);
                sizeOfTreeletsMap[treeletId]=make_pair(treelet.getSize(), treelet.getSizeNonWildcard());//.getSize();
                }
            };
    
    
        class TreeletStoreByChildrenIndices: public boost::noncopyable
            {
            map< vector<TreeletId>,TreeletId > mapTreeletsToId;
            map< vector<TreeletId>,TreeletId > mapTreeletsToIdWildcardRoot;
            
            private:
            TreeletId _getLeftTreeletId(const Treelet& treelet,int index,
                const map< vector<TreeletId>,TreeletId >& mapTreeletsToId) const
                {
                assert (treelet.checkTreeletHasAllTopChildrenEmptyAfterIndex(index));
                vector<TreeletId> childVector=treelet.getChildrenIndices();
                childVector[index]=TREELET_ID_SPECIAL_VALUE;
                    
                map< vector<TreeletId>,TreeletId >::const_iterator it1=mapTreeletsToId.find(childVector);
                if (it1==mapTreeletsToId.end() )
                    {
                    throw logic_error("bug");
                    }

                TreeletId leftTreeletId=it1->second;
                return leftTreeletId;
                
                }
            
            
            public:
            
            void addTreelet(const Treelet& treelet,TreeletId idTreelet)
                {
                if (treelet.isRootWildcard())
                    {
                    assert (mapTreeletsToIdWildcardRoot.count(treelet.getChildrenIndices())==0);
                    mapTreeletsToIdWildcardRoot[treelet.getChildrenIndices()]=idTreelet;
                    }
                else
                    {
                    //std::cerr<<treelet<<std::endl;
                    //std::cerr<<mapTreeletsToId<<std::endl;
                    assert (mapTreeletsToId.count(treelet.getChildrenIndices())==0);
                    mapTreeletsToId[treelet.getChildrenIndices()]=idTreelet;
                    }
                }
                
            TreeletId getLeftTreeletId(const Treelet& treelet,int index) const
                {
                if (treelet.isRootWildcard())
                    {
                    return _getLeftTreeletId(treelet,index,mapTreeletsToIdWildcardRoot);
                    }
                else
                    {
                    return _getLeftTreeletId(treelet,index,mapTreeletsToId);
                    }
                }
                

            };

        class CompareTreeletsBySize {
            public:
                bool operator()(const Treelet& t1, const Treelet& t2) // Returns true if t1 is larger than t2
                {  if (t1.getSize()==t2.getSize())
                    {
                    return t1.getSizeNonWildcard()>t2.getSizeNonWildcard();
                    }
                   else
                    {
                    return t1.getSize()>t2.getSize();  // or the opposite?
                    }
                }   
            };
    
        //General constructor
        Treelet(const Node *pRootNodeInQueryTree,std::vector<TreeletId> childrenIndices,size_t size,size_t sizeNonWildcard,std::vector<size_t> sizeOfChildrenVector,std::vector<size_t> sizeNonWildcardOfChildrenVector,
        bool isRootWildcard)
                        :
                        TreeletBaseClass(size, false),
                        //size(size),
                        
                        recursiveRepr(pRootNodeInQueryTree,childrenIndices,isRootWildcard),
                        sizeNonWildcard(sizeNonWildcard),
                        sizeOfChildrenVector(sizeOfChildrenVector),
                        sizeNonWildcardOfChildrenVector(sizeNonWildcardOfChildrenVector)
                        //,seqOfPosIsConsistent(false),treeletIdIsSet(false),nbSentOccIsSet(false),isElementaryTreeletFlag(false),
                        //wasGeneratedAsDescendingSuperTreeletFlag(false),isTypeRepresentationComputed(false)
                        {
                        assert(size>=sizeNonWildcard);
                        assert(sizeOfChildrenVector.size()==sizeNonWildcardOfChildrenVector.size());
                        for (unsigned int i=0;i<sizeOfChildrenVector.size();i++)
                            {
                            assert(sizeNonWildcardOfChildrenVector[i]<=sizeOfChildrenVector[i]);
                            }
                        }; 

        Treelet clone() const
            {
            return Treelet(recursiveRepr.getRootPointer(),recursiveRepr.getChildrenIndices(),size,sizeNonWildcard,sizeOfChildrenVector,sizeNonWildcardOfChildrenVector,recursiveRepr.checkIfRootWildcard());
            };
                        
        // Constructor for elementary treelets    
        Treelet(const Node *pRootNodeInQueryTree,bool isRootWildcard=true)
                        :
                        TreeletBaseClass(1, true),
                        //size(1),
                        
                        recursiveRepr(pRootNodeInQueryTree,
                        pRootNodeInQueryTree->getNbOfChildren(),isRootWildcard), 
                        sizeNonWildcard(isRootWildcard?0:1),
                        sizeOfChildrenVector(pRootNodeInQueryTree->getNbOfChildren(),0),
                        sizeNonWildcardOfChildrenVector(pRootNodeInQueryTree->getNbOfChildren(),0)
                        //,seqOfPosIsConsistent(false),treeletIdIsSet(false),nbSentOccIsSet(false),isElementaryTreeletFlag(true),
                        //wasGeneratedAsDescendingSuperTreeletFlag(false),isTypeRepresentationComputed(false)
                        {};

        // Constructor for treelet with only one child at the root    
        Treelet(const Node *pRootNodeInQueryTree, size_t nbChildren,size_t numChild, TreeletId uniqueChildId, size_t sizeOfUniqueChild, size_t sizeNonWildcardOfUniqueChild,bool isRootWildcard)
                        :
                        TreeletBaseClass(1, false),
                        //size(1),
                        
                        recursiveRepr(pRootNodeInQueryTree,nbChildren,isRootWildcard),
                        sizeNonWildcard(isRootWildcard?0:1),
                        sizeOfChildrenVector(nbChildren,0),
                        sizeNonWildcardOfChildrenVector(nbChildren,0)
                        //,seqOfPosIsConsistent(false),treeletIdIsSet(false),nbSentOccIsSet(false),isElementaryTreeletFlag(false),
                        //wasGeneratedAsDescendingSuperTreeletFlag(false),isTypeRepresentationComputed(false)
            {
            assert(sizeOfUniqueChild>=sizeNonWildcardOfUniqueChild);
            extendChildrenDownwardInPlace(numChild,uniqueChildId,sizeOfUniqueChild,sizeNonWildcardOfUniqueChild);
            };
    
/*    
    void resetTypeRepresentation()
        {
        typeRepresentation="";
        isTypeRepresentationComputed=false;
        };
*/    
    void computeTypeRepresentation(const TreeletSimpleIndexer& tsi)
        {
        std::ostringstream typeAsStream;
        
        if (recursiveRepr.checkIfRootWildcard())
            {
            typeAsStream<<"*";
            }
        else
            {
            typeAsStream<<getRootLabel();
            }
        typeAsStream<<"(";
        const vector<TreeletId>& childrenIndices=recursiveRepr.getChildrenIndices();
        for (unsigned int i=0;i<childrenIndices.size();i++)
            {
            if (i==recursiveRepr.getNbChildrenLeft())
                {
                typeAsStream<<"| ";
                }
            TreeletId childId=childrenIndices[i];

            if (childId==TREELET_ID_SPECIAL_VALUE)
                {
                continue;
                }
            typeAsStream<<tsi.getTreelet(childId).getTypeRepresentation()<<" ";
            }
        typeAsStream<<")";
        typeRepresentation=typeAsStream.str();
        isTypeRepresentationComputed=true;
        };

/*        
    std::string getTypeRepresentation() const
        {
        if (!isTypeRepresentationComputed)
            {
            throw std::logic_error("type representation not computed");
            }
        return typeRepresentation;
        };
*/    
        bool isElementaryWildcard() const
            {
            return isElementaryTreelet() && isRootWildcard();
            }

        bool isWildcardElemPair(const TreeletSimpleIndexer& tsi,
                                bool& isChildOnLeft, LabelType& childLabel) const {
            if (!isRootWildcard()) {
                return false;                
            }
            // if (getNbTopSiblings() == 1) {
            //     return false;
            // }
            TreeletId singleChildId = recursiveRepr.getTopSibling(isChildOnLeft);
            if (singleChildId == TREELET_ID_SPECIAL_VALUE) {
                return false;
            }
            const Treelet& childTreelet = tsi.getTreelet(singleChildId);
            if (!childTreelet.isElementaryTreelet()) {
                return false;
            }
            childLabel = childTreelet.getRootLabel();
            return true;
        }


        bool hasGaps() const
            {
            assert(size>=sizeNonWildcard);
            return size!=sizeNonWildcard;
            }
            
       int nbGaps() const
            {
            assert(size>=sizeNonWildcard);
            return size-sizeNonWildcard;
            }
        
        bool hasTopWildcardChain() const
            {
            return recursiveRepr.checkIfRootWildcard() && (recursiveRepr.getNbTopSiblings()==1);
            }
        
        // bool rootIsWildcardAndHasDiscontiguousSiblings() const {
            // return recursiveRepr.checkIfRootWildcard() && !recursiveRepr.checkIfChildrenContiguous();
        // }
        
        bool isDescendingSupertreesRoot() const
            {
            return isElementaryWildcard();
            }
        bool isRootWildcard() const
            {
            return recursiveRepr.checkIfRootWildcard();
            }
            
        bool hasContiguousChildrenBeforeIndex(int index) const {
            return recursiveRepr.hasContiguousChildrenBeforeIndex(index);
        }
    
        bool hasStrictlyMoreThanNTopSiblings(int n) const
            {
            return recursiveRepr.hasStrictlyMoreThanNTopSiblings(n);
            }
            
        const vector<TreeletId>& getChildrenIndices() const
            {
            return recursiveRepr.getChildrenIndices();
            }
            
        const SeqOfPosRepresentation& getSeqOfPosRepresentation() const
            {
            if (!seqOfPosIsConsistent)
                {
                std::cerr<<"seq of pos is not consistent"<<std::endl;
                throw TreeletBase::UsingNonSetValueInTreeletException();
                }
            return seqOfPos;
            }
            
    
    void buildSeqOfPos	(	const TreeletSimpleIndexer & 	treeletIndexer	)	
        {
        //std::cout<<"building seqOfPos for "<<*this<<std::endl;
        SeqOfPosRepresentation seqOfPos(recursiveRepr.getRootPointer()->getPosition(),POSTYPE_SPECIAL_VALUE,recursiveRepr.checkIfRootWildcard());//X

        if ( recursiveRepr.getRootPointer()->hasParent() )
            {
            seqOfPos.addParent( recursiveRepr.getParentNode()->getPosition() );
            }
        BOOST_FOREACH(TreeletId treeletId, recursiveRepr.getChildrenIndices())
            {
            if (treeletId==TREELET_ID_SPECIAL_VALUE) continue;
            const SeqOfPosRepresentation& seqOfPosChild= treeletIndexer.getTreelet(treeletId).getSeqOfPosRepresentation();
            seqOfPos.mergeWith(seqOfPosChild);
            }
        this->seqOfPos=seqOfPos;
        seqOfPosIsConsistent=true;
        }        
//        LabelType getRootLabel() const
//            {
//            return recursiveRepr.getRootLabel();
 //           }

 /*
        bool isElementaryTreelet() const
            {
            return isElementaryTreeletFlag;
            }

        void setGeneratedAsDescendingSuperTreeletFlag(bool b)
            {
            wasGeneratedAsDescendingSuperTreeletFlag=b;
            }
            
        bool wasGeneratedAsDescendingSuperTreelet() const
            {
            return wasGeneratedAsDescendingSuperTreeletFlag;
            }
*/        
        void extendChildrenDownwardInPlace(size_t childNumber, TreeletId newChildId, size_t sizeOfNewChild,size_t sizeNonWildcardOfNewChild)
            {
            //std::cout<<"extending downward "<<*this<<" to ";
            //Treelet result(*this);
            assert( (int)size - sizeOfChildrenVector[childNumber] +  sizeOfNewChild >=0 );
            size=size - sizeOfChildrenVector[childNumber] +  sizeOfNewChild ;
            sizeOfChildrenVector[childNumber]=sizeOfNewChild;
            
            sizeNonWildcard=(int)sizeNonWildcard- sizeNonWildcardOfChildrenVector[childNumber] +sizeNonWildcardOfNewChild;
            //TO DO: check here: updating size vertices??
            sizeOfChildrenVector[childNumber]=sizeOfNewChild;
            sizeNonWildcardOfChildrenVector[childNumber]=sizeNonWildcardOfNewChild;
            
            recursiveRepr.replaceChild(childNumber,newChildId);
            
            seqOfPosIsConsistent=false;
            nbSentOccIsSet=false;
            treeletIdIsSet=false;
            nbSentOcc=0;
            index=0;
            isElementaryTreeletFlag=false;
            //std::cout<<*this<<std::endl;
            }
       
       Treelet getExtendedDownwardTreelet(size_t childNumber, TreeletId newChildId, const TreeletSizeInfos &treeletSizeInfos) const
            {
            Treelet result=this->clone();
            pair<size_t,size_t> sizePair =treeletSizeInfos.getSizeOfTreelet(newChildId);
            size_t sizeOfNewChild=sizePair.first;
            size_t sizeNonWildcardOfNewChild=sizePair.second;
            result.extendChildrenDownwardInPlace(childNumber, newChildId, sizeOfNewChild,sizeNonWildcardOfNewChild);
            return result;
            }
       
       Treelet getExtendedUpwardTreelet() const
            {
            const Node *pParentNode =recursiveRepr.getParentNode();
            size_t nbChildrenOfParent=pParentNode->getNbOfChildren();
            const Node *thisNode=recursiveRepr.getRootPointer();
            size_t posInParent=thisNode->getPosInParent();
            //std::cout<<"extending upward:"<< *this << " id:"<< getId()<<" size:"<<size<<" posInParent:"<<posInParent<<std::endl;
            return Treelet(pParentNode, nbChildrenOfParent,posInParent, getId(), size , sizeNonWildcard,true);// TO DO: check whether we should generate onl wildcard parent ...
            }
       
       
        bool hasSpecialSuperTreelets(int index) const
            {
            assert(index==-1);
            if (size==1 && sizeNonWildcard==0) return true;
            return false;
            };
       
        void addSpecialSupertreelets(vector<Treelet>& result) const
            {
            assert (sizeNonWildcard==0 && size==1);
            //std::cerr<<"adding special supertreelet"<<std::endl;
            Treelet groundedElementaryTreelet=Treelet(recursiveRepr.getRootPointer(),false);
            //this->clone();
            
            //groundedElementaryTreelet.recursiveRepr.setRootWildcard(false);
            //groundedElementaryTreelet.sizeNonWildcard+=1;
            //groundedElementaryTreelet.seqOfPosIsConsistent=false;
            groundedElementaryTreelet.setGeneratedAsDescendingSuperTreeletFlag(true);
            result.push_back(groundedElementaryTreelet);
            };
       
        bool hasUpwardExtension() const
            {
            return recursiveRepr.hasUpwardExtension();
            };
/*       
        inline size_t getSize() const
            {
            return size;
            };
*/        
        inline size_t getSizeNonWildcard() const
            {
            return sizeNonWildcard;
            };
        
        inline bool operator<(const Treelet &other) const
            {
            return recursiveRepr<other.recursiveRepr;
            };
/*        
        TreeletId getId() const
            {
            if (!treeletIdIsSet)
                {
                std::cerr<<"id not defined"<<std::endl;
                throw TreeletBase::UsingNonSetValueInTreeletException();
                }
            return index;
            }
         
         void setId(TreeletId id)
            {
            index=id;
            treeletIdIsSet=true;
            }
*/         
         size_t getNbOfChildren() const
            {
            return recursiveRepr.getNbOfChildren();
            }
            
        TreeletId getChildIndex(size_t num) const
            {
            return recursiveRepr.getChildIndex(num);
            }
        PosType getPosNodeAtChildNum(size_t num) const
            {
            return recursiveRepr.getPosNodeAtChildNum(num);
            }
            
        
        bool safeToExtendTopSiblings(int safety) const
            {
            assert(false); //should not be needed now ...
            int nbTopSiblings=recursiveRepr.getNbTopSiblings();
            int nodeWidth=recursiveRepr.getNodeWidth();
            return nbTopSiblings<=TreeletBase::getSafeNbSiblingsForGivenNodeWidth(nodeWidth,safety);
            }
        
        const Node *getRootPointer() const
            {
            return recursiveRepr.getRootPointer();
            }
            
        PosType getRootPos() const
            {
            return getRootPointer()->getPosition();
            }
/*            
        size_t getNbSentOcc() const
            {
            if (!nbSentOccIsSet )
                {
                std::cerr<<"nbSentOcc not defined"<<std::endl;
                throw TreeletBase::UsingNonSetValueInTreeletException();
                }
            return nbSentOcc;
            }
            
        void setNbSentOcc(size_t nbOcc)
            {
            nbSentOccIsSet=true;
            nbSentOcc=nbOcc;
            }
*/        
        LabelType getRootLabel() const
            {
            return recursiveRepr.getRootLabel();
            }
            
        pair<std::vector<PosType> ,std::vector<PosType> > separateWildcardFromNWInMapp(const std::vector<PosType>& mapp) const
            {
            return seqOfPos.separateWildcardFromNWInMapp(mapp);
            }
            
    bool checkTreeletHasAllTopChildrenEmptyAfterIndex(int index) const
        {
        assert(index<(int)getNbOfChildren());
        assert(index>=-1);
        for (int i=index+1;i<(int)getNbOfChildren();i++)
            {
            if (getChildIndex(i)!=TREELET_ID_SPECIAL_VALUE)
                {
                return false;
                }
            }
        return true;
        }
    };




    

    
std::ostream &operator<< (std::ostream &stream,const Treelet &treelet)
  {

  stream<<treelet.recursiveRepr<<"(s:"<<treelet.size;
  stream<<"/"<<treelet.sizeNonWildcard;
  if (treelet.treeletIdIsSet) stream<<" #"<<treelet.getId()<<"#";
  if (treelet.nbSentOccIsSet) stream<<" |"<<treelet.getNbSentOcc()<<"|";
  stream<<")";
  if (treelet.seqOfPosIsConsistent) stream<<treelet.getSeqOfPosRepresentation();
  if (treelet.isTypeRepresentationComputed) stream<<"'"<<treelet.typeRepresentation<<"'";
  return stream;
  }
//int main(){ return 0; }
   
std::ostream &operator<< (std::ostream &stream,const Treelet::TreeletSimpleIndexer &treeletIdx)
  {
  for(TreeletId idx=0;idx< treeletIdx.treeletList.size();++idx)
    {
    stream<<idx<<" : "<<treeletIdx.treeletList[idx]<<"\n";
    }
  return stream;
  }
    
}; //namespace TreeletWildcard
#endif

