/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup global_glue_group
    \brief Some definitions that should be shared by all components.
*/

#include <vector>
#include <set>
#include <map>
#include <boost/foreach.hpp>
#include <limits>

#ifndef _HPP_TYPE_DEFINITIONS_
#define _HPP_TYPE_DEFINITIONS_

//namespace baseDefinitions
//{

typedef unsigned int LabelType;
typedef unsigned char PosType;
#define POSTYPE_SPECIAL_VALUE 255
#define POSTYPE_MAXIMAL_VALUE  (std::numeric_limits<PosType>::max())

template<typename T>
bool isValidPosTypeValue(T v)
    {
    return (v>=std::numeric_limits<PosType>::min() && v<=std::numeric_limits<PosType>::max());
    }
    
template<typename T>
bool isValidNonSpecialPosTypeValue(T v)
    {
    return (v>=std::numeric_limits<PosType>::min() && v<=std::numeric_limits<PosType>::max() && v!=POSTYPE_SPECIAL_VALUE );
    }
    
typedef size_t TreeletId;
#define TREELET_ID_SPECIAL_VALUE 2147483647  //Necessary?? check again...




typedef size_t SlotType;

//class TreeletId
//    {
//    int id;
//    static TreeletId specialId(-1);
//    }

// for MergeConstraints
struct MapSlotPair{
    char numMapping;
    SlotType slot;
    MapSlotPair(char numMapping,SlotType slot):numMapping(numMapping),slot(slot){};
    };

std::ostream &operator<< (std::ostream &stream,const MapSlotPair &msp)
  {
 
  stream<<(int)msp.numMapping<<":"<<msp.slot;
  return stream;
  }

typedef std::pair<SlotType,SlotType> SlotPairType;
typedef std::pair<std::vector<SlotType>,std::vector<SlotType> > SlotListPairType;

// for binary merge
#define RATIO_BINARY_MERGE 32

typedef int TreeIdType;

namespace streamOperatorsUtils
{

// for compatibility with Clang
template<typename T>
std::ostream &operator<< (std::ostream &stream,const std::vector<T> &vect);
template<>
std::ostream &operator<< <unsigned char>(std::ostream &stream,const std::vector<unsigned char> &vect);


template<typename T1,typename T2>
std::ostream &operator<< (std::ostream &stream,const std::pair<T1,T2> &p)
  {
  stream<<"p("<<p.first<<","<<p.second<<")"<<std::endl;
  return stream;
  }

template<typename T>
std::ostream &operator<< (std::ostream &stream,const std::vector<T> &vect)
  {

  stream<<"Vect("<<vect.size()<<"/"<<vect.capacity()<<")<";
  for (unsigned int i=0;i<vect.size();++i)
    {
    stream<<vect[i]<<" ";
    }

  stream<<">";

  return stream;
  }

template<>
std::ostream &operator<< <unsigned char>(std::ostream &stream,const std::vector<unsigned char> &vect)
  {

  stream<<"Vect("<<vect.size()<<"/"<<vect.capacity()<<")<";
  for (unsigned int i=0;i<vect.size();++i)
    {
    stream<<(int)vect[i]<<" ";
    }

  stream<<">";

  return stream;
  }




template<typename T>
std::ostream &operator<< (std::ostream &stream,const std::set<T> &s)
  {

  stream<<"Set(";
  BOOST_FOREACH(const T& elem,s)
    {
    stream<<elem<<",";
    }
  stream<<")";
  return stream;
  }

template<>
std::ostream &operator<< <unsigned char>(std::ostream &stream,const std::set<unsigned char> &s)
  {

  stream<<"Set(";
  BOOST_FOREACH(unsigned char elem,s)
    {
    stream<<(int)elem<<",";
    }
  stream<<")";
  return stream;
  }



template<typename T1,typename T2>
std::ostream &operator<< (std::ostream &stream,const std::map<T1,T2> &s)
  {
  stream<<"Map(";
  typedef std::pair<T1,T2> KeyValPair;
  BOOST_FOREACH(const KeyValPair& keyval,s)
    {
    stream<<"("<<keyval.first <<":" << keyval.second<<")\n";
    }
  stream<<")";
  return stream;
  }

};//namespace streamOperatorsUtils

using namespace streamOperatorsUtils;

//}; //namespace baseDefinitions

//using namespace baseDefinitions;

#endif
