/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup occurences_merging_group
    \brief Describes structure of occurence lists.
*/

//@+leo-ver=4-thin
//@+node:fabien.20110805110723.2556:@thin binaryMerge.hpp
#include <iostream>
#include<vector>
#include <algorithm>

#include "TypeDefinitions.hpp"
#include "ArrayOfPairs.hpp"

//@@language c++

#ifndef BINARYMERGE_HPP
#define BINARYMERGE_HPP

#include <boost/foreach.hpp>
// #include <boost/range/adaptors.hpp>  //boost>=1.43

#include<cassert>
#include<ctime>


#include <fstream>
#include <boost/serialization/vector.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>

//using ext::hash;  // or 

// using namespace __gnu_cxx;
//using namespace boost;
//using namespace std;
//using namespace ext;
//using __gnu_cxx::hash; //__gnu_cxx::hash; //, or maybe tr1::hash, depending on your OS

using std::vector;

/*****************************************************************
 *  WARNING: in the OccList, the no-parent position is indicated by the position just after the end of the string, not by POSTYPE_SPECIAL_VALUE (dangerous: need to change)
 ******************************************************************/
 
 
template<typename TypeOfMappingList>
struct GenericMappingsPerTree{
    public:
    TreeIdType numTree;
    TypeOfMappingList mappingList;

    private:
    friend class boost::serialization::access;
    template<class Archive> void serialize(Archive & ar, const unsigned int version)
        {
            // serialize base class information
            ar & numTree;
            ar & mappingList;
        }

    public:
    template<typename OtherType>
    bool operator<(const GenericMappingsPerTree<OtherType>& other) const
        {
        return numTree<other.numTree;
        }
    /*
    ~GenericMappingsPerTree()
        {
        std::cout<<"destructing GenericMappingsPerTree (numtree:"<<numTree<<std::endl;
        }    
    */    
    };
    
template<typename T>
std::ostream &operator<< (std::ostream &stream,const GenericMappingsPerTree<T> &gmp)
  {

  stream<<"Match("<<gmp.numTree<<"|"<<gmp.mappingList<<")";
  return stream;
  }

typedef vector<PosType> Mapping; 
typedef GenericMappingsPerTree< vector<Mapping> > MappingsPerTree;
typedef GenericMappingsPerTree< ArrayOfPairs<PosType> > MappingsPerTreeCompactPair;


typedef vector<MappingsPerTree> OccList;
typedef vector<MappingsPerTreeCompactPair> OccListCompactPair;
/*
class GenericOccListInterface
    {
    public:
    //typedef boost::shared_ptr<GenericOccListInterface> Ptr;
    virtual GenericOccListInterface *mergeWith(const GenericMatchingsListContainer& other, const MergeConstraints& constraints) const =0;
    virtual GenericOccListInterface *beMergedWith(const GenericOccListInterface& other, const MergeConstraints& constraints) const =0;
    //virtual Ptr beMergedWith(const CompactPairMatchingsListContainer& other, const MergeConstraints& constraints) const =0;
    virtual size_t getNbSentOcc() const =0;
    };
    
template<ListType>  
class OccListTemplate: public vector<MappingsPerTree>,public GenericOccListInterface
    {
    template<Container>
    GenericOccListInterface *beMergedWith(const Container& other, const MergeConstraints& constraints) const
        {
        merge<Container, vector<MappingsPerTree> >  (other,*this,constraints);
        }
    
    GenericOccListInterface * mergeWith(const GenericMatchingsListContainer& other, const MergeConstraints& constraints) const
        {
        other.beMergedWith(*this,constraints);
        
        }
    
    size_t getNbSentOcc() const
        {
        return size();
        }
    }
*/


typedef OccList::iterator OccListIterator;

vector<vector<MappingsPerTreeCompactPair> > *loadVectorOccListCompactPair(const char *filename);

template<typename GenericOccList1,typename GenericOccList2>
OccList *intersectAndMerge(const GenericOccList1 &set1, const GenericOccList2 &set2,
                    const vector<SlotPairType>& equal,
                    const vector<SlotListPairType>& diff,
                    const vector<MapSlotPair>& orderRes,
                    bool preserveOrder);

#endif
//@nonl
//@-node:fabien.20110805110723.2556:@thin binaryMerge.hpp
//@-leo
