/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "PostProcessor/en.hpp"
#include <glog/logging.h>
#include <boost/regex.hpp>
#include <string>
#include <iostream>
#include "utils.hpp"

std::string PostProcessorTargetEn::post_process(const std::string &original) const {
    std::string result = original;
    // Replace multiple spaces with a single space.
    result = boost::regex_replace(result, boost::regex("\\s+"), " ");
    // Strip whitespace and punctuation from the start of the sentence.
    result = boost::regex_replace(result, boost::regex("^[\\s\\.\\,\\?\\!]*"), "");
    // Strip whitespace and punctuation other than ? and ! from the end.
    result = boost::regex_replace(result, boost::regex("[\\s\\.\\,]*$"), "");
    // Ensure sentence ends in a period, unless it already ends in ? or !.
    if (result[result.size() - 1] != '?' && result[result.size() - 1] != '!') {
        result += " .";
    }
    // Replace consecutive punctuation within sentence with a comma.
    result = boost::regex_replace(result, boost::regex("([,;:\\.]\\s*)+[,;:\\.]"), ",");
    // Replace bracket tags with real brackets.
    convert_bracket_symbols(result);
    // Capitalize first letter.
    result[0] = toupper(result[0]);
    // TODO(John): Automatically add closing brackets and inverted commas.
    LOG(INFO) << "PostProcessor (en): input=" << original << " result=" << result;
    return result;
}
