/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <glog/logging.h>
#include <string>
#include "PostProcessor/ja.hpp"
#include "utils.hpp"

std::string PostProcessorTargetJa::post_process(const std::string &original) const {
    std::string result = original;
    // Replace multiple spaces with a single space.
    result = safe_regex_replace(result, "\\s+", " ");
    // Strip whitespace and punctuation from the start of the sentence.
    result = safe_regex_replace(result, "^[\\s。、，\\.\\,?!]*", "");
    // Strip whitespace and punctuation from the end.
    result = safe_regex_replace(result, "[\\s。、，\\.\\,]*$", "");
    // Ensure sentence ends in a period.
    result += " 。";
    // Replace consecutive punctuation within sentence with a comma.
    result = safe_regex_replace(result, "([,;:\\.、，；：。]\\s*)+[,;:\\.、，；：。]", "、");
    LOG(INFO) << "PostProcessor (ja): input=" << original << " result=" << result;
    return result;
}


