/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! @file
  This file contains TM processing functions for English.
*/
#ifndef TM_PROCESSOR_EN_INCLUDED
#define TM_PROCESSOR_EN_INCLUDED

#include <string>
#include "source.hpp"
#include "target.hpp"

class TMProcessorSourceEn : public TMProcessorSource {
  public:
    virtual ~TMProcessorSourceEn() {}
    
    virtual bool is_child_bond(const TM& tm, const DtreeNode *i_child, const DtreeNode *s_child) const;
    virtual std::string get_abst_word_pos(const DtreeNode *w) const;
    virtual std::string get_abst_cat(const DtreeNode *w) const;
    virtual double check_tm_root(DataForTM* data_for_tm) const;
    virtual double check_parent_bond(const DtreeNode *i_parent, const DtreeNode *s_parent) const;
    virtual double calc_word_similarity(const DtreeNode *w1, const DtreeNode *w2) const;
    //! check if the string is a punctuation.
    virtual bool is_punctuation(std::string p) const {
        return (p == "." || p == "?" || p == "!" || p == ",");
    }
};

class TMProcessorTargetEn : public TMProcessorTarget {
};

#endif
