/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/regex.hpp>
#include <glog/logging.h>
#include <string>
#include "ja.hpp"

bool TMProcessorSourceJa::is_child_bond(const TM& tm, const DtreeNode *i_child, const DtreeNode *s_child) const {
    if (TMProcessorSource::is_child_bond(tm, i_child, s_child) &&
        // typeの制約
        ((i_child->word.type != "基本句内" && i_child->word.type == s_child->word.type) ||
         (i_child->word.type == "基本句内" && s_child->word.type == "基本句内" &&
          get_abst_word_pos(i_child) == get_abst_word_pos(s_child)) ||
         ((i_child->word.type == "ヲ格" || i_child->word.type == "隣") &&
          (s_child->word.type == "ヲ格" || s_child->word.type == "隣")) ||
         ((i_child->word.type == "ニ格" || i_child->word.type == "ヘ格") &&
          (s_child->word.type == "ニ格" || s_child->word.type == "ヘ格")))
        ) {
        return true;
    }
    return false;
}

std::string TMProcessorSourceJa::get_abst_word_pos(const DtreeNode *w) const {
    boost::regex pos_regex("(.+?):(.+)");
    boost::smatch pos_results;
    // boost::regex conj_regex("語幹$");
    // boost::smatch conj_results;
    if (boost::regex_match(w->word.pos, pos_results, pos_regex)) {
        if (pos_results.str(1) == "接尾辞") {
            boost::regex suffix_regex("^(.+?)性");
            boost::smatch suffix_results;
            if (boost::regex_search(pos_results.str(2), suffix_results, suffix_regex)) {
                return suffix_results.str(1);
            } else {
                LOG(WARNING) << "接尾辞 ERROR";
                return "接尾辞";
            }
        } else if (pos_results.str(1) == "特殊") {
            return "名詞";
        } else {
            return pos_results.str(1);
        }
    } else {
        return w->word.pos;
    }
    // else if (w.pos == "形容詞" &&
    //          boost::regex_search(w->conj, conj_results, conj_regex)) {
    //     return "名詞";
    // }
}

double TMProcessorSourceJa::check_tm_root(DataForTM* data_for_tm) const {
    double result = 0.0;
    const boost::regex eos_regex("^文末");
    boost::smatch eos_results;
    // catが一致しなければ減点
    if (data_for_tm->i_root->word.cat != data_for_tm->s_root->word.cat) {
        result -= 0.3;
    }
    // typeが一致しなければ減点
    if ((data_for_tm->i_root->word.type != "基本句内" && data_for_tm->i_root->word.type != data_for_tm->s_root->word.type) ||
        (data_for_tm->i_root->word.type == "基本句内" && data_for_tm->s_root->word.type == "基本句内" &&
         get_abst_word_pos(data_for_tm->i_root) != get_abst_word_pos(data_for_tm->s_root))) {
        if (boost::regex_search(data_for_tm->i_root->word.type, eos_results, eos_regex) || 
                boost::regex_search(data_for_tm->s_root->word.type, eos_results, eos_regex)) {
            result -= 0.3;
        } else {
            result -= 0.1;
        }
    }
    return result;
}
        
double TMProcessorSourceJa::calc_word_similarity(const DtreeNode *w1, const DtreeNode *w2) const {
    if (w1->word.lem == w2->word.lem || (w1->word.pos == "名詞:数詞" && w2->word.pos == "名詞:数詞")) {
        return 1.0;
    } else {
        return 0.0;   
    }
}
