/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/regex.hpp>
#include <map>
#include <string>
#include <utility>
#include "ja_en.hpp"

std::string TMProcessorPairJaEn::num_translation(const std::string& content, const std::string& num, const std::string& type) const {
    static std::map<std::string, std::string> except_trans;
    std::string new_content = content;
    if (except_trans.size() == 0) {
        except_trans.insert(std::pair<std::string, std::string>("1", "first"));
        except_trans.insert(std::pair<std::string, std::string>("2", "second"));
        except_trans.insert(std::pair<std::string, std::string>("3", "third"));
        except_trans.insert(std::pair<std::string, std::string>("4", "fourth"));
        except_trans.insert(std::pair<std::string, std::string>("5", "fifth"));
        except_trans.insert(std::pair<std::string, std::string>("6", "sixth"));
        except_trans.insert(std::pair<std::string, std::string>("7", "seventh"));
        except_trans.insert(std::pair<std::string, std::string>("8", "eighth"));
        except_trans.insert(std::pair<std::string, std::string>("9", "ninth"));
    }

    if (type == "th") {
        if (except_trans.find(num) != except_trans.end()) {
            new_content = except_trans[num];
        } else if (num.rfind("1") == num.size()) {
            new_content = num+"st";
        } else if (num.rfind("2") == num.size()) {
            new_content = num+"nd";
        } else if (num.rfind("3") == num.size()) {
            new_content = num+"rd";
        } else {
            new_content = num+"th";
        }
    } else {
        const boost::regex reg("[\\d\\-\\.\\,]*\\d");
        std::string tmp = boost::regex_replace(content, reg, num);
        if (tmp == content) {
            // TODO(Nakazawa): Numeral Translation (eg. 3 -> three)
            // content = $trans_kit_trg->num2words("$num:no");
            new_content = num;
        } else {
            new_content = tmp;
        }
    }
    return new_content;
}

bool TMProcessorPairJaEn::is_skip_translation(const DtreeNode *w) const {
    return (w->word.lem == "この/この" ||
            w->word.lem == "その/その" ||
            w->word.lem == "あの/あの" ||
            w->word.lem == "ます/ます" ||
            (w->word.lem == "です/です" && w->word.pos.find("助動詞") != std::string::npos) ||
            w->word.lem == "御/お");
}

