/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <string>
#include "source.hpp"
#include "glog/logging.h"

bool TMProcessorSource::is_child_bond(const TM& tm, const DtreeNode *i_child, const DtreeNode *s_child) const {
    const Alignment* i_s_align = tm.get_alignment_i(i_child->dpnd_ref);
    if (i_s_align != NULL &&
        s_child->dpnd != -1 &&
        // 係り先の対応が正しいか
        grep(s_child->dpnd_ref->num, i_s_align->p2).size() > 0 &&
        // かかる方向が同じか
        ((i_child->dpnd_ref->num > i_child->num && s_child->dpnd_ref->num > s_child->num) ||
         (i_child->dpnd_ref->num < i_child->num && s_child->dpnd_ref->num < s_child->num))) {
        return true;
    }
    return false;
}

std::string TMProcessorSource::get_abst_word_pos(const DtreeNode *w) const {
    return w->word.pos;
}

double TMProcessorSource::check_tm_root(DataForTM* data_for_tm) const {
    return 0.0;
}

double TMProcessorSource::check_parent_bond(const DtreeNode *i_parent,
                                            const DtreeNode *s_parent) const {
    LOG(INFO) << "check parent bond: input(" << i_parent->word.cat << ") vs source(" << s_parent->word.cat << ")";
    double parent_bond_LD = 0.0;
    if (i_parent->word.cat != s_parent->word.cat) {
        parent_bond_LD = 1.0;
    }
    return parent_bond_LD;
}

std::string TMProcessorSource::get_abst_cat(const DtreeNode *w) const {
    return w->word.cat;
}

double TMProcessorSource::calc_word_similarity(const DtreeNode *w1, const DtreeNode *w2) const {
    if (w1->word.lem == w2->word.lem) {
        return 1.0;
    } else {
        return 0.0;   
    }
}
