/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/regex.hpp>
#include <glog/logging.h>
#include <string>
#include "zh.hpp"

bool TMProcessorSourceZh::is_child_bond(const TM& tm, const DtreeNode *i_child, const DtreeNode *s_child) const {
    if (TMProcessorSource::is_child_bond(tm, i_child, s_child) &&
        // catの制約
        (i_child->word.cat == s_child->word.cat ||
         (i_child->word.cat.find("_") == std::string::npos &&
          s_child->word.cat.find("_") == std::string::npos &&
          (((i_child->word.cat.find("N") == 0 || i_child->word.cat.find("F") == 0 || i_child->word.cat.find("DT") == 0) &&
            (s_child->word.cat.find("N") == 0 || s_child->word.cat.find("F") == 0 || s_child->word.cat.find("DT") == 0)) ||
           (i_child->word.cat.find("DE") == 0 && s_child->word.cat.find("DE") == 0)))) &&
        // 但是など、文頭と文中で意味が変わる
        (i_child->word.cat != "AD" ||
         ((i_child->num == 0 && s_child->num == 0) || (i_child->num != 0 && s_child->num != 0)))) {
        return true;
    }
    return false;
}

std::string TMProcessorSourceZh::get_abst_word_pos(const DtreeNode *w) const {
    if (w->word.pos.find("V") == 0) {
        return w->word.pos;
    } else {
        std::string result;
        result = w->word.pos.substr(0, 2);
        return result;
    }
}

std::string TMProcessorSourceZh::get_abst_cat(const DtreeNode *w) const {
    if (w->word.cat.find("N") == 0) {
        return "NP";
    } else if (w->word.cat.find("V") == 0) {
        return "VP";
    } else {
        return w->word.cat;
    }
}

double TMProcessorSourceZh::check_tm_root(DataForTM* data_for_tm) const {
    double result = 0.0;

    std::string i_cat = get_abst_cat(data_for_tm->i_root);
    std::string s_cat = get_abst_cat(data_for_tm->s_root);
    LOG(INFO) << "check tm root: i_cat(" << i_cat << ") vs s_cat(" << s_cat << ")";
    if (i_cat != s_cat) {
        result -= 0.2;
    }

    std::string i_content_pos = "";
    if (data_for_tm->i_root->word.content_p == 1) {
        i_content_pos = get_abst_word_pos(data_for_tm->i_root);
    }
    std::string s_content_pos = "";
    if (data_for_tm->s_root->word.content_p == 1) {
        s_content_pos = get_abst_word_pos(data_for_tm->s_root);
    }
    LOG(INFO) << "check tm root: i_content_pos(" << i_content_pos << ") vs s_content_pos(" << s_content_pos << ")";
    if (i_content_pos != s_content_pos) {
        result -= 0.2;
    }

    // ADは先頭同士じゃなければ減点
    if (data_for_tm->i_root->word.pos == "AD" && data_for_tm->s_root->word.pos == "AD" &&
        ((data_for_tm->i_root->num == 0 && data_for_tm->s_root->num != 0) ||
         (data_for_tm->i_root->num != 0 && data_for_tm->s_root->num == 0))) {
        LOG(INFO) << "check tm root: AD should be both at the top or both in the middle";
        result -= 0.3;
    }

    return result;
}
        
double TMProcessorSourceZh::calc_word_similarity(const DtreeNode *w1, const DtreeNode *w2) const {
    if (w1->word.lem == w2->word.lem || (w1->word.pos == "CD" && w2->word.pos == "CD")) {
        return 1.0;
    } else {
        return 0.0;   
    }
}

double TMProcessorSourceZh::check_parent_bond(const DtreeNode *i_parent,
                                              const DtreeNode *s_parent) const {
    double parent_bond_LD = 0.0;

    std::string i_content_pos = "";
    if (i_parent->word.content_p == 1) {
        i_content_pos = get_abst_word_pos(i_parent);
    }
    std::string s_content_pos = "";
    if (s_parent->word.content_p == 1) {
        s_content_pos = get_abst_word_pos(s_parent);
    }
    if (i_content_pos != s_content_pos) {
        parent_bond_LD = 1.0;
    }

    return parent_bond_LD;
}
