/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/foreach.hpp>
#include <string>
#include <vector>
#include "confidence_estimator.hpp"

double ConfidenceEstimator::estimate(const Dtree& input_dtree, const SingleNBestTranslationResult& tl_result, const std::vector<TMdata>& tm_list) const {
    int num_of_real_tm = 0;
    int size_of_real_input = 0;
    int num_of_words_covered_by_large_tm = 0;
    BOOST_FOREACH(TMdata tmp, tm_list) {
        if (tmp.first->id().find("OOV") == std::string::npos) {
            num_of_real_tm++;
            int tm_size = tmp.second->i_body_ref.size();
            size_of_real_input += tm_size;
            if (tm_size > 2) {
                num_of_words_covered_by_large_tm += tm_size;
            }
        }
    }
    double average_tm_size = static_cast<double>(size_of_real_input) / static_cast<double>(num_of_real_tm);
    double ratio_of_using_large_tm = static_cast<double>(num_of_words_covered_by_large_tm) / static_cast<double>(size_of_real_input);
    if (size_of_real_input <= 5) {
        return 3.0;
    } else if (ratio_of_using_large_tm >= 0.5 || average_tm_size > 3) {
        return 3.0;
    } else if (ratio_of_using_large_tm >= 0.3 || average_tm_size > 2) {
        return 2.0;
    }
    return 1.0;
}
