/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! \file
    \ingroup translation_core
    \brief Main code for finding optimal Example combinations.
*/

#ifndef __COMBI_SEARCH_HPP__
#define __COMBI_SEARCH_HPP__

#include <KenLM/lm/model.hh>
#include <map>
#include <set>
#include <string>
#include <vector>
#include "../def.hpp"

#include "../hypotheses_storage.hpp"
#include "augmented_pattern.hpp"
#include "table_display_output.hpp"

#include "translation_result.hpp"
#include "translation_log.hpp"

/********************************************************************************************************
 * Main call to the decoder.
 * Input:
 *   @params initial_patterns: a list of AugmentedPatterns associated with each pattern in trans_cand
 *   @params trans_cand: a TranslationCandidate object storing the initial hypotheses
 *   @params lm_rest: language model object, including rest costs
 *   @params weights: the weights to be used for the decoding
 *   @params translation_log: a log object (just for logging, has no effect on result)
 * Output:
 *   A NBestTranslationResult object
 */

NBestTranslationResult lattice_search(
        const std::vector<AugmentedPattern>& initial_patterns,
         InitialHypothesesStorage *trans_cand,
         const lm::ngram::RestProbingModel& lm_rest,
         const WeightVector& weights,
         t_log::TranslationLog* translation_log);

#endif // __COMBI_SEARCH_HPP__
