/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "free_edge.hpp"

namespace latt {

void print_parent_relationship_to_stream(std::ostream* stream, ParentRelationship pr) {
    if (pr != UNSET) {
        switch (pr) {
            case BEFORE:
                *stream <<"(X->P)";
                break;
            case AFTER:
                *stream << "(P<-X)";
                break;
            case UNKNOWN:
                *stream << "(?<-?)";
                break;
            default:
                assert(false);
        }
    }
}

std::ostream& operator<<(std::ostream& stream, const Bond& bond) {
    stream << "<BOND:";
    switch (bond.type_) {
        case Bond::REFERENCE:
            stream << "REF:" << bond.insertion_id();
            break;
        case Bond::ADDITIONAL:
            stream << "ADD:" << bond.insertion_id();
            break;
        case Bond::INVALID:
            stream << "INVALID";
            break;
        default:
            assert(false);
    }
    stream << ">";
    return stream;
}

std::ostream& operator<<(std::ostream& stream, const FreeEdge& edge) {
    switch (edge.type_) {
    case FreeEdge::WORD:
        stream << edge.word_ << "[" << edge.score() << "]:" << edge.idx_;
        break;
    case FreeEdge::EOS:
        stream << "<EOS>";
        break;
    case FreeEdge::EPSILON:
        stream << "<eps>";
        break;
    case FreeEdge::BOND:
        stream << "<BILMLATTICE>";
        break;
    default:
        assert(false);
    }
    print_parent_relationship_to_stream(&stream, edge.prefered_parent_relationship());
    return stream;
}

} // namespace latt


