/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "latt_state.hpp"

#include <cassert>

namespace latt {

std::ostream& operator<<(std::ostream& stream, const LatticeState& state) {
    stream << "S<" << state.v_id() << "||" << state.history() << ">";
    print_parent_relationship_to_stream(&stream, state.prefered_parent_relationship_);
    return stream;
}

StateFeatures StateUpdater::compute_state_transition_simple(const LatticeState& starting_state, const FreeEdge& free_edge,
                    LatticeState* new_state) const {
    double delta_lm;
    switch (free_edge.type()) {
    case FreeEdge::WORD:
    case FreeEdge::EOS:
        delta_lm = starting_state.history().set_next(lm_, free_edge.word_idx(), &(new_state->get_modifyable_history()));
        break;
    case FreeEdge::EPSILON:
        delta_lm = 0;
        new_state->get_modifyable_history() = starting_state.history();
        break;
    case FreeEdge::BOND:
        assert(false);
        delete new_state;
        delta_lm = generic::kBadScore;
        std::cerr << "Bug here" << std::endl;
        exit(1);
        break;
    default:
        assert(false);
        delta_lm = generic::kBadScore;
        std::cerr << "Bug here" << std::endl;
        delete new_state;
        exit(1);
        break;
    }

#ifdef USE_PR
    if (free_edge.prefered_parent_relationship() != UNSET) {
        assert(starting_state.prefered_parent_relationship() == UNSET);
        new_state->set_prefered_parent_relationship(free_edge.prefered_parent_relationship());
    } else {
        new_state->set_prefered_parent_relationship(starting_state.prefered_parent_relationship());
    }
#endif // USE_PR

    //create state features
    StateFeatures st_feat(delta_lm);
    return st_feat;
}

StateFeatures StateUpdater::compute_state_transition(const LatticeState& starting_state, const LatticeState& transition_state, const FreeEdge& free_edge,
                    LatticeState* new_state) const {
    double delta_lm = starting_state.history().set_next(lm_, transition_state.history(), &(new_state->get_modifyable_history()));
    new_state->set_prefered_parent_relationship(starting_state.prefered_parent_relationship());

    // create new state feature
    StateFeatures st_feat(delta_lm);

#ifdef USE_PR
    assert(transition_state.prefered_parent_relationship() != UNSET);
    if ((transition_state.prefered_parent_relationship() != UNKNOWN) &&
            (transition_state.prefered_parent_relationship() !=  free_edge.bond().parent_relationship())) {
        switch (free_edge.bond().type()) {
        case Bond::ADDITIONAL:
            st_feat.set_additional_PR_mismatch(1);
            break;
        case Bond::REFERENCE:
            st_feat.set_reference_PR_mismatch(1);
            break;
        default:
            assert(false);
            std::cerr << "bug here" << std::endl;
            exit(1);
            break;
        }
    }
#endif //USE_PR
    return st_feat;
}

} // namespace latt
