/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LM_HISTORY_HPP_
#define LM_HISTORY_HPP_

#include <iostream>

#include "lm/state.hh"
#include "lm/word_index.hh"
#include "base_lattice.hpp"

namespace latt {

class History {
  public:
    friend std::ostream& operator<<(std::ostream& stream, const History& hist);
    enum Type {BOS, INIT, UNSET, CHART};

    History();

    explicit History(bool starts_with_BOS);

    bool operator<(const History& other) const;

    double set_next(const LanguageModel& model, lm::WordIndex word_index, History* new_history) const;

    double set_next(const LanguageModel& model, const History& hist_edge, History* new_history) const;

    double eval_delta_with_BOS_EOS(const LanguageModel& model) const;

    double eval_delta_with_OOV(const LanguageModel& model) const;

  private:
    lm::ngram::ChartState chart_state_;
    Type type_;
    bool starts_with_BOS_;
    bool ends_with_EOS_;
};

}// namespace latt

#endif /* LM_HISTORY_HPP_ */
