/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef STATE_STORAGE_HPP_
#define STATE_STORAGE_HPP_

#include <boost/noncopyable.hpp>
#include <boost/ptr_container/ptr_set.hpp>
#include <boost/ptr_container/ptr_vector.hpp>

#include <cassert>
#include <iostream>
#include <string>
#include <vector>

#include "base_lattice.hpp"
#include "lattice_generic.hpp"
#include "lm_edge.hpp"
#include "translation_log.hpp"

namespace latt {

template<typename StateType>
class StateManager: boost::noncopyable {
  public:
    typedef boost::ptr_set<StateType> IterableStateContainer;
    explicit StateManager(size_t nb_original_vertice);

    StateType& add_or_get_identical_state(VertexId vertex, StateType* state);

    StateType* get_state_buffer(VertexId v_end_id) {
        assert(state_buffer_ == NULL);
        state_buffer_ = new StateType(v_end_id);
        return state_buffer_;
    }

//    template<typename StateType>
    StateType* add_buffer() {
        //std::cerr << "get_or_create_state " << vertex << " " << states_.size() << "\n";
//        assert(vertex < static_cast<int>(states_.size()));
        assert(state_buffer_ != NULL);
        IterableStateContainer& states_of_vertex = states_[state_buffer_->v_id()];

        typename IterableStateContainer::iterator it = states_of_vertex.insert(state_buffer_).first;

        state_buffer_ = NULL;

        return &(*it);
    }


//    const BiLmState& get_state_from_history(VertexId vertex, const BiLmState& state) const;

    const IterableStateContainer& get_final_states_list() const {
        assert(states_[kFinalVertex].size() > 0); // We should have at least one final state
        //assert(history_to_state_map_.count(kFinalVertex) == 1);
        //assert(history_to_state_map_.at(kFinalVertex).size() == states_[kFinalVertex].size());
        return get_states_for_vertex(kFinalVertex);
    }

    const generic::StateInterface* final_state() const {
        return &final_state_;
    }

    const IterableStateContainer& get_states_for_vertex(VertexId vertex) const {
        assert(vertex >= 0);
        assert(static_cast<size_t>(vertex) < states_.size());
        return states_[vertex];
    }

  private:
//    std::map<VertexId, std::map<BiLmState, LmVertexId> > history_to_state_map_;
    boost::ptr_vector<IterableStateContainer> states_;
    FinalLmState final_state_;
    StateType* state_buffer_;

#ifndef NDEBUG
  public: //NOLINT
    unsigned int nb_merged_states_;
    unsigned int nb_created_states_;

#endif
};

template<typename StateType>
StateManager<StateType>::StateManager(size_t nb_original_vertice)
        : states_(nb_original_vertice), state_buffer_(NULL)
#ifndef NDEBUG
    , nb_merged_states_(0), nb_created_states_(0)
#endif
{ //NOLINT
    // Warning: the boost::ptr_vector(size_t) constructor is different from the std::vector one and only *reserve* the memory...
    for (size_t i = 0; i < nb_original_vertice; ++i) {
        states_.push_back(new IterableStateContainer());
    }
    //std::cerr << "BiLmLattice constr " << lattice.nb_vertices() << " " << states_.size() <<"\n";
    assert(states_.size() == nb_original_vertice);
}

//template<typename StateType>
//StateType& StateManager<StateType>::add_or_get_identical_state(VertexId vertex, StateType* state) {
//    //std::cerr << "get_or_create_state " << vertex << " " << states_.size() << "\n";
//    assert(vertex < static_cast<int>(states_.size()));
//    IterableStateContainer& states_of_vertex = states_[vertex];
//
//#ifndef NDEBUG
//    bool already_in = (states_of_vertex.find(*state) != states_of_vertex.end());
//    if (already_in) {
//        nb_merged_states_ += 1;
//    } else {
//        nb_created_states_ += 1;
//    }
//#endif
//
//    typename IterableStateContainer::iterator it = states_of_vertex.insert(state).first;
//
//#ifndef NDEBUG
//    if (already_in) {
//        assert(&(*(it)) != state);
//        //assert(!it.second)
//    } else {
//        //rt(it.second);
//        assert(&(*(it)) == state);
//    }
//#endif
//    // if (&(*it) != state) delete state;
//    return *it;
//}

} // namespace latt


#endif /* STATE_STORAGE_HPP_ */
