/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! @file
  This file contains definitions of types used everywhere in the
  EBMT++.
 */

#ifndef DEF_INCLUDED
#define DEF_INCLUDED

// #if __GNUC__>2
// #include <ext/hash_map>
// using __gnu_cxx::hash_map;
// #else
// #include <hash_map>
// #endif

//! the following conditional define is to try to avoid deprecation 
//! warnings in gcc >= 4.3 while keeping compability with gcc <4.3
//#define GCC_VERSION (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
//
//#if GCC_VERSION >= 40300
//#include <tr1/unordered_map>
//#define hash_map std::tr1::unordered_map
//namespace std {
//    using std::tr1::hash;
//}
//#elif defined __clang__
//#include <unordered_map>
//#define hash_map std::tr1::unordered_map
//namespace std {
//    using std::hash;
//}
//#else
//#include <ext/hash_map>
//#define hash_map __gnu_cxx::hash_map
//namespace std {
//    using __gnu_cxx::hash;
//}
//#endif

#include <boost/tuple/tuple.hpp>
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>
#include <functional>
#include <map>
#include <set>
#include <string>
#include <vector>
#include <iostream>

typedef std::vector<std::string> VS;
typedef unsigned int WordIndex;
//typedef unsigned int PhraseIndex;
typedef std::string Content;
typedef std::string Lem;
typedef std::string POS;
typedef int ContentP;
typedef int Dpnd;
typedef std::vector<Dpnd> DpndList;
typedef std::string Category;
typedef std::string Type;
typedef int Depth;
typedef int AlignIndex;

/// \brief PatternType is the representation used to define a pattern (ie a set of position in the input sentence matched to some examples)
typedef std::set<WordIndex> PatternType;
inline std::string pattern_type_str(PatternType pattern) {
    std::string result = "";
    PatternType::const_iterator it = pattern.begin();
    while (it != pattern.end()) {
        if (result != "") result += ",";
        result += boost::lexical_cast<std::string>(*it);
        it++;
    }
    return result;
}
// inline std::ostream& operator<<(std::ostream& stream, const PatternType& pat) {
    // stream << "P<";
    // BOOST_FOREACH(WordIndex idx, pat) {
        // stream<< static_cast<int>(idx) <<" ";
    // }
    // stream << ">";
    // return stream;
// }

inline std::ostream& operator<<(std::ostream& stream, const std::set<WordIndex>& pattern) {
    BOOST_FOREACH(WordIndex idx, pattern) {
        stream << idx << " ";
    }
    return stream;
}


typedef std::map<std::string, double> WeightVector;

//! flag if the alignment is accepted (used) or not
enum AlignAccepted {
    YES, //!< accepted (used)
    NO   //!< rejected (not used)
};

//! information of which direction a example depends on its parent
enum ParentBondRelation {
    PRE,   //!< the example depends on the parent word from left (= pre-position)
    POST,  //!< the example depends on the parent word from right (= post-position)
    UNK    //!< unknown
};

//! languages available
enum Language {
    JA, //!< Japanese
    EN, //!< English
    ZH, //!< Chinese
    DEFAULT_LANG //!< only uses language independent components
};

// Used in hash function
//const int HASH_MAGIC_NUMBER = 599231;
//const PhraseIndex NULL_PHRASE = 0;

//class stringhash : public std::unary_function< std::string, size_t > {
//  public:
//    size_t operator() (const std::string& key) const {
//        std::hash<const char*> H;
//        return H(key.c_str());
//    }
//};

#endif
