/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <glog/logging.h>
#include <cassert>
#include <cstdlib>
#include <map>
#include <string>
#include "dictionary.hpp"

Dictionary::Dictionary(const std::string &filename) {
    std::ifstream ifs(filename.c_str());
    if (!ifs) {
        LOG(FATAL) << "Error loading dictionary file " << filename << ".";
    }
    std::string buffer;
    while (!ifs.eof()) {
        getline(ifs, buffer);
        for (unsigned i = 0; i < buffer.size(); i++) {
            if (buffer[i] == '\t') {
                std::string word_e = buffer.substr(0, i);
                std::string word_f = buffer.substr(i + 1);
                dic_[word_e] = word_f;
            }
        }
    }
    ifs.close();
}

std::string Dictionary::query(const std::string &word) const {
    std::string result;
    std::map<std::string, std::string>::const_iterator it = dic_.find(word);
    if (it == dic_.end()) {
        result = "";
        LOG(INFO) << "Dictionary module: query=" << word << " result=OOV";
        return result;
    }
    result = it->second;
    LOG(INFO) << "Dictionary module: query=" << word << " result=" << result;
    return result;
}
