/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __FEATURES__HPP__
#define __FEATURES__HPP__

#include <boost/foreach.hpp>
#include <exception>
#include <map>
#include <string>
#include "def.hpp"
#include "glog/logging.h"

class FeatureNotFoundException: public std::exception {
    virtual const char* what() const throw() {
        return "FeatureNotFoundException";
    }
};

class FeatureVector: public std::map<std::string, double> {
    friend std::ostream &operator<<(std::ostream &stream, const FeatureVector &f);

  public:
    FeatureVector(): std::map<std::string, double>() {}
    FeatureVector(const FeatureVector& fv1, const FeatureVector& fv2) 
            : std::map<std::string, double>(fv1) {
        BOOST_FOREACH(const value_type& name_val, fv2) {
            (*this)[name_val.first] += name_val.second;
        }  
    }

    void merge_in(const FeatureVector& fv2) {
        BOOST_FOREACH(const value_type& name_val, fv2) {
            (*this)[name_val.first] += name_val.second;
        }  
    }    
    
    double compute_score(const WeightVector& weight_vector) const {
        double score = 0;
        WeightVector::const_iterator weight_end = weight_vector.end();
        BOOST_FOREACH(const FeatureVector::value_type& name_val, *this) {
            WeightVector::const_iterator it_weight = weight_vector.find(name_val.first);
            if (it_weight == weight_end) {
                LOG(INFO) << name_val.first;
                LOG(WARNING) << "Feature '" << name_val.first << "' does not exist in weight file.";
                throw FeatureNotFoundException();
            }
            score += it_weight->second * name_val.second;
            LOG(INFO) << "score = " << score << " (" << name_val.first << ": " << it_weight->second << " * " << name_val.second << ")";
        }
        return score;
    }
};

#endif
