/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "hypotheses_storage.hpp"

//#if GCC_VERSION >= 40300
//#include <tr1/unordered_map>
//#elif defined __clang__
//#include <unordered_map>
//#else
//#include <ext/hash_map>
//#endif

#include <glog/logging.h>
#include <algorithm>
#include <deque>
#include <list>
#include <map>
#include <string>
#include <limits>
#include <utility>
#include <vector>
#include <set>

void calc_trans_prob(size_t pattern_size,
        const std::map<std::string, double>& t_exp_freq,
        const std::map<std::string, std::map<std::string, double> >& t_lm_exp_freq, const std::vector<Hypothesis*> &cand) {
    // used t-side expressions
    double max = 0.0;
    for (std::map<std::string, double>::const_iterator it = t_exp_freq.begin();
            it != t_exp_freq.end(); it++) {
        if (max < it->second)
            max = it->second;
    }

    for (std::vector<Hypothesis*>::const_iterator hypothesis = cand.begin();
            hypothesis != cand.end(); hypothesis++) {
        std::map<std::string, double>::const_iterator tmp = t_exp_freq.find(
                (*hypothesis)->real_str());
        if (tmp == t_exp_freq.end()) {
            LOG(INFO)<< "Warning (t_exp)" << "\t" << (*hypothesis)->real_str() << "\t" << (*hypothesis)->lm_str();
        } else {
            double trans_prob = tmp->second / max;
            (*hypothesis)->set_feature_value("trans_prob", trans_prob);
            (*hypothesis)->set_feature_value("trans_prob_multiplied_by_size", trans_prob * static_cast<double>(pattern_size));
            std::map<std::string, std::map<std::string, double> >::const_iterator lm_it1 = t_lm_exp_freq.find(tmp->first);
            if (lm_it1 == t_lm_exp_freq.end()) {
                LOG(INFO) << "Warning (t_lm_exp_1)" << "\t" << (*hypothesis)->real_str() << "\t" << (*hypothesis)->lm_str();
            } else {
                std::map<std::string, double>::const_iterator lm_it2 = lm_it1->second.find((*hypothesis)->lm_str());
                if (lm_it2 == lm_it1->second.end()) {
                    LOG(INFO) << "Warning (t_lm_exp_2)" << "\t" << (*hypothesis)->real_str() << "\t" << (*hypothesis)->lm_str();
                } else {
                    (*hypothesis)->set_feature_value("trans_lm_prob", lm_it2->second / tmp->second);
                }
            }
        }
    }
    // TODO(Nakazawa) need to check the head-switch probability in the future
}

void compute_target_frequency_feature(int pattern_freq, std::vector<Hypothesis*>* hyp_vect) {
    // pattern_freq is actually not used, but kept in the signature for now
    if (hyp_vect->size() == 0)
        return; // "Cannot compute t_string frequency on a empty stack";

    typedef std::map<Hypothesis*, int, HypothesisTStringCompare> MapDupCount;
    MapDupCount duplicates;

    //std::vector<Hypothesis*>::iterator it = candidate_.begin();

    BOOST_FOREACH(Hypothesis* p_hypothesis, *hyp_vect) {
        if (duplicates.count(p_hypothesis) == 0) {
            duplicates[p_hypothesis] = 1;
        } else {
            duplicates[p_hypothesis] += 1;
        }
    }

    double nb_hyp = hyp_vect->size();

    BOOST_FOREACH(Hypothesis* p_hypothesis, *hyp_vect) {
        p_hypothesis->set_feature_value("t_string_frequency",
                static_cast<double>(duplicates[p_hypothesis])
                        / nb_hyp);
    }
}

std::ostream& operator<<(std::ostream& stream, const InitialHypothesesStorage& store) {
    stream << ">> BEGIN HYP STORE\n";
    for (std::map<PatternType, std::vector<Hypothesis*> >::const_iterator it = store.hypotheses_map_.begin();
         it != store.hypotheses_map_.end(); ++it) {
        for (std::vector<Hypothesis*>::const_iterator hyp = it->second.begin();
             hyp != it->second.end(); hyp++) {
            stream << **hyp << "\n";
        }
    }
    stream << ">> END HYP STORE\n";
    return stream;
}
