/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __HYPOTHESIS_MANAGEMENT_AND_COMBINATION_HPP__
#define __HYPOTHESIS_MANAGEMENT_AND_COMBINATION_HPP__

#include <boost/range/adaptor/map.hpp>
#include <algorithm>
#include <list>
#include <map>
#include <string>
#include <vector>
#include <set>
#include <limits>
#include "def.hpp"
//#include "../TR++/tr_interface.hpp"
#include "hypothesis.hpp"
#include "params.hpp"
//#include "lazy_hypothesis_manager.hpp"
//#include "table_display_output.hpp"
//#include "lm/model.hh"
//#include "translation_result.hpp"

void calc_trans_prob(size_t pattern_size,
        const std::map<std::string, double>& t_exp_freq,
        const std::map<std::string, std::map<std::string, double> >& t_lm_exp_freq, const std::vector<Hypothesis*> &cand);

void compute_target_frequency_feature(int pattern_freq, std::vector<Hypothesis*>* hyp_vect);

class InitialHypothesesStorage: boost::noncopyable {
  public:
    friend class HumanReadableHypothesisSerialization;
    friend std::ostream& operator<<(std::ostream& stream, const InitialHypothesesStorage& store);

    ~InitialHypothesesStorage() {
        BOOST_FOREACH(HypothesesList hyp_list, hypotheses_map_ | boost::adaptors::map_values) {
            BOOST_FOREACH(Hypothesis* hyp, hyp_list) {
                delete hyp;
            }
        }
    }

    void push(const PatternType& pattern, Hypothesis *hyp) {
        hypotheses_map_[pattern].push_back(hyp);
    }

    std::vector<PatternType> get_pattern_list() const {
        std::vector<PatternType> result;
        BOOST_FOREACH(const PatternType& pat, hypotheses_map_ | boost::adaptors::map_keys) {
            result.push_back(pat);
        }
        return result;
    }

    const std::vector<Hypothesis*>& _get_cand_list_for_pattern(const PatternType& pat) const {
        return hypotheses_map_.at(pat);
    }

//    void compute_target_frequency_feature(int pattern_freq);

    bool has_empty_stack() const {
        for (HypothesesMapType::const_iterator it = hypotheses_map_.begin();
                it != hypotheses_map_.end(); ++it) {
            if (it->second.size() == 0) {
                return true;
            }
        }
        return false;
    }

    size_t nb_patterns() const { return hypotheses_map_.size(); }

    void calc_trans_prob_for_pattern(const PatternType& pattern,
                const std::map<std::string, double>& t_exp_freq,
                const std::map<std::string, std::map<std::string, double> >& t_lm_exp_freq) {
        //BOOST_FOREACH(std::vector<Hypothesis*>& hyp_vect, hypotheses_map_ | boost::adaptors::map_values) {
        if (hypotheses_map_.count(pattern) > 0) {
            calc_trans_prob(pattern.size(), t_exp_freq, t_lm_exp_freq, hypotheses_map_.at(pattern));
        }
        //}
    }

    void calc_target_frequency_feature(const PatternType& pattern, int pattern_freq) {
        if (hypotheses_map_.count(pattern)) {
            std::vector<Hypothesis*>* hyp_vect = &hypotheses_map_.at(pattern);
            compute_target_frequency_feature(pattern_freq, hyp_vect);
        }
    }

  private:
    typedef std::vector<Hypothesis*> HypothesesList;
    typedef std::map<PatternType, HypothesesList> HypothesesMapType;
    HypothesesMapType hypotheses_map_;
};



#endif // __HYPOTHESIS_MANAGEMENT_AND_COMBINATION_HPP__


