/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PARAMS_HPP__
#define PARAMS_HPP__

#include <set>
#include <string>
#include <vector>
#include <fstream>
#include <iostream>
#include <cassert>

namespace Params {
    namespace Global {
        extern std::string config_filename;
        extern bool web;
        extern bool web_verbose;
        extern bool web_all;
        extern std::string web_dir;
        extern std::string trans_dir;
        extern std::string output_mode;
        extern std::string input_mode;
        extern int sentence_id;
        extern bool keep_tms_in_memory;
        extern std::string time_log_file;
    }
    namespace Features {
        extern std::set<std::string> all_features;
        extern std::set<std::string> ignored_features;
    }
    namespace Translator {
        extern size_t nb_threads;
        extern std::vector<std::string> input_filename_list;
        extern std::vector<std::string> tr_db_filename_list;
        extern std::string source_language;
        extern std::string target_language;
        extern std::string dic_words_filename;
        extern std::string dic_punc_filename;
        extern std::string dic_translit_filename;
        extern std::string lm_filename;
        extern std::string tr_result_filename;
        extern std::string lexical_translation_prob_filename;
        extern std::string default_dpnd;
        extern std::string feature_weight;
        extern bool no_head_switch;
        extern bool no_sibling;
        extern bool iso_dic_trans;
        extern std::string tr_mode;
        extern std::string tr_language;
        extern bool server_mode;
        extern int port;
        extern std::string parse_command;
        extern int n_best_length;
        extern int beam_width;
        extern bool use_tr_server;
        extern std::string tr_server_language;
        extern std::string tr_server_host;
        extern std::string tr_server_port;
        extern std::string hypothesis_output_prefix;
        extern int leading_zeros;
        extern int rerank_beam;
        extern std::string decoder;
        extern bool display_progress;
        extern bool tr_fast_siblings;
        extern int tr_max_nb_tms_per_pattern;
        extern bool use_wildcard_pair_cache;
        extern std::string reference_file;
        extern std::vector<std::vector<std::string> > reference_translations;
        extern bool first_word_should_be_uppercased_for_lm_eval;
        extern bool no_keep_input_order_of_additionals;
        extern bool shuffle_translation_order;
        extern bool verbose_features;
        extern bool disable_lattice_reduction;
        extern bool reconstruct_final_hypotheses;
        extern int lattice_beam_pre_truncation;
        extern bool no_siblings_retrieval;
        extern bool use_nbest_parses;
    }

    // The following empty structure can be used to print the parameters as follow:
    // output_stream << ParamsInPrintableForm();
    struct ParamsInPrintableForm {};
    std::ostream& operator<<(std::ostream& stream, ParamsInPrintableForm);

    /// Function parsing the command line.
    /// It will terminate the program if bad arguments are passed
    /// either on the command line or in the config file.
    void parse_options(int argc, char* argv[]);
}

#endif
