/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "partial_translation.hpp"
#include <algorithm>
#include <string>
#include <vector>

WordIndex get_ref_id(const std::string& str) {
    // a bit hackish
    // since get_ref_id get called exceedingly often, it is actually
    // interesting to have a very optimized version
    size_t size_ = str.size();
    if (size_ < 3 || str[0] != '[' || str[size_ - 1] != ']') {
        return -1;
    }
    unsigned char zero = '0';
    unsigned char val = str[2];
    assert(val >= zero && val <= zero + 10);
    WordIndex result = val - zero;
    for (size_t i = 3; i < size_ - 1; i++) {
        unsigned char val = str[i];
        result *= 10;
        result += val - zero;
    }
    return result;
}

/*
// Currently not used.
PartialTranslation::PartialTranslation(const VS& t_string, StringIndexerPtr indexer) {
    this->indexer = indexer;
    BOOST_FOREACH(const std::string& word, t_string) {
        WordIndex ref = get_ref_id(word);
        if (ref == (WordIndex) -1) {
            StringIndex string_id = indexer->getIdAndUpdate(word);
            sequence.push_back(WordOrNT::make_string_id(string_id));            
        } else {
            sequence.push_back(WordOrNT::make_ref(ref));
        }
    }
}
*/

/*
// Currently not used.
PartialTranslation::PartialTranslation(const PartialTranslation& partial_trans_outside, 
                                       const PartialTranslation& partial_trans_inside,
                                       size_t                    insertion_point,
                                       bool                      replace_element) : 
        indexer(partial_trans_outside.indexer) {
    // check both use the same indexer
    assert(partial_trans_outside.indexer.get() == partial_trans_inside.indexer.get());  
    size_t cand1_t_string_size = partial_trans_inside.sequence.size();
    size_t cand2_t_string_size = partial_trans_outside.sequence.size();
    assert(insertion_point <= cand2_t_string_size);
    int jump_distance;
    size_t final_size;
    
    if (replace_element) {
        jump_distance = 1;
        final_size = cand2_t_string_size + cand1_t_string_size - 1;
    } else {
        jump_distance = 0;
        final_size = cand2_t_string_size + cand1_t_string_size;            
    }
    
    sequence.reserve(final_size);
    std::back_insert_iterator<std::vector<WordOrNT> > sequence_iterator = back_inserter(sequence);

    std::copy(partial_trans_outside.sequence.begin(),
              partial_trans_outside.sequence.begin() + insertion_point,
              sequence_iterator);
    std::copy(partial_trans_inside.sequence.begin(),
              partial_trans_inside.sequence.end(),
              sequence_iterator);
    std::copy(partial_trans_outside.sequence.begin() + insertion_point + jump_distance,
              partial_trans_outside.sequence.end(),
              sequence_iterator);
}
*/

void PartialTranslation::hash(std::string *result) const {
    BOOST_FOREACH(WordOrNT word, sequence) {
        if (word.is_ref()) {
            *result += "[X" + boost::lexical_cast<std::string>(word.get_ref()) + "]";
        } else {
            *result += indexer->getLabel(word.get_string_id());
        }
        *result += " ";
    }
}

std::ostream& operator<<(std::ostream& out, const PartialTranslation& partial_translation) {
    out << "PT[ ";
    BOOST_FOREACH(WordOrNT word, partial_translation.sequence) {
        if (word.is_ref()) {
            out << "[X" + boost::lexical_cast<std::string>(word.get_ref())+"]";
        } else {
            out << (partial_translation.indexer->getLabel(word.get_string_id()));
        }
        out << " ";
    }
    out << "]";
    return out;
}
