/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "partial_translation_string.hpp"
#include <boost/foreach.hpp>
#include <algorithm>
#include <string>

void expand_t_string(const VS &t_string2, const VS &t_string1, VS &t_string_result, int i, bool replace) {
    assert(i <= static_cast<int>(t_string2.size()));
    size_t final_size;
    int jump_size;
    if (replace) {
        final_size = t_string2.size() + t_string1.size() - 1;
        jump_size = 1;
    } else {
        final_size = t_string2.size() + t_string1.size();
        jump_size = 0;
    }
    t_string_result.reserve(final_size);
    std::back_insert_iterator<VS> new_t_string_iterator = back_inserter(t_string_result);
    std::copy(t_string2.begin(), t_string2.begin() + i, new_t_string_iterator);
    std::copy(t_string1.begin(), t_string1.end(), new_t_string_iterator);
    std::copy(t_string2.begin() + i + jump_size, t_string2.end(), new_t_string_iterator);
}

PartialTranslationString::PartialTranslationString(const PartialTranslationString& outside, 
                                                   const PartialTranslationString& inside, 
                                                   int insertion_point, bool replace) {
    expand_t_string(outside, inside, *this, insertion_point, replace);                          
}

int PartialTranslationString::find_compatible_ref_id(const PatternType &list) const {
    for (int i = size() - 1; i >= 0; i--) {
        InputInsertionId ref_id = InputInsertionId::get_ref_id((*this)[i]);
        if (ref_id.is_valid() && ref_id.is_included_in_pattern(list)) {
            return i;
        }
    }
    return -1; // not found
}

std::ostream& operator<<(std::ostream& out, const PartialTranslationString& partial_translation) {
    out << "PT[ ";
    BOOST_FOREACH(const std::string& word, partial_translation) {
        out << word << " ";
    }
    out << "]";
    return out;
}

void PartialTranslationString::hash(std::string *result) const {
    BOOST_FOREACH(const std::string& word, *this) {
        *result += word + " ";
    }
}

