/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __SCORE_COMPUTER_HPP_
#define __SCORE_COMPUTER_HPP_

#include <string>
#include <map>
#include <limits>
#include <vector>

#include "def.hpp"
//#include "language_model.hpp"
//#include "lm_computer.hpp"
#include "bleu_computer.hpp"
//#include "tree_lm.hpp"


struct ScoreComputer {
    //LMComputer *lm_comp;
    //const LanguageModel &lm;
    const WeightVector &weights;
    //const TreeLM &tree_lm;
    int input_size;
    //bool need_to_estimate_lm;
    
    ModifiedNgramPrecisionComputer ngram_computer_;

  public:
    ScoreComputer(//LMComputer *lm_comp, const LanguageModel &lm,
            const WeightVector &weights, int input_size): //, const TreeLM &tree_lm) :
            //lm_comp(lm_comp), lm(lm),
            weights(weights), //tree_lm(tree_lm)
            input_size(input_size) {} 
             //need_to_estimate_lm(true) 
    ScoreComputer(//LMComputer *lm_comp, const LanguageModel &lm,
            const WeightVector &weights, const VS& reference, int input_size): //, const TreeLM &tree_lm) :
            //lm_comp(lm_comp), lm(lm),
            weights(weights), input_size(input_size), //tree_lm(tree_lm),
             //need_to_estimate_lm(true),
             ngram_computer_(reference) {
    }

    double eval_oracle_bleu_diff(const VS& t_string, int pattern_size) {
        assert(ngram_computer_.is_valid());
        assert(input_size != std::numeric_limits<int>::min());
        ngram_computer_.set_current_counts(50, 30, 15, 8, 100, 90, 80, 70, 100, 100);
        return ngram_computer_.compute_BLEU_delta_preset(t_string, static_cast<double>(pattern_size)/input_size);
    }

    double eval_oracle_bleu_plus_one(const VS& t_string, int pattern_size) {
        assert(ngram_computer_.is_valid());
        assert(input_size != std::numeric_limits<int>::min());
        ngram_computer_.set_current_counts(50, 30, 15, 8, 100, 90, 80, 70, 100, 100);
        return ngram_computer_.compute_BLEU_delta_preset(t_string, static_cast<double>(pattern_size)/input_size);
    }

    double eval_oracle(const VS& t_string, int pattern_size) {
        assert(ngram_computer_.is_valid());
        assert(input_size != std::numeric_limits<int>::min());
        return eval_oracle_bleu_plus_one(t_string, pattern_size);
    }

    double compute_bleu_of_flat_string(const std::string& str, bool plus_one = false) {
        if (!ngram_computer_.is_valid()) return -std::numeric_limits<double>::max();
        VS segmented_string = segment_sentence_along_spaces(str);
        return ngram_computer_.compute_BLEU(segmented_string, plus_one);
    }

    double compute_bleu(const VS& segmented_string, bool plus_one = false) {
            if (!ngram_computer_.is_valid()) return -std::numeric_limits<double>::max();
            return ngram_computer_.compute_BLEU(segmented_string, plus_one);
        }
};

#endif
