/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/regex.hpp>
#include <boost/regex/icu.hpp>
#include <string>
#include <vector>
#include <utility>
#include <map>
#include "table_display_output.hpp"

void TableDisplayOutput::print_tm(const int index, const TM* tm, const DataForTM *data_for_tm) {
    println("["+boost::lexical_cast<std::string>(index)+"] "+tm->id(), table(), row(), 1, "colspan=3 extern=show_alignment.cgi?fname="+tm->id());
    next_row();

    VS s_tree_buffer, t_tree_buffer;
    std::vector<char> mark;
    tm->para_sentence().s_dtree().print_dtree(data_for_tm->s_bond_root->dpnd_ref != NULL ? data_for_tm->s_bond_root->dpnd_ref : data_for_tm->s_bond_root, mark, s_tree_buffer, false, data_for_tm->s_bond);
    tm->para_sentence().t_dtree().print_dtree(data_for_tm->t_bond_root->dpnd_ref != NULL ? data_for_tm->t_bond_root->dpnd_ref : data_for_tm->t_bond_root, mark, t_tree_buffer, false, data_for_tm->t_bond);

    int s_tree_buffer_size = s_tree_buffer.size();
    for (int s = 0; s < s_tree_buffer_size; s++) {
        std::string words = s_tree_buffer[s];
        convert_bracket_symbols(words);
        println(words, table(), row()+s, 2, "width=50%");
    }
    int t_tree_buffer_size = t_tree_buffer.size();
    for (int t = 0; t < t_tree_buffer_size; t++) {
        println(t_tree_buffer[t], table(), row()+t, 3, "width=50%");
    }
    // phrase num
    for (int i = 0; i < s_tree_buffer_size || i < t_tree_buffer_size; i++) {
        println(boost::lexical_cast<std::string>(i), table(), row()+i, 1);
    }
}

// output for TableDisplay
void TableDisplayOutput::output_for_table_display(int m, const SingleNBestTranslationResult& tl_result) {
    // gather TM used for the translation
    std::vector<std::pair<const TM*, const DataForTM*> > tm_list;
    tl_result.hypothesis->get_used_tm(&tm_list);

    // verbose output
    if (Params::Global::web_all || Params::Global::web_verbose) {
        println("*** Output Summary (" + boost::lexical_cast<std::string>(m+1) + ") ***", table(), 1, 1, "bgcolor=#E0FFFF");
        next_table();

        // input string
        println("Input", table(), 1, 1, "bgcolor=#98FB98");
        println(input_tree_->sentence(), table(), 1, 2, "bgcolor=#98FB98");
        // reserve 2nd line for ref
        // output string
        println("Output", table(), 3, 1, "bgcolor=#E0FFFF");
        println(tl_result.t_string_flat, table(), 3, 2, "bgcolor=#E0FFFF");
        // score other than LM
        println("No-LM score", table(), 4, 1, "bgcolor=#E0FFFF");
        println(boost::lexical_cast<std::string>(tl_result.hypothesis->score()), table(), 4, 2);
        // LM score 
        println("LM score", table(), 5, 1, "bgcolor=#E0FFFF");
        println(boost::lexical_cast<std::string>(tl_result.features.find("lm")->second), table(), 5, 2);
        // final score
        println("score", table(), 6, 1, "bgcolor=#E0FFFF");
        println(boost::lexical_cast<std::string>(tl_result.score), table(), 6, 2);
        next_table();
    }

    println("*** Input and Output Dependency Trees ***", table(), 1, 1, "bgcolor=#E0FFFF");
    next_table();
    unsigned int dtree_table = table();

    // initialize
    for (size_t i = 0; i < input_tree_->size(); i++) {
        input_tree_->get_modifiable_node_at(i)->align_index = -1;
    }

    std::map<const TM*, size_t> tm_mapper;
    // TM
    next_table();
    println("*** List of Used Translation Examples ***", table(), 1, 1, "bgcolor=#E0FFFF");
    next_table();
    for (size_t tm_index = 0; tm_index < tm_list.size(); tm_index++) {
        const std::vector<Alignment>& i_s_match = tm_list[tm_index].first->i_s_match();
        tm_mapper[tm_list[tm_index].first] = tm_index;
        for (std::vector<Alignment>::const_iterator it = i_s_match.begin(); it != i_s_match.end(); it++) {
            // skip wildcard nodes 
            if (word_overlap(tm_list[tm_index].first->wildcards(), it->p1)) {
                continue;
            }
            for (std::vector<WordIndex>::const_iterator wd = it->p1.begin(); wd != it->p1.end(); wd++) {
                input_tree_->get_modifiable_node_at(*wd)->align_index = tm_index;
                if (m == 0) {
                    in_out_mapper_[tm_index].first.insert(*wd);
                }
            }
        }
        print_tm(tm_index, tm_list[tm_index].first, tm_list[tm_index].second);
        next_table();
    }

    // input dtree
    VS i_tree_buffer;
    std::vector<char> mark;
    input_tree_->print_dtree(input_tree_->root(), mark, i_tree_buffer);
    int i_tree_buffer_size = i_tree_buffer.size();
    for (int buf_index = 0; buf_index < i_tree_buffer_size; buf_index++) {
        const boost::regex reg("(\\[\\d+\\] .+)$");
        std::string words = i_tree_buffer[buf_index];
        convert_bracket_symbols(words);
        std::string tmp_str = boost::regex_replace(words, reg, "<font color=\"" + color(input_tree_->node_at(buf_index)->align_index)+"\">$1</font>", boost::format_all);
        println(boost::lexical_cast<std::string>(buf_index), dtree_table, buf_index + 1, 1);
        if (input_tree_->node_at(buf_index)->align_index < 0) {
            println(tmp_str, dtree_table, buf_index+1, 2, "width=50%");
        } else {
            println(tmp_str, dtree_table, buf_index+1, 2, "width=50% extern=show_alignment.cgi?fname=" + tm_list[input_tree_->node_at(buf_index)->align_index].first->id());
        }
    }

    // output dtree
    const VS &content = tl_result.hypothesis->t_string();
    const std::vector<Dpnd> &dpnd = tl_result.hypothesis->get_t_dpnd();
    Dtree t_dtree;
    for (size_t j = 0; j < content.size(); j++) {
        if (content[j] == "#") {
            // Ignore EOS marker.
            continue;
        }
        DtreeNode node;
        node.word.content = content[j];
        node.dpnd = dpnd[j];

        std::map<const TM*, size_t>::iterator it = tm_mapper.find(tl_result.hypothesis->get_initial_hypotheses().at(j)->tm());
        if (it == tm_mapper.end()) {
            node.align_index = -1;
        } else {
            node.align_index = tm_mapper[tl_result.hypothesis->get_initial_hypotheses().at(j)->tm()];
        }
        t_dtree.push_node(node);
    }
    t_dtree.format_dtree();
    VS t_tree_buffer;
    mark.clear();
    t_dtree.print_dtree(t_dtree.root(), mark, t_tree_buffer);
    int t_tree_buffer_size = t_tree_buffer.size();
    for (int buf_index = 0; buf_index < t_tree_buffer_size; buf_index++) {
        const boost::regex reg("(\\[\\d+\\] .+)$");
        std::string tmp_str = boost::regex_replace(t_tree_buffer[buf_index], reg, "<font color=\"" + color(t_dtree.node_at(buf_index)->align_index)+"\">$1</font>", boost::format_all);
        if (t_dtree.node_at(buf_index)->align_index < 0) {
            println(tmp_str, dtree_table, buf_index+2, 3, "width=50%");
        } else {
            println(tmp_str, dtree_table, buf_index+2, 3, "width=50% extern=show_alignment.cgi?fname=" + tm_list[t_dtree.node_at(buf_index)->align_index].first->id());
        }
    }

    // mapping between input and output words
    if (m == 0) {
        const std::vector<const Hypothesis*> initial_hypothesis_list = tl_result.hypothesis->get_initial_hypotheses();
        VS output_words = split(tl_result.t_string_flat, " ");
        int output_words_size = output_words.size();
        VS t_string_words;
        std::vector<int> t_string_index_mapper;
        int t_string_index = 0;
        BOOST_FOREACH(const std::string tmp_w, tl_result.hypothesis->t_string()) {
            VS tmp_t_string_words = split(tmp_w, " ");
            t_string_words.insert(t_string_words.end(), tmp_t_string_words.begin(), tmp_t_string_words.end());
            for (size_t i = 0; i < tmp_t_string_words.size(); i++) {
                t_string_index_mapper.push_back(t_string_index);
            }
            t_string_index++;
        }
        t_string_index = 0;
        int t_string_size = t_string_words.size();
        for (int i = 0; i < output_words_size; i++) {
            // bourlon add
            const boost::regex esc("[\\^\\.\\$\\|\\(\\)\\[\\]\\*\\+\\?\\/\\\\]");
            const std::string rep("\\\\\\1&");
            std::string result;
            std::string esc_output_word = regex_replace(output_words[i], esc, rep, boost::match_default | boost::format_sed);
            // bourlon end
            
            // const boost::u32regex output_w = boost::make_u32regex(output_words[i], boost::regex::icase);
            // const boost::u32regex output_w_with_mark = boost::make_u32regex(output_words[i]+"\\*", boost::regex::icase);
            // for (; t_string_index < t_string_size; t_string_index++) {
            //     if (boost::u32regex_match(t_string_words[t_string_index], output_w) ||
            //         (((t_string_index+1 < t_string_size &&
            //            !boost::u32regex_match(t_string_words[t_string_index+1], output_w)) ||
            //           t_string_index+1 == t_string_size) &&
            //          boost::u32regex_match(t_string_words[t_string_index], output_w_with_mark))) {
            //         break;
            //     }
            // }
            const boost::u32regex output_w = boost::make_u32regex(esc_output_word, boost::regex::icase);
            const boost::u32regex output_w_with_mark = boost::make_u32regex(esc_output_word+"\\*", boost::regex::icase);
            for (; t_string_index < t_string_size; t_string_index++) {
                if (boost::u32regex_match(t_string_words[t_string_index], output_w) ||
                    (((t_string_index+1 < t_string_size &&
                       !boost::u32regex_match(t_string_words[t_string_index+1], output_w)) ||
                      t_string_index+1 == t_string_size) &&
                     boost::u32regex_match(t_string_words[t_string_index], output_w_with_mark))) {
                    break;
                }
            }

            if (t_string_index == t_string_size) {
                std::cerr << "Error of mapping between input and output" << std::endl;
                break;
            }
            in_out_mapper_[tm_mapper[initial_hypothesis_list[t_string_index_mapper[t_string_index]]->tm()]].second.insert(i);
            t_string_index++;
        }
        // translation confidence estimation
        translation_confidence_ = confidence_estimator_.estimate(*input_tree_, tl_result, tm_list);
    }
}
