/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "tm.hpp"
#include <vector>

std::ostream &operator<<(std::ostream &os, const DataForTM &t) {
    os << "--------------------" << std::endl;

    os << "i_body_ref: (root: " << t.i_root->num << ")" << std::endl;
    for (std::vector<const DtreeNode*>::const_iterator i = t.i_body_ref.begin(); i != t.i_body_ref.end(); i++) {
        os << (*i)->num << " ";
    }
    os << std::endl;

    os << "s_body_ref: (root: " << t.s_root->num << ")" << std::endl;
    for (std::vector<const DtreeNode*>::const_iterator i = t.s_body_ref.begin(); i != t.s_body_ref.end(); i++) {
        os << (*i)->num << " ";
    }
    os << std::endl;

    os << "t_body_ref: (root: " << t.t_root->num << ")" << std::endl;
    for (std::vector<const DtreeNode*>::const_iterator i = t.t_body_ref.begin(); i != t.t_body_ref.end(); i++) {
        os << (*i)->num << " ";
    }
    os << std::endl;

    os << "s_bond: (root: " << t.s_bond_root->num << ")" << std::endl;
    for (size_t i = 0; i < t.s_size_; i++) {
        os << t.s_bond[i] << " ";
    }
    os << std::endl;

    os << "t_bond: (root: " << t.t_bond_root->num << ")" << std::endl;
    for (size_t i = 0; i < t.t_size_; i++) {
        os << t.t_bond[i] << " ";
    }
    os << std::endl;

    os << "t_lm_check:" << std::endl;
    for (size_t i = 0; i < t.t_size_; i++) {
        os << t.t_lm_check[i] << " ";
    }
    os << std::endl;

    os << "t_lm_check:" << std::endl;
    for (size_t i = 0; i < t.t_size_; i++) {
        os << t.t_lm_check[i] << " ";
    }
    os << std::endl;

    os << "i_child_ref:" << std::endl;
    for (std::vector<const DtreeNode*>::const_iterator i = t.i_child_ref.begin(); i != t.i_child_ref.end(); i++) {
        os << (*i)->num << " ";
    }
    os << std::endl;

    os << "s_child_ref:" << std::endl;
    for (std::vector<const DtreeNode*>::const_iterator i = t.s_child_ref.begin(); i != t.s_child_ref.end(); i++) {
        os << (*i)->num << " ";
    }
    os << std::endl;

    os << "s_other_child_ref:" << std::endl;
    for (std::vector<const DtreeNode*>::const_iterator i = t.s_other_child_ref.begin(); i != t.s_other_child_ref.end(); i++) {
        os << (*i)->num << " ";
    }
    os << std::endl;

    os << "--------------------" << std::endl;
    return os;
}

const Alignment* TM::get_alignment_i(const DtreeNode *w) const {
    for (std::vector<Alignment>::const_iterator it = i_s_match_.begin(); it != i_s_match_.end(); it++) {
        if (grep(w->num, it->p1).size() > 0) return &*it;
    }
    return NULL;
}

const Alignment* TM::get_alignment_s(const DtreeNode *w) const {
    for (std::vector<Alignment>::const_iterator it = i_s_match_.begin(); it != i_s_match_.end(); it++) {
        if (grep(w->num, it->p2).size() > 0) return &*it;
    }
    return NULL;
}

const Alignment* TM::get_alignment_s(const WordIndex& w) const {
    for (std::vector<Alignment>::const_iterator it = i_s_match_.begin(); it != i_s_match_.end(); it++) {
        if (grep(w, it->p2).size() > 0) return &*it;
    }
    return NULL;
}

const PatternType TM::get_i_of_t(const DtreeNode *w) const {
    PatternType result;

    BOOST_FOREACH(const DtreeNode *c, w->pre_children_ref) {
        PatternType i_of_t = get_i_of_t(c);
        result.insert(i_of_t.begin(), i_of_t.end());
    }
    BOOST_FOREACH(const DtreeNode *c, w->post_children_ref) {
        PatternType i_of_t = get_i_of_t(c);
        result.insert(i_of_t.begin(), i_of_t.end());
    }

    const Alignment* s_t_align = para_sentence_.get_alignment_t(w);
    if (s_t_align != NULL) {
        BOOST_FOREACH(WordIndex s_w, s_t_align->p1) {
            const Alignment* i_s_align = get_alignment_s(s_w);
            if (i_s_align != NULL) {
                BOOST_FOREACH(WordIndex i_w, i_s_align->p1) {
                    result.insert(i_w);
                }
            }
        }
    }
    return result;
}

void TM::print(std::ostream *os) const {
    *os << para_sentence_;
    print_i_s_match(os);
    *os << std::endl;
}
void TM::print(std::ostream *os, DataForTM *data_for_tm) const {
    para_sentence_.print(os, data_for_tm->s_bond_root->dpnd_ref != NULL ? data_for_tm->s_bond_root->dpnd_ref : data_for_tm->s_bond_root,
                         data_for_tm->t_bond_root->dpnd_ref != NULL ? data_for_tm->t_bond_root->dpnd_ref : data_for_tm->t_bond_root,
                         data_for_tm->s_bond, data_for_tm->t_bond);
}
std::ostream &operator<<(std::ostream &os, const TM &t) {
    t.print(&os);
    return os;
}

