/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/regex.hpp>
#include <map>
#include <string>
#include <vector>
#include "translation_result.hpp"
#include "utils.hpp"

void remove_asterisk(std::string *str) {
    *str = boost::regex_replace(*str, boost::regex("\\*$"), "");
}

void process_optionals_and_print(const SingleNBestTranslationResult &translation_result, std::ostream *os) {
    const VS &t_string_with_optionals = translation_result.hypothesis->get_t_string_as_VS();
    const std::vector<int> &t_dpnd_with_optionals = translation_result.hypothesis->get_t_dpnd();
    VS t_string;
    std::vector<int> t_dpnd;
    std::vector<double> score;
    std::map<int, int> index_map;
    int j = 0;
    for (size_t i = 0; i < t_string_with_optionals.size(); ++i) {
        if (translation_result.details_vector[i].epsilon == 0) {
            std::string word = t_string_with_optionals[i];
            t_string.push_back(safe_regex_replace(word, " ", "{SPACE}"));
            remove_asterisk(&t_string.back());
            if (t_string.back() == "") {
                t_string.back() = "{ASTERISK}";
            }
            t_dpnd.push_back(t_dpnd_with_optionals[i]);
            index_map[i] = j;
            j++;
        }
    }
    for (size_t i = 0; i < t_dpnd.size(); ++i) {
        if (t_dpnd[i] != -1) {
            if (index_map.count(t_dpnd[i])) {
                t_dpnd[i] = index_map.at(t_dpnd[i]);
            } else {
                //LOG(WARNING) << "Error: Optional word '" << t_string_with_optionals[t_dpnd[i]] << 
                //        "' has a non-optional child '" << t_string[i] << "'.";
                t_dpnd[i] = -1;
            }
        }
        score.push_back(1.0);
    }
    /*
    for (size_t i = 0; i < t_string_with_optionals.size(); ++i) {
        std::cout << t_string_with_optionals[i] << ":" << dpnd_with_optionals[i] << std::endl;
    }
    for (size_t i = 0; i < t_string.size(); ++i) {
        std::cout << t_string[i] << ":" << dpnd[i] << std::endl;
    }
    */
    assert(t_string.size() == t_dpnd.size());
    assert(t_string.size() == score.size());
    for (size_t i = 0; i < t_string.size(); ++i) {
        *os << t_string[i] << " ";
    }
    *os << "||| ";
    for (size_t i = 0; i < t_dpnd.size(); ++i) {
        *os << t_dpnd[i] << " ";
    }
    *os << "||| ";
    for (size_t i = 0; i < score.size(); ++i) {
        *os << score[i] << " ";
    }
    *os << std::endl;
}
