/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef TRANSLATION_RESULT_HPP__
#define TRANSLATION_RESULT_HPP__

#include <limits>
#include <string>
#include <vector>
#include "decoding/base_lattice.hpp"
#include "hypothesis.hpp"

struct SingleNBestTranslationResult {
    SingleNBestTranslationResult() :
    hypothesis(NULL), bleu(-std::numeric_limits<double>::max()) {}
    std::string      t_string_flat;
    VS               t_string;
    double           score;
    double           score_no_lm;
    FeatureVector    features;
    DpndList         dpnd;
    std::string      debug_information;
    const Hypothesis*      hypothesis;
    double           bleu;
    boost::shared_ptr<boost::ptr_vector<Hypothesis> > intermediate_hypotheses_holder;
    std::vector<const Hypothesis*> initial_hypothesis_list;
    std::vector<latt::WordHypPair> details_vector;
};

typedef std::vector<SingleNBestTranslationResult> NBestTranslationResult;

void remove_asterisk(std::string *str);
void process_optionals_and_print(const SingleNBestTranslationResult &translation_result, std::ostream *ofs);

#endif
