/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/regex.hpp>
#include <boost/regex/icu.hpp>
#include <cstdio>
#include <map>
#include <set>
#include <string>
#include <vector>

#include "utils.hpp"
#include "dictionary.hpp"

std::vector<std::string> split(std::string s, std::string c) {
    std::vector<std::string> ret;
    int len = s.length();
    std::string::size_type n = 0;
    for (int i = 0; i <= len; i = n + c.length()) {
        n = s.find(c, i);
        if (n == std::string::npos) n = s.length();
        std::string tmp = s.substr(i, n - i);
        ret.push_back(tmp);
    }
    return ret;
}

bool word_overlap(const std::vector<WordIndex>& list1, const std::vector<WordIndex>& list2) {
    for (std::vector<WordIndex>::const_iterator it1 = list1.begin(); it1 != list1.end(); it1++) {
        if (grep(*it1, list2).size() > 0) return true;
    }
    return false;
}

bool word_overlap(const PatternType& list1, const std::vector<WordIndex>& list2) {
    for (PatternType::const_iterator it1 = list1.begin(); it1 != list1.end(); it1++) {
        if (grep(*it1, list2).size() > 0) return true;
    }
    return false;
}

bool word_overlap(const std::vector<WordIndex>& list1, const std::set<const DtreeNode*> &list2) {
    for (std::set<const DtreeNode*>::const_iterator it2 = list2.begin(); it2 != list2.end(); it2++) {
        if (grep((*it2)->num, list1).size() > 0) return true;
    }
    return false;
}

bool word_overlap(const std::vector<const DtreeNode*> &list1, const std::set<const DtreeNode*> &list2) {
    for (std::set<const DtreeNode*>::const_iterator it2 = list2.begin(); it2 != list2.end(); it2++) {
        if (grep(const_cast<const DtreeNode*>(*it2), list1).size() > 0) return true;
    }
    return false;
}

int exec_cmd(const char* cmd, std::string* output) {
    FILE* fp = popen(cmd, "r");
    if (fp == NULL) {
        return -1;
    }

    char line[1024];
    while (fgets(line, sizeof(line), fp)) {
        *output += line;
    }
    return pclose(fp);
}

std::string safe_regex_replace(const std::string &input, const std::string &pattern, const std::string &replace) {
    const boost::u32regex e = boost::make_u32regex(pattern);
    UnicodeString unicode_input(input.c_str(), "utf-8");
    UnicodeString unicode_replace(replace.c_str(), "utf-8");
    UnicodeString unicode_result = boost::u32regex_replace(unicode_input, e, unicode_replace);
    int32_t convertedLength = unicode_result.extract(0, unicode_result.length(), 0, "utf-8"); 
    char* result_char = new char[convertedLength + 1];
    unicode_result.extract(0, unicode_result.length(), result_char, "utf-8"); 
    std::string result(result_char);
    return result;
}

void replace_in_place(std::string &input, const std::string &search, const std::string &replace) {
    size_t pos = 0;
    while ((pos = input.find(search, pos)) != std::string::npos) {
        input.replace(pos, search.length(), replace);
        pos += replace.length();
    }
}

void convert_bracket_symbols(std::string &input_sentence) {
    replace_in_place(input_sentence, "-LRB-", "(");
    replace_in_place(input_sentence, "-RRB-", ")");
    replace_in_place(input_sentence, "-LSB-", "[");
    replace_in_place(input_sentence, "-RSB-", "]");
    replace_in_place(input_sentence, "-LCB-", "{");
    replace_in_place(input_sentence, "-RCB-", "}");
}
