/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef UTILS_HPP__
#define UTILS_HPP__

#include <string>
#include <vector>
#include <sstream>
#include <set>
#include "def.hpp"
#include "word.hpp"
#include "boost/lexical_cast.hpp"

template <typename T> std::string join(const std::string &c, const std::vector<T>& body) {
    std::string result = "";
    typename std::vector<T>::const_iterator it = body.begin();
    while (it != body.end()) {
        if (result != "") result += c;
        result += boost::lexical_cast<std::string>(*it);
        it++;
    }
    return result;
}

template <typename T> std::vector<T> grep(const T& self, const std::vector<T>& check) {
    std::vector<T> result;
    size_t end = check.size();
    for (size_t i = 0; i < end; i++) {
        if (self == check[i]) result.push_back(check[i]);
    }
    return result;
}

std::vector<std::string> split(const std::string s, const std::string c);
bool word_overlap(const std::vector<WordIndex> &list1, const std::vector<WordIndex> &list2);
bool word_overlap(const PatternType& list1, const std::vector<WordIndex>& list2);
bool word_overlap(const std::vector<WordIndex> &list1, const std::set<const DtreeNode*> &list2);
bool word_overlap(const std::vector<const DtreeNode*> &list1, const std::set<const DtreeNode*> &list2);

int exec_cmd(const char* cmd, std::string* output);
std::string safe_regex_replace(const std::string &input, const std::string &pattern, const std::string &replace);

void replace_in_place(std::string &input, const std::string &search, const std::string &replace);
void convert_bracket_symbols(std::string &input_sentence);

#endif
