/*
KyotoEBMT Example-Based Machine Translation System
Copyright (C) 2014 John Richardson, Fabien Cromieres, 
Toshiaki Nakazawa and Sadao Kurohashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*! @file
  This file contains the definition of Word structure
 */

#ifndef WORD_INCLUDED
#define WORD_INCLUDED

#include <boost/serialization/string.hpp>
#include <boost/serialization/vector.hpp>
#include <vector>
#include "def.hpp"

//! The Word structure defines the member fields of each word.
struct WordData {
    Content  content;                   //!< surface string
    Lem      lem;                       //!< lemma
    POS      pos;                       //!< part-of-speech
    ContentP content_p;                 //!< flag if the word is a content word (== 1) or function word (== 0) or others (== -1)
    Category cat;                       //!< phrasal category which the word belongs to
    Type     type;                      //!< type of dependency (mainly used for Japanese)
};

//! The DtreeNode structure defines the nodes for a Dtree.
struct DtreeNode {
    DtreeNode() : align_index(-1) {}            //!< constructor which sets -1 to align_index
    WordData                 word;              //!< Word containing information for this DtreeNode
    Dpnd                     dpnd;              //!< index of the parent DtreeNode
    WordIndex                num;               //!< word index
    AlignIndex               align_index;       //!< index of alignment information stored in ParaSentence::s_t_match_
    Depth                    depth;             //!< depth of the DtreeNode in dependency tree from the root (== 0)
    DtreeNode*               dpnd_ref;          //!< pointer to the parent DtreeNode
    std::vector<DtreeNode*>  pre_children_ref;  //!< vector of pointers to the pre-children DtreeNodes
    std::vector<DtreeNode*>  post_children_ref; //!< vector of pointers to the post-children DtreeNodes
};

#endif
