# KyotoEBMT Example-Based Machine Translation System
# Copyright (C) 2014 John Richardson, Fabien Cromieres, 
# Toshiaki Nakazawa and Sadao Kurohashi

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# TODO allow serialisation of only some hypotheses
# TODO run eval on selected weight files

import os
import subprocess
import sys
import time
import thread
import urwid
import config

def gui_data(input, defaults, main, main_widget):
    radio_button_group = []
    button2name = {}
    def change_data_set(button, state):
        if state:
            data_set = button2name[button]
            input["DATA_SET"] = data_set
            for attr, val in defaults[data_set].iteritems():
                input[attr] = val
            textbox_dev_input.set_edit_text(input["DEV_INPUT"])
            textbox_test_input.set_edit_text(input["TEST_INPUT"])
            textbox_tr_language.set_edit_text(str(input["TR_LANGUAGE"]))
            textbox_tr_db_filenames.set_edit_text(input["TR_DB_FILENAMES"])
            textbox_default_dpnd.set_edit_text(input["DEFAULT_DPND"])
            textbox_lex_trans_prob_filename.set_edit_text(input["LEX_TRANS_PROB_FILENAME"])
            textbox_dic_words_filename.set_edit_text(input["DIC_WORDS_FILENAME"])
            textbox_dic_punc_filename.set_edit_text(input["DIC_PUNC_FILENAME"])
            textbox_dev_prefix.set_edit_text(input["DEV_PREFIX"])
            textbox_dev_ref.set_edit_text(input["DEV_REF"])
            textbox_test_prefix.set_edit_text(input["TEST_PREFIX"])
            textbox_test_ref.set_edit_text(input["TEST_REF"])
            textbox_lm.set_edit_text(input["LM"])
            if input["OUTPUT_HANKAKU"]:
                radio_kaku1.set_state(False)
                radio_kaku2.set_state(True)
            else:
                radio_kaku1.set_state(True)
                radio_kaku2.set_state(False)
    for data_set in defaults:
        state = False
        if data_set == input["DATA_SET"]:
            state = True
        button = urwid.RadioButton(radio_button_group, data_set, state, on_state_change=change_data_set)
        button2name[button] = data_set
    temp = [urwid.Text(("attr", "Defaults:"))]
    temp.extend(radio_button_group)
    radio_lang_column = urwid.Columns(temp)
    textbox_dev_input = urwid.Edit(("attr", "Input XML files (dev): "), edit_text=input["DEV_INPUT"])
    textbox_test_input = urwid.Edit(("attr", "Input XML files (test): "), edit_text=input["TEST_INPUT"])
    textbox_tr_language = urwid.Edit(("attr", "TR++ language: "), edit_text=str(input["TR_LANGUAGE"]))
    textbox_tr_db_filenames = urwid.Edit(("attr", "TR++ DB prefix: "), edit_text=input["TR_DB_FILENAMES"])
    textbox_default_dpnd = urwid.Edit(("attr", "Default dependency DB: "), edit_text=input["DEFAULT_DPND"])
    textbox_lex_trans_prob_filename = urwid.Edit(("attr", "Lexical translation probability table: "), edit_text=input["LEX_TRANS_PROB_FILENAME"])
    textbox_dic_words_filename = urwid.Edit(("attr", "Dictionary (words): "), edit_text=input["DIC_WORDS_FILENAME"])
    textbox_dic_punc_filename = urwid.Edit(("attr", "Dictionary (punctuation): "), edit_text=input["DIC_PUNC_FILENAME"])
    textbox_dev_prefix = urwid.Edit(("attr", "Hypothesis prefix (dev): "), edit_text=input["DEV_PREFIX"])
    textbox_test_prefix = urwid.Edit(("attr", "Hypothesis prefix (test): "), edit_text=input["TEST_PREFIX"])
    textbox_dev_ref = urwid.Edit(("attr", "Translation reference (dev): "), edit_text=input["DEV_REF"])
    textbox_test_ref = urwid.Edit(("attr", "Translation reference (test): "), edit_text=input["TEST_REF"])
    textbox_lm = urwid.Edit(("attr", "Language model: "), edit_text=input["LM"])
    # Hankaku/zenkaku
    radio_button_group2 = []
    state1 = True
    state2 = False
    if input["OUTPUT_HANKAKU"]:
        state1 = False
        state2 = True
    radio_kaku1 = urwid.RadioButton(radio_button_group2, "Full-width", state1)
    radio_kaku2 = urwid.RadioButton(radio_button_group2, "Half-width", state2)
    temp = [urwid.Text(("attr", "Output format: ")), radio_kaku1, radio_kaku2]
    radio_kaku_column = urwid.Columns(temp)
    # Done
    def done(event):
        input["DEV_INPUT"] = textbox_dev_input.get_edit_text()
        input["TEST_INPUT"] = textbox_test_input.get_edit_text()
        input["TR_LANGUAGE"] = textbox_tr_language.get_edit_text()
        input["TR_DB_FILENAMES"] = textbox_tr_db_filenames.get_edit_text()
        input["DEFAULT_DPND"] = textbox_default_dpnd.get_edit_text()
        input["LEX_TRANS_PROB_FILENAME"] = textbox_lex_trans_prob_filename.get_edit_text()
        input["DIC_WORDS_FILENAME"] = textbox_dic_words_filename.get_edit_text()
        input["DIC_PUNC_FILENAME"] = textbox_dic_punc_filename.get_edit_text()
        input["DEV_PREFIX"] = textbox_dev_prefix.get_edit_text()
        input["TEST_PREFIX"] = textbox_test_prefix.get_edit_text()
        input["DEV_REF"] = textbox_dev_ref.get_edit_text()
        input["TEST_REF"] = textbox_test_ref.get_edit_text()
        input["LM"] = textbox_lm.get_edit_text()
        if radio_kaku1.get_state():
            input["OUTPUT_HANKAKU"] = False
        else:
            input["OUTPUT_HANKAKU"] = True
        main.original_widget = main_widget
    button_done = urwid.Button("Done")
    urwid.connect_signal(button_done, 'click', done)
    # Draw
    pile = urwid.Pile([div,
            urwid.Text(("bold", "Language Pair")), radio_lang_column, div,
            urwid.Text(("bold", "Serialization")), 
                       textbox_dev_input, textbox_test_input, textbox_tr_language,
                       textbox_tr_db_filenames, textbox_default_dpnd, textbox_lex_trans_prob_filename,
                       textbox_dic_words_filename, textbox_dic_punc_filename, div,
            urwid.Text(("bold", "Translation")), 
                       textbox_dev_prefix, textbox_dev_ref,
                       textbox_test_prefix, textbox_test_ref, 
                       textbox_lm, radio_kaku_column, div,
            div, button_done])
    top = urwid.Filler(pile, valign='top')
    main.original_widget = top

def gui_settings(settings, main, main_widget):
    textbox_n_best = urwid.Edit(("attr", "Length of n-best: "), edit_text=str(settings["N_BEST"]))
    textbox_n_best_rerank = urwid.Edit(("attr", "Length of n-best (reranking): "), edit_text=str(settings["N_BEST_RERANK"]))
    textbox_beam_width = urwid.Edit(("attr", "Beam width: "), edit_text=str(settings["BEAM_WIDTH"]))
    textbox_ebmt_options = urwid.Edit(("attr", "EBMT options: "), edit_text=settings["EBMT_OPTIONS"])
    textbox_config = urwid.Edit(("attr", "Config file: "), edit_text=settings["CONFIG"])
    # USE_GXP
    radio_button_group_gxp = []
    gxp1 = True
    gxp2 = False
    if settings["USE_GXP"]:
        gxp1 = False
        gxp2 = True
    radio_gxp1 = urwid.RadioButton(radio_button_group_gxp, "Local", gxp1)
    radio_gxp2 = urwid.RadioButton(radio_button_group_gxp, "GXP", gxp2)
    temp = [urwid.Text(("attr", "Tuning mode: ")), radio_gxp1, radio_gxp2]
    radio_gxp_column = urwid.Columns(temp)
    textbox_gxp_batch_size = urwid.Edit(("attr", "GXP batch size: "), edit_text=str(settings["GXP_BATCH_SIZE"]))
    textbox_nb_threads = urwid.Edit(("attr", "Number of threads: "), edit_text=str(settings["NB_THREADS"]))
    textbox_first_iteration = urwid.Edit(("attr", "First iteration: "), edit_text=str(settings["FIRST_ITERATION"]))
    textbox_last_iteration = urwid.Edit(("attr", "Last iteration: "), edit_text=str(settings["LAST_ITERATION"]))
    textbox_initial_weights = urwid.Edit(("attr", "Initial weights: "), edit_text=settings["INITIAL_WEIGHTS"])
    textbox_out_dir = urwid.Edit(("attr", "Output directory: "), edit_text=settings["OUT_DIR"])
    textbox_nb_sentences = urwid.Edit(("attr", "Number of sentences: "), edit_text=str(settings["NB_SENTENCES"]))

    textbox_ebmt = urwid.Edit(("attr", "KyotoEBMT: "), edit_text=settings["EBMT"])
    textbox_ebmt_gxp = urwid.Edit(("attr", "KyotoEBMT (for GXP): "), edit_text=settings["EBMT_GXP"])
    textbox_bleu = urwid.Edit(("attr", "BLEU: "), edit_text=settings["BLEU"])
    textbox_extractor = urwid.Edit(("attr", "Extractor: "), edit_text=settings["EXTRACTOR"])
    textbox_pro = urwid.Edit(("attr", "PRO: "), edit_text=settings["PRO"])
    textbox_megam = urwid.Edit(("attr", "MegaM: "), edit_text=settings["MEGAM"])
    textbox_gxp = urwid.Edit(("attr", "GXP: "), edit_text=settings["GXP"])

    # Done
    def done(event):
        settings["N_BEST"] = int(textbox_n_best.get_edit_text())
        settings["N_BEST_RERANK"] = int(textbox_n_best_rerank.get_edit_text())
        settings["BEAM_WIDTH"] = int(textbox_beam_width.get_edit_text())
        settings["EBMT_OPTIONS"] = textbox_ebmt_options.get_edit_text()
        settings["CONFIG"] = textbox_config.get_edit_text()
        # EVAL_TEST
        if radio_gxp1.get_state():
            settings["USE_GXP"] = False
        else:
            settings["USE_GXP"] = True
        settings["GXP_BATCH_SIZE"] = int(textbox_gxp_batch_size.get_edit_text())
        settings["NB_THREADS"] = int(textbox_nb_threads.get_edit_text())
        settings["FIRST_ITERATION"] = int(textbox_first_iteration.get_edit_text())
        settings["LAST_ITERATION"] = int(textbox_last_iteration.get_edit_text())
        settings["INITIAL_WEIGHTS"] = textbox_initial_weights.get_edit_text()
        settings["OUT_DIR"] = textbox_out_dir.get_edit_text()
        settings["NB_SENTENCES"] = int(textbox_nb_sentences.get_edit_text())
        settings["EBMT"] = textbox_ebmt.get_edit_text()
        settings["EBMT_GXP"] = textbox_ebmt_gxp.get_edit_text()
        settings["BLEU"] = textbox_bleu.get_edit_text()
        settings["EXTRACTOR"] = textbox_extractor.get_edit_text()
        settings["PRO"] = textbox_pro.get_edit_text()
        settings["MEGAM"] = textbox_megam.get_edit_text()
        settings["GXP"] = textbox_gxp.get_edit_text()
        main.original_widget = main_widget
    button_done = urwid.Button("Done")
    urwid.connect_signal(button_done, 'click', done)
    # Draw
    pile = urwid.Pile([div,
            urwid.Text(("bold", "Decoder Settings")), textbox_n_best, textbox_n_best_rerank,
                       textbox_beam_width, textbox_ebmt_options, textbox_config, div,
            urwid.Text(("bold", "Tuning Settings")), radio_gxp_column,
                       textbox_gxp_batch_size, textbox_nb_threads,
                       textbox_first_iteration, textbox_last_iteration, div,
            urwid.Text(("bold", "Input/Output Settings")), textbox_initial_weights,
                       textbox_out_dir, textbox_nb_sentences, div,
            urwid.Text(("bold", "External Tool Paths")), textbox_ebmt, textbox_ebmt_gxp,
                       textbox_bleu, textbox_extractor, textbox_pro, 
                       textbox_megam, textbox_gxp, div,
            div, button_done])
    top = urwid.Filler(pile, valign='top')
    main.original_widget = top

def gui_tuning_warning(input, settings, main, main_widget):
    def cancel(event):
        main.original_widget = main_widget
    def ok(event):
        gui_tuning(input, settings, main, main_widget)
    text = "===============\n=== Warning ===\n===============\n\n\n"
    text += "1. Are you sure you want to delete the tuning directory?\n"
    text += "[Output directory: %s]\n\n" % (settings["OUT_DIR"])
    text += "2. Have you compiled the latest version of KyotoEBMT?\n"
    text += "[Last modified: %s]\n\n" % (time.ctime(os.path.getmtime(settings["EBMT"])))
    text += "3. Have you serialized the latest hypotheses?\n"
    text += "[Last modified: %s]\n\n" % (time.ctime(os.path.getmtime(input["DEV_PREFIX"] + "00000.hyp")))
    text += "4. [GXP] Have you copied the hypotheses to /data?\n\n"
    response = urwid.Text([text])
    button_cancel = urwid.Button("Cancel")
    button_ok = urwid.Button("OK")
    urwid.connect_signal(button_cancel, "click", cancel)
    urwid.connect_signal(button_ok, "click", ok)
    #main.original_widget = urwid.Filler(urwid.Pile([response, div,
    #        urwid.Columns([button_cancel, div, div, div, button_ok])]))
    main.original_widget = urwid.Filler(urwid.Pile([response, div, button_cancel, button_ok]))

def gui_tuning(input, settings, main, main_widget):
    update_nb_sentences(input, settings)
    # Status
    iteration = 0
    text_status = urwid.Text("Status: Initializing...")
    text_iteration = urwid.Text("Iteration: 0 / %d" % (settings["LAST_ITERATION"] - 1))
    text_init_bleu = urwid.Text("Init BLEU@dev: 0.00")
    text_last_bleu = urwid.Text("Last BLEU@dev: 0.00")
    text_best_bleu = urwid.Text("Best BLEU@dev: 0.00")
    # Progress
    progress_iteration = urwid.ProgressBar("pg normal", "pg complete")
    progress_iteration.set_completion(0)
    temp = [urwid.Text("Translation Progress: "), progress_iteration]
    progress_iteration_column = urwid.Columns(temp)
    progress_total = urwid.ProgressBar("pg normal", "pg complete")
    progress_total.set_completion(0)
    temp = [urwid.Text("Tuning Progress: "), progress_total]
    progress_total_column = urwid.Columns(temp)
    # Graph
    text_graph = []
    for i in xrange(10):
        text_graph.append(urwid.Text(""))
    # Done
    def done(event):
        main.original_widget = main_widget
    button_done = urwid.Button("Done")
    urwid.connect_signal(button_done, 'click', done)
    # Draw
    pile_list = [div, text_status, text_iteration, text_init_bleu, text_last_bleu, 
                       text_best_bleu, progress_iteration_column, 
                       progress_total_column, div]
    pile_list.extend(text_graph)
    pile_list.extend([div, button_done])
    pile = urwid.Pile(pile_list)
    text_log = urwid.Text("")
    top = urwid.Filler(urwid.Pile([pile, div, text_log]), valign='top')
    main.original_widget = top
    widgets = (text_status, text_iteration, text_init_bleu, text_last_bleu, text_best_bleu, 
               progress_iteration, progress_total, text_graph, text_log)
    thread.start_new_thread(run_tuning, (widgets, input, settings, main, main_widget))

def gui_output_rerank(input, settings, main, main_widget):
    update_nb_sentences(input, settings)
    text_status = urwid.Text("Status: Generating reranking data...")
    progress_total = urwid.ProgressBar("pg normal", "pg complete")
    progress_total.set_completion(0)
    temp = [urwid.Text("Tuning Progress: "), progress_total]
    progress_total_column = urwid.Columns(temp)
    def done(event):
        main.original_widget = main_widget
    button_done = urwid.Button("Done")
    urwid.connect_signal(button_done, 'click', done)
    pile_list = [div, text_status, progress_total_column, div, button_done]
    pile = urwid.Pile(pile_list)
    top = urwid.Filler(pile, valign='top')
    main.original_widget = top
    #widgets = (text_status, text_iteration, text_init_bleu, text_last_bleu, text_best_bleu, 
    #           progress_iteration, progress_total, text_graph)
    widgets = (text_status)
    thread.start_new_thread(run_output_rerank, (widgets, input, settings))

def gui_serialize_warning(mode, input, settings, main, main_widget):
    def cancel(event):
        main.original_widget = main_widget
    def ok(event):
        gui_serialize(mode, input, settings, main, main_widget)
    text = "===============\n=== Warning ===\n===============\n\n\n"
    text += "1. Are you sure you want to overwrite all hypotheses?\n"
    if mode == "dev":
        text += "[Output directory: %s]\n\n" % (input["DEV_PREFIX"])
    else:
        text += "[Output directory: %s]\n\n" % (input["TEST_PREFIX"])
    text += "2. Have you compiled the latest version of KyotoEBMT?\n"
    text += "[Last modified: %s]\n\n" % (time.ctime(os.path.getmtime(settings["EBMT"])))
    response = urwid.Text([text])
    button_cancel = urwid.Button("Cancel")
    button_ok = urwid.Button("OK")
    urwid.connect_signal(button_cancel, "click", cancel)
    urwid.connect_signal(button_ok, "click", ok)
    main.original_widget = urwid.Filler(urwid.Pile([response, div, button_cancel, button_ok]))

def gui_serialize(mode, input, settings, main, main_widget):
    if settings["USE_GXP"]:
        shell_call("cp %s %s" % (settings["EBMT_GXP"], settings["EBMT_TUNING"]))
    else:
        shell_call("cp %s %s" % (settings["EBMT"], settings["EBMT_TUNING"]))
    text_status = urwid.Text("Serializing hypotheses [0/0].\nPlease wait...")
    progress_bar = urwid.ProgressBar("pg normal", "pg complete")
    progress_bar.set_completion(0)
    temp = [urwid.Text("Progress: "), progress_bar]
    progress_bar_column = urwid.Columns(temp)
    # Done
    def done(event):
        main.original_widget = main_widget
    button_done = urwid.Button("Done")
    urwid.connect_signal(button_done, 'click', done)
    # Draw
    pile = urwid.Pile([div, text_status, progress_bar_column, div, button_done])
    text_log = urwid.Text("")
    top = urwid.Filler(urwid.Pile([pile, div, text_log]), valign='top')
    main.original_widget = top
    widgets = text_status, progress_bar, text_log
    thread.start_new_thread(serialize, (mode, widgets, input, settings))

def gui_eval(input, settings, main, main_widget, use_web=False):
    if settings["USE_GXP"]:
        shell_call("cp %s %s" % (settings["EBMT_GXP"], settings["EBMT_TUNING"]))
    else:
        shell_call("cp %s %s" % (settings["EBMT"], settings["EBMT_TUNING"]))
    text_status = urwid.Text("Running evaluation.\nPlease wait...")
    progress_bar = urwid.ProgressBar("pg normal", "pg complete")
    progress_bar.set_completion(0)
    temp = [urwid.Text("Progress: "), progress_bar]
    progress_bar_column = urwid.Columns(temp)
    # Done
    def done(event):
        main.original_widget = main_widget
    button_done = urwid.Button("Done")
    urwid.connect_signal(button_done, 'click', done)
    # Draw
    pile = urwid.Pile([div, text_status, progress_bar_column, div, button_done])
    text_log = urwid.Text("")
    top = urwid.Filler(urwid.Pile([pile, div, text_log]), valign='top')
    main.original_widget = top
    widgets = text_status, progress_bar, text_log
    thread.start_new_thread(eval, (widgets, input, settings, use_web))

def gui_main(input, defaults, settings):
    global main_loop
    choices = ["Select Data", "Change Settings", "Serialize Hypotheses (Dev)",
               "Serialize Hypotheses (Test)", "Run Tuning", "Generate Reranking Data", 
               "Evaluate", "Build Web Output", "Save Results", "Quit"]
    def menu(title, choices):
        body = [urwid.Text(title), urwid.Divider()]
        for c in choices:
            button = urwid.Button(c)
            urwid.connect_signal(button, 'click', item_chosen, c)
            body.append(urwid.AttrMap(button, None, focus_map='reversed'))
        return urwid.ListBox(urwid.SimpleFocusListWalker(body))
    def item_chosen(button, choice):
        if choice == "Select Data":
            gui_data(input, defaults, main, main.original_widget)
        elif choice == "Change Settings":
            gui_settings(settings, main, main.original_widget)
        elif choice == "Serialize Hypotheses (Dev)":
            gui_serialize_warning("dev", input, settings, main, main.original_widget)
        elif choice == "Serialize Hypotheses (Test)":
            gui_serialize_warning("test", input, settings, main, main.original_widget)
        elif choice == "Run Tuning":
            gui_tuning_warning(input, settings, main, main.original_widget)
        elif choice == "Generate Reranking Data":
            gui_output_rerank(input, settings, main, main.original_widget)
        elif choice == "Evaluate":
            gui_eval(input, settings, main, main.original_widget)
        elif choice == "Build Web Output":
            gui_eval(input, settings, main, main.original_widget, use_web=True)
        elif choice == "Save Results":
            save(input, settings, main, main.original_widget)
        elif choice == "Quit":
            raise urwid.ExitMainLoop()
    header = "=== KyotoEBMT Tuner v0.4 ==="
    main = urwid.Padding(menu(("bold", header), choices), left=2, right=2)
    top = urwid.Overlay(main, urwid.SolidFill(u'\N{MEDIUM SHADE}'),
        align='center', width=('relative', 80),
        valign='middle', height=('relative', 80),
        min_width=20, min_height=9)
    main_loop = urwid.MainLoop(top, screen=scr, palette=palette)
    main_loop.run()

def check_tools(settings):
    for tool in [settings["EBMT"], settings["EXTRACTOR"], settings["PRO"], settings["MEGAM"], settings["GXP"], settings["BLEU"]]:
        if not os.path.exists(tool):
            return "Cannot find required tool: " + tool
    return 

def run_tuning(widgets, input, settings, main, main_widget):
    # Check required tools are available.
    error = check_tools(settings)
    if error:
        def done(event):
            main.original_widget = main_widget
        response = urwid.Text(["Error\n", error, "\n"])
        button_done = urwid.Button("Back")
        urwid.connect_signal(button_done, "click", done)
        main_loop.draw_screen()
        main.original_widget = urwid.Filler(urwid.Pile([response,
                urwid.AttrMap(button_done, None, focus_map="reversed")]))
        main_loop.draw_screen()
        return
    prepare_tuning(settings)
    tune(widgets, input, settings)

def date():
    return time.strftime("%a, %d %b %Y %H:%M:%S")

def log(text):
    with open(settings["LOG"], "a") as file_out:
        time_str = date()
        file_out.write("[%s] %s\n" % (time_str, text))

def log_tail():
    result = ""
    shell_call("tail -n5 %s > %s.tail" % (settings["LOG"], settings["LOG"]), enable_log=False)
    with open("%s.tail" % settings["LOG"]) as file_in:
        for line in file_in:
            result += line
    return result

def shell_call(call, enable_log=True):
    if enable_log:
        log("$ %s" % (call)) 
    subprocess.call("%s 2>>%s" % (call, settings["LOG"]), shell=True)

def prepare_tuning(settings):
    if settings["FIRST_ITERATION"] != 0:
        return
    shell_call("rm -rf %s" % (settings["OUT_DIR"]))
    shell_call("mkdir %s" % (settings["OUT_DIR"]))
    shell_call("cp %s %s/run0.weights.dat" % (settings["INITIAL_WEIGHTS"], settings["OUT_DIR"]))
    if settings["USE_GXP"]:
        shell_call("cp %s %s" % (settings["EBMT_GXP"], settings["EBMT_TUNING"]))
    else:
        shell_call("cp %s %s" % (settings["EBMT"], settings["EBMT_TUNING"]))

def generate_gxp_task(iteration, sentence_i, batch, batch_i):
    command = "nice -19 %s %s " % (settings["EBMT_TUNING"], settings["EBMT_OPTIONS"])
    command += "-c %s " % settings["CONFIG"]
    command += "--input_mode hypothesis "
    command += "--output_mode tuning "
    command += "--nb_threads %d " % (settings["NB_THREADS"])
    command += "--weight %s/run%d.weights.dat " % (settings["OUT_DIR"], iteration)
    command += "--sentence_id %d " % (sentence_i)
    command += "--source_language %s " % (input["LANG_S"])
    command += "--target_language %s " % (input["LANG_T"])
    command += "--lm %s " % (input["LM"])
    command += "--n_best_length %d " % (settings["N_BEST"])
    command += "--beam_width %d " % (settings["BEAM_WIDTH"])
    command += "--input %s " % (batch)
    command += "> %s/run%d.n_best.part%03d.dat " % (settings["OUT_DIR"], iteration, batch_i)
    command += "\n"
    return command
            
def generate_task(iteration, fold, rerank, input, settings):
    input_list = "%s/hypotheses.lst" % settings["OUT_DIR"]
    if fold == "dev":
        shell_call("ls %s*.hyp > %s" % (input["DEV_PREFIX"], input_list))
    elif fold == "test":
        shell_call("ls %s*.hyp > %s" % (input["TEST_PREFIX"], input_list))
    else:
        log("ERROR: invalid fold name '%d'" % fold)
        quit()
    command = "nice -19 %s %s " % (settings["EBMT_TUNING"], settings["EBMT_OPTIONS"])
    command += "-c %s " % settings["CONFIG"]
    command += "--input_mode hypothesis "
    command += "--nb_threads %d " % (settings["NB_THREADS"])
    command += "--sentence_id 0 "
    command += "--source_language %s " % (input["LANG_S"])
    command += "--target_language %s " % (input["LANG_T"])
    command += "--lm %s " % (input["LM"])
    command += "--beam_width %d " % (settings["BEAM_WIDTH"])
    command += "--input_list %s " % input_list
    if rerank:
        command += "--output_mode tuning-rerank "
        command += "--weight %s/best.weights.dat " % settings["OUT_DIR"]
        command += "--n_best_length %d " % settings["N_BEST_RERANK"]
        command += "--reconstruct_final_hypotheses "
        command += "> %s/rerank.output.%s " % (settings["OUT_DIR"], fold)
    else:
        command += "--output_mode tuning "
        command += "--weight %s/run%d.weights.dat " % (settings["OUT_DIR"], iteration)
        command += "--n_best_length %d " % (settings["N_BEST"])
        command += "> %s/run%d.n_best.dat " % (settings["OUT_DIR"], iteration)
    return command

def generate_task_serialization(mode, input, settings):
    command = "nice -19 %s %s " % (settings["EBMT_TUNING"], settings["EBMT_OPTIONS"])
    command += "-c %s " % settings["CONFIG"]
    command += "--nb_threads %d " % (settings["NB_THREADS"])
    command += "--weight %s " % (settings["INITIAL_WEIGHTS"])
    command += "--output_mode hypothesis "
    if mode == "dev":
        command += "--hypothesis_output_prefix %s " % (input["DEV_PREFIX"])
    else:
        command += "--hypothesis_output_prefix %s " % (input["TEST_PREFIX"])
    command += "--source_language %s " % (input["LANG_S"])
    command += "--target_language %s " % (input["LANG_T"])
    command += "--tr_language %s " % (input["TR_LANGUAGE"])
    command += "--tr_db_filenames %s " % (input["TR_DB_FILENAMES"])
    command += "--default_dpnd %s " % (input["DEFAULT_DPND"])
    command += "--lexical_translation_prob_filename %s " % (input["LEX_TRANS_PROB_FILENAME"])
    command += "--dic_words_filename %s " % (input["DIC_WORDS_FILENAME"])
    command += "--dic_punc_filename %s " % (input["DIC_PUNC_FILENAME"])
    command += "--lm %s " % (input["LM"])
    if mode == "dev":
        command += "--input %s " % (input["DEV_INPUT"])
    else:
        command += "--input %s " % (input["TEST_INPUT"])
    return command

def generate_task_eval(iteration, input, settings, use_web=False):
    command = "nice -19 %s %s " % (settings["EBMT_TUNING"], settings["EBMT_OPTIONS"])
    command += "-c %s " % settings["CONFIG"]
    command += "--nb_threads %d " % (settings["NB_THREADS"])
    command += "--weight %s/run%d.weights.dat " % (settings["OUT_DIR"], iteration)
    command += "-n %d " % settings["N_BEST"]
    command += "--output_mode eval "
    if use_web:
        command += "--reconstruct_final_hypotheses --web --web_verbose --web_dir %s " % settings["OUT_DIR"]
        command += "--tr_language %s " % (input["TR_LANGUAGE"])
        command += "--tr_db_filenames %s " % (input["TR_DB_FILENAMES"])
        command += "--default_dpnd %s " % (input["DEFAULT_DPND"])
        command += "--lexical_translation_prob_filename %s " % (input["LEX_TRANS_PROB_FILENAME"])
        command += "--dic_words_filename %s " % (input["DIC_WORDS_FILENAME"])
        command += "--dic_punc_filename %s " % (input["DIC_PUNC_FILENAME"])
        command += "--input %s " % (input["TEST_INPUT"])
    else:
        command += "--input_mode hypothesis "
        command += "--input %s*.hyp " % input["TEST_PREFIX"]
    command += "--source_language %s " % input["LANG_S"]
    command += "--target_language %s " % input["LANG_T"]
    command += "--lm %s " % input["LM"]
    command += "--beam_width %d " % settings["BEAM_WIDTH"]
    if not use_web:
        command += "> %s/run%d.test.dat " % (settings["OUT_DIR"], iteration)
    return command

def get_bleu(filename):
    bleu = 0
    with open(filename) as file_in:
        for line in file_in:
            bleu = float(line.strip().split()[2][:-1])
    return bleu

def format_weights(base_filename, megam_filename, output_filename):
    weights = []
    features = []
    with open(base_filename) as file_in:
        for line in file_in:
            name, weight = line.strip().split(" ")
            features.append(name)
            weights.append("0")
    with open(megam_filename) as file_in:
        for line in file_in:
            feature_id, weight = line.strip().split(" ")
            print feature_id, weight
            weights[int(feature_id[1:])] = weight
    with open(output_filename, "w") as file_out:
        for name, weight in zip(features, weights):
            file_out.write(name + " " + weight + "\n")

def extract_top1(filename_in, filename_out):
    top1 = {}
    done = {}
    id = 0
    with open(filename_in) as file_in:
        last_id = -1
        for line in file_in:
            parts = line.strip().split("|||")
            id = int(parts[0].strip())
            translation = parts[1].strip()
            if id != last_id:
                top1[id] = translation
                done[id] = True
            last_id = id
    final_id = id
    with open(filename_out, "w") as file_out:
        for id in xrange(final_id + 1):
            if id not in top1:
                log("ERROR: cannot find translation for sentence: %d" % (id))
                file_out.write("\n")
            else:
                file_out.write(top1[id] + "\n")

def update_gui_status(widgets, text):
    global main_loop
    text_status, text_iteration, text_init_bleu, text_last_bleu, text_best_bleu, \
            progress_iteration, progress_total, text_graph, text_log = widgets
    text_status.set_text("Status: %s" % (text))
    main_loop.draw_screen()

def update_gui_bleu(widgets, bleu_history):
    global main_loop
    text_status, text_iteration, text_init_bleu, text_last_bleu, text_best_bleu, \
            progress_iteration, progress_total, text_graph, text_log = widgets
    min_bleu = 100.0
    max_bleu = 0.0
    best_iteration = 0
    for i, bleu in enumerate(bleu_history):
        if bleu > max_bleu:
            max_bleu = bleu
            best_iteration = i
        if bleu < min_bleu:
            min_bleu = bleu
    for y, row in enumerate(text_graph):
        height = min_bleu + 1.0 / 9 * (max_bleu - min_bleu) * (9 - y)
        text = "%6.2f   " % (height)
        for bleu in bleu_history:
            if bleu >= height:
                text += "X"
            else:
                text += " "
        row.set_text(text)
    text_init_bleu.set_text("Init BLEU@dev: %.2f" % (bleu_history[0]))
    text_last_bleu.set_text("Last BLEU@dev: %.2f" % (bleu_history[-1]))
    text_best_bleu.set_text("Best BLEU@dev: %.2f [@it%d]" % (max_bleu, best_iteration))
    main_loop.draw_screen()

def update_gui_iteration(widgets, iteration, settings):
    global main_loop
    text_status, text_iteration, text_init_bleu, text_last_bleu, text_best_bleu, \
            progress_iteration, progress_total, text_graph, text_log = widgets
    text_iteration.set_text("Iteration: %d / %d [%s]" % (iteration, settings["LAST_ITERATION"] - 1, date()))
    progress = 100.0 * iteration / settings["LAST_ITERATION"]
    progress_total.set_completion(progress)
    main_loop.draw_screen()

def update_gui_done(widgets):
    global main_loop
    text_status, text_iteration, text_init_bleu, text_last_bleu, text_best_bleu, \
            progress_iteration, progress_total, text_graph, text_log = widgets
    text_status.set_text("Status: Done")
    text_iteration.set_text("Iteration: %d iterations complete" % (settings["LAST_ITERATION"]))
    progress_total.set_completion(100.0)
    progress_iteration.set_completion(100.0)
    main_loop.draw_screen()

def check_completion_tuning(widgets, settings):
    global main_loop
    global iteration
    global stop_request_tuning
    text_status, text_iteration, text_init_bleu, text_last_bleu, text_best_bleu, \
            progress_iteration, progress_total, text_graph, text_log = widgets
    progress_iteration.set_completion(0)
    main_loop.draw_screen()
    while not stop_request_tuning:
        time.sleep(REFRESH_TIME)
        last_line = ""
        last_sentence_id = -1
        try:
            with open("%s/run%d.n_best.dat" % (settings["OUT_DIR"], iteration)) as file_in:
                for line in file_in:
                    last_line = line
            last_sentence_id = int(last_line.strip().split("|||")[0].strip())
        except:
            pass
        progress = 100.0 * (last_sentence_id + 1) / settings["NB_SENTENCES"]
        progress_iteration.set_completion(progress)
        text_log.set_text(("text_log", log_tail()))
        main_loop.draw_screen()

def tune(widgets, input, settings):
    global iteration
    global stop_request_tuning
    text_status, text_iteration, text_init_bleu, text_last_bleu, text_best_bleu, \
            progress_iteration, progress_total, text_graph, text_log = widgets
    bleu_history = []
    settings["RESULTS_DEV"] = settings["OUT_DIR"] + "/results_dev.dat"
    thread.start_new_thread(check_completion_tuning, (widgets, settings))
    for iteration in xrange(settings["FIRST_ITERATION"], settings["LAST_ITERATION"]):
        update_gui_iteration(widgets, iteration, settings)
        update_gui_status(widgets, "Translating %d sentences..." % settings["NB_SENTENCES"])
        if settings["USE_GXP"]:
            # Generate tasks file for GXP. 
            batch = ""
            batch_i = 0
            with open(settings["TASKS"], "w") as file_tasks:
                start_i = 0
                for sentence_i in xrange(settings["NB_SENTENCES"]):
                    batch += "%s%05d.hyp " % (input["DEV_PREFIX"], sentence_i)
                    if sentence_i % settings["GXP_BATCH_SIZE"] == settings["GXP_BATCH_SIZE"] - 1:
                        file_tasks.write(generate_gxp_task(iteration, start_i, batch, batch_i))
                        batch = ""
                        batch_i += 1
                        start_i = sentence_i + 1
                file_tasks.write(generate_gxp_task(iteration, start_i, batch, batch_i))
            # Execute GXP tasks.
            shell_call("%s js -a work_file=%s -a cpu_factor=0.125" % \
                    (settings["GXP"], settings["TASKS"]))
            shell_call("cat %s/run%d.n_best.part*.dat > %s/run%d.n_best.dat" % \
                    (settings["OUT_DIR"], iteration, settings["OUT_DIR"], iteration))
        else:
            shell_call(generate_task(iteration, "dev", False, input, settings))

        # Evaluate on dev set
        update_gui_status(widgets, "Evaluating on dev set...")
        extract_top1("%s/run%d.n_best.dat" % (settings["OUT_DIR"], iteration), 
                     "%s/run%d.dev.dat" % (settings["OUT_DIR"], iteration))
        # Convert zenkaku/hankaku
        if input["OUTPUT_HANKAKU"]:
            shell_call("%s < %s/run%d.dev.dat > %s/zen2han.tmp" % (settings["ZEN2HAN"], settings["OUT_DIR"], iteration, settings["OUT_DIR"]))
            shell_call("mv %s/zen2han.tmp %s/run%d.dev.dat" % (settings["OUT_DIR"], settings["OUT_DIR"], iteration))
        # Prepare reference
        shell_call("head -n %d %s > %s/ref.tmp" % (settings["NB_SENTENCES"], input["DEV_REF"], settings["OUT_DIR"]))
        shell_call("%s %s/ref.tmp < %s/run%d.dev.dat > %s/bleu.tmp" % \
                (settings["BLEU"], settings["OUT_DIR"], settings["OUT_DIR"], iteration, settings["OUT_DIR"]))
        bleu = get_bleu("%s/bleu.tmp" % (settings["OUT_DIR"]))
        shell_call("rm %s/ref.tmp" % (settings["OUT_DIR"]))
        shell_call("rm %s/bleu.tmp" % (settings["OUT_DIR"]))
        # Run and parse BLEU
        shell_call("echo '%d %f' >> %s" % (iteration, bleu, settings["RESULTS_DEV"]))
        bleu_history.append(bleu)
        if bleu == max(bleu_history):
            shell_call("cp %s/run%d.weights.dat %s/best.weights.dat" % (settings["OUT_DIR"], iteration, settings["OUT_DIR"]))
        update_gui_bleu(widgets, bleu_history)
        if iteration == settings["LAST_ITERATION"] - 1:
            break
        # Run tuning on dev set
        update_gui_status(widgets, "Preparing tuning...")
        # Run extractor.
        shell_call("nice -19 %s --scconfig 'case:true' \
                --scfile %s/run%d.scores.dat \
                --ffile %s/run%d.features.dat \
                -r %s -n %s/run%d.n_best.dat" % \
                (settings["EXTRACTOR"], settings["OUT_DIR"], iteration, 
                 settings["OUT_DIR"], iteration, input["DEV_REF"], 
                 settings["OUT_DIR"], iteration))
        # Generate list of feature/score files for all iterations.         
        list_of_previous_features_dat = []
        list_of_previous_scores_dat = []
        for num_iter in xrange(iteration + 1):
            list_of_previous_features_dat.append("%s/run%i.features.dat" % (settings["OUT_DIR"], num_iter))
            list_of_previous_scores_dat.append("%s/run%i.scores.dat" % (settings["OUT_DIR"], num_iter))
        scfile_string = " --scfile ".join(list_of_previous_scores_dat)
        ffile_string = " --ffile ".join(list_of_previous_features_dat)
        # Run tuning with PRO.
        update_gui_status(widgets, "Running PRO...")
        shell_call("nice -19 %s -b --scfile %s \
                --ffile %s \
                -o %s/run%d.pro.dat" % \
                (settings["PRO"], scfile_string, ffile_string, 
                 settings["OUT_DIR"], iteration))
        # Optimize weights with MegaM.
        update_gui_status(widgets, "Optimizing weights with MegaM...")
        shell_call("nice -19 %s -fvals -nobias binary \
                %s/run%d.pro.dat > %s/run%d.megam.dat" % \
                (settings["MEGAM"], settings["OUT_DIR"], iteration, 
                 settings["OUT_DIR"], iteration))
        # Convert MegaM's new weights to KyotoEBMT format.
        format_weights("%s/run0.weights.dat" % (settings["OUT_DIR"]),
                "%s/run%d.megam.dat" % (settings["OUT_DIR"], iteration),
                "%s/run%d.weights.dat" % (settings["OUT_DIR"], iteration + 1))
    update_gui_done(widgets)
    stop_request_tuning = True

def check_completion_serialize(mode, widgets):
    global main_loop
    global stop_request_serialize
    text_status, progress_bar, text_log = widgets
    while not stop_request_serialize:
        time.sleep(REFRESH_TIME)
        nb_output = 0
        nb_input = 0
        if mode == "dev":
            nb_output = len(os.listdir(os.path.dirname(input["DEV_PREFIX"])))
            nb_input = len(os.listdir(os.path.dirname(input["DEV_INPUT"])))
        else:
            nb_output = len(os.listdir(os.path.dirname(input["TEST_PREFIX"])))
            nb_input = len(os.listdir(os.path.dirname(input["TEST_INPUT"])))
        progress = 100.0 * nb_output / nb_input
        progress_bar.set_completion(progress)
        text_status.set_text("Serializing hypotheses [%d/%d].\nPlease wait..." % (nb_output, nb_input))
        text_log.set_text(("text_log", log_tail()))
        main_loop.draw_screen()

def serialize(mode, widgets, input, settings):
    global stop_request_serialize
    stop_request_serialize = False
    thread.start_new_thread(check_completion_serialize, (mode, widgets))
    shell_call(generate_task_serialization(mode, input, settings))
    text_status, progress_bar, text_log = widgets
    text_status.set_text("Serialization complete.")
    progress_bar.set_completion(100)
    stop_request_serialize = True

def check_completion_eval(widgets, settings):
    global main_loop
    global iteration
    global stop_request_eval
    text_status, progress_bar, text_log = widgets
    progress_bar.set_completion(0)
    main_loop.draw_screen()
    while not stop_request_eval:
        time.sleep(REFRESH_TIME)
        nb_lines = 0
        try:
            with open("%s/run%d.test.dat" % (settings["OUT_DIR"], iteration)) as file_in:
                for line in file_in:
                    nb_lines += 1
        except:
            pass
        progress = 100.0 #* nb_lines / settings["NB_SENTENCES"] # TODO: this is wrong
        progress_bar.set_completion(progress)
        text_log.set_text(("text_log", log_tail()))
        main_loop.draw_screen()

def eval(widgets, input, settings, use_web=False):
    global iteration
    global stop_request_serialize
    settings["RESULTS_TEST"] = settings["OUT_DIR"] + "/results_test.dat"
    stop_request_serialize = False
    thread.start_new_thread(check_completion_eval, (widgets, settings))
    # Find best iteration
    best_score = 0.0
    best_iteration = 0
    with open("%s/results_dev.dat" % settings["OUT_DIR"]) as file_in:
        for line in file_in:
            it, score = line.strip().split()
            if float(score) > best_score:
                best_score = float(score)
                best_iteration = int(it)
    iteration = best_iteration
    shell_call(generate_task_eval(iteration, input, settings, use_web))
    # Convert zenkaku/hankaku
    if input["OUTPUT_HANKAKU"]:
        shell_call("%s < %s/run%d.test.dat > %s/zen2han.tmp" % 
                   (settings["ZEN2HAN"], settings["OUT_DIR"], iteration, settings["OUT_DIR"]))
        shell_call("mv %s/zen2han.tmp %s/run%d.test.dat" % 
                   (settings["OUT_DIR"], settings["OUT_DIR"], iteration))
    # Calculate BLEU
    shell_call("%s %s < %s/run%d.test.dat > %s/bleu.tmp" % \
            (settings["BLEU"], input["TEST_REF"], settings["OUT_DIR"], iteration, settings["OUT_DIR"]))
    bleu = get_bleu("%s/bleu.tmp" % (settings["OUT_DIR"]))
    shell_call("rm %s/bleu.tmp" % (settings["OUT_DIR"]))
    shell_call("echo '%d %f' >> %s" % (iteration, bleu, settings["RESULTS_TEST"]))
    # Finish off
    text_status, progress_bar, text_log = widgets
    text_status.set_text("Evaluation complete.\nBLEU = %5.2f" % bleu)
    progress_bar.set_completion(100)
    stop_request_serialize = True


def save(input, settings, main, main_widget):
    # Save settings to file
    with open("%s/settings.ini" % settings["OUT_DIR"], "w") as file_out:
        file_out.write("# Input\n")
        for attr, val in input.iteritems():
            if type(val) == int:
                file_out.write('%s=%d\n' % (attr, val))
            else:
                file_out.write('%s="%s"\n' % (attr, val))
        file_out.write("\n# Settings\n")
        for attr, val in settings.iteritems():
            if type(val) == int:
                file_out.write('%s=%d\n' % (attr, val))
            else:
                file_out.write('%s="%s"\n' % (attr, val))
    # Write gnuplot script
    with open("%s/results.gp" % settings["OUT_DIR"], "w") as file_out:
        file_out.write("set t png\n")
        file_out.write("set output 'results.png'\n")
        file_out.write("set xlabel 'Iteration'\n")
        file_out.write("set ylabel 'BLEU'\n")
        file_out.write("set xrange[0:%d]\n" % (settings["LAST_ITERATION"] - 1))
        file_out.write("set yrange[0:40]\n")
        file_out.write("set style data linespoints\n")
        file_out.write("plot 'results_dev.dat' using 1:2 title 'Dev', ")
        file_out.write("'results_test.dat' using 1:2 title 'Test'\n")
    # Create package
    package_dir = settings["OUT_DIR"] + "/package"
    shell_call("mkdir -p %s" % package_dir)
    shell_call("cp %s/run*.weights.dat %s" % (settings["OUT_DIR"], package_dir))
    shell_call("cp %s/run*.dev.dat %s" % (settings["OUT_DIR"], package_dir))
    shell_call("cp %s/run*.test.dat %s" % (settings["OUT_DIR"], package_dir))
    shell_call("cp %s/results_dev.dat %s" % (settings["OUT_DIR"], package_dir))
    shell_call("cp %s/results_test.dat %s" % (settings["OUT_DIR"], package_dir))
    shell_call("cp %s/results.gp %s" % (settings["OUT_DIR"], package_dir))
    shell_call("cp %s/settings.ini %s" % (settings["OUT_DIR"], package_dir))
    shell_call("cp %s %s/ref_dev.txt" % (input["DEV_REF"], package_dir))
    shell_call("cp %s %s/ref_test.txt" % (input["TEST_REF"], package_dir))
    # Completion dialog
    def done(event):
        main.original_widget = main_widget
    title = urwid.Text(["Save Results\n\n", "Results successfully saved.", "\n"])
    button_done = urwid.Button("OK")
    urwid.connect_signal(button_done, "click", done)
    main.original_widget = urwid.Filler(urwid.Pile([title,
            urwid.AttrMap(button_done, None, focus_map="reversed")]))

def update_nb_sentences(input, settings):
    if settings["NB_SENTENCES"] == 0:
        settings["NB_SENTENCES"] = len(os.listdir(os.path.dirname(input["DEV_PREFIX"])))

def run_output_rerank(widgets, input, settings):
    text_status = widgets
    shell_call("cp %s %s" % (settings["EBMT"], settings["EBMT_TUNING"]))
    shell_call(generate_task(settings, "dev", True, input, settings))
    shell_call(generate_task(settings, "test", True, input, settings))
    for fold in ["dev", "test"]:
        text_status.set_text("Splitting %s fold..." % fold)
        with open("%s/rerank.output.%s" % (settings["OUT_DIR"], fold)) as file_in:
            with open("%s/rerank.nbest.%s" % (settings["OUT_DIR"], fold), "w") as file_out_nbest:
                with open("%s/rerank.string.%s" % (settings["OUT_DIR"], fold), "w") as file_out_string:
                    with open("%s/rerank.tree.%s" % (settings["OUT_DIR"], fold), "w") as file_out_tree:
                        with open("%s/rerank.raw_string.%s" % (settings["OUT_DIR"], fold), "w") as file_out_raw_string:
                            for i, line in enumerate(file_in):
                                if i % 2 == 0:
                                    file_out_nbest.write(line.strip() + "\n")
                                    file_out_string.write(line.strip().split("|||")[1].strip() + "\n")
                                else:
                                    file_out_tree.write(line.strip() + "\n")
                                    file_out_raw_string.write(line.strip().split("|||")[0].strip() + "\n")
    text_status.set_text("Status: Done!")
    main_loop.draw_screen()


REFRESH_TIME = 1

input, defaults = config.load_defaults("defaults.ini")
settings = config.load_settings("settings.ini")

settings["EBMT_TUNING"] = "./KyotoEBMT-tuning"
settings["LOG"] = "tuner.log"

palette = [('pg normal', 'white', 'black', 'standout'),
           ('pg complete', 'white', 'dark magenta'),
           ('text_log', 'light green', ''),
           ('bold', 'default,bold', 'default', 'bold'),
           ('attr', 'light green', 'default', ''),
           ('reversed', 'standout', '')]
div = urwid.Divider()
main_loop = None
main = None
scr = urwid.raw_display.Screen()
stop_request_serialize = False
stop_request_tuning = False
stop_request_eval = False
iteration = 0
gui_main(input, defaults, settings)

