# KyotoEBMT Example-Based Machine Translation System
# Copyright (C) 2014 John Richardson, Fabien Cromieres, 
# Toshiaki Nakazawa and Sadao Kurohashi

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

#!/usr/bin/env perl

use strict;
use utf8;
use Getopt::Long;

binmode STDOUT, ":utf8";

my $opt = {};
GetOptions($opt, 'fertility');

my @sid;
my @sfreq;
open(SID, "<:utf8", $ARGV[0]) || die "$!: $ARGV[0]\n";
while (<SID>) {
    chomp;
    my ($index, $word, $freq) = split(/\t/);
    $sid[$index] = $word;
    $sfreq[$index] = $freq;
}
close SID;

my @tid;
my @tfreq;
open(TID, "<:utf8", $ARGV[1]) || die "$!: $ARGV[1]\n";
while (<TID>) {
    chomp;
    my ($index, $word, $freq) = split(/\t/);
    $tid[$index] = $word;
    $tfreq[$index] = $freq;
}
close TID;
$tid[0] = 'NULL';
$tfreq[0] = 1;

my @tfirt;
open(TFIRT, "<:utf8", $ARGV[2]) || die "$!: $ARGV[2]\n";
while (<TFIRT>) {
    chomp;
    my ($tid, @firt) = split(/\s+/);
    for (my $i = 0; $i < @firt; $i++) {
	$firt[$i] = 1/(10**7) if ($firt[$i] == 0);
    }
    printf "%s\t%s\t%s\t%g\n", $ARGV[4], $tid[$tid], 'NULL', $firt[0] if ($opt->{fertility});
    $tfirt[$tid] = \@firt;
}
close TFIRT;

open(TABLE, "<:utf8", $ARGV[3]) || die "$!: $ARGV[3]\n";
my $old_tid = "";
my %tmp;
while (<TABLE>) {
    chomp;
    my ($tid, $sid, $prob) = split(/\s+/);
    if ($old_tid ne "" && $old_tid ne $tid) {
	foreach my $sid_tmp (keys %tmp) {
	    if ($opt->{fertility}) {
		printf "%s\t%s\t%s\t%g\n", $ARGV[4], $tid[$old_tid], $sid[$sid_tmp], $tmp{$sid_tmp}*(1-$tfirt[$old_tid]->[0]);
	    } else {
		printf "%s\t%s\t%s\t%g\n", $ARGV[4], $tid[$old_tid], $sid[$sid_tmp], $tmp{$sid_tmp};
	    }
	}
	%tmp = ();
    }
    $tmp{$sid} = $prob;
    $old_tid = $tid;
}
close TABLE;

foreach my $sid_tmp (keys %tmp) {
    if ($opt->{fertility}) {
	printf "%s\t%s\t%s\t%g\n", $ARGV[4], $tid[$old_tid], $sid[$sid_tmp], $tmp{$sid_tmp}*(1-$tfirt[$old_tid]->[0]);
    } else {
	printf "%s\t%s\t%s\t%g\n", $ARGV[4], $tid[$old_tid], $sid[$sid_tmp], $tmp{$sid_tmp};
    }
}

$ARGV[4] =~ /lex_(.)/;
printf "%s\t%s\t%s\t%d\n", 'vocab', 'count', $1, scalar(@sid)-1;
